dnl
dnl AC_VCHAT_GCC_WALL_CHECK
dnl

AC_DEFUN(AC_VCHAT_GCC_WALL_CHECK,
[
	# When running gcc, add "-Wall" to CFLAGS
	AC_MSG_CHECKING(whether to add '-Wall' to CFLAGS)
	if test x$GCC != x ; then
		CFLAGS="-Wall $CFLAGS"
		AC_MSG_RESULT(yes)
	else
		AC_MSG_RESULT(no)
	fi
])

dnl
dnl AC_VCHAT_OSTYPE_CHECK
dnl

AC_DEFUN(AC_VCHAT_OSTYPE_CHECK,
[
	AC_MSG_CHECKING(for appropriate OS_TYPE string)

	case $host_os in
		freebsd2*)	OSTP="BSD2"
				;;

		freebsd3*)	OSTP="BSD3"
				;;

		freebsd4*)	OSTP="BSD4"
				;;

		freebsd5*)	OSTP="BSD5"
				;;

		freebsd*)	OSTP="FBSD"
				;;

		netbsd*)	OSTP="NBSD"
				;;

		openbsd*)	OSTP="OBSD"
				;;

		linux*)		OSTP="LNXE"
				;;

		solaris2*)	OSTP="SOL2"
				;;

		bsdi*)		OSTP="BSDI"
				;;

		next*)		OSTP="NEXT"
				;;

		irix5*)		OSTP="IRX5"
				;;

		irix6*)		OSTP="IRX6"
				;;

		aix*)		OSTP="AIX-"
				;;

		sun*)		OSTP="SUNO"
				;;

		*)		OSTP="UKWN"
				;;
	esac

	AC_MSG_RESULT($OSTP)

	AC_DEFINE_UNQUOTED(OS_TYPE, "${OSTP}",The appropriate operating system type string.)
])

dnl
dnl AC_VCHAT_VERSION_CHECK
dnl

AC_DEFUN(AC_VCHAT_VERSION_CHECK,
[
	AC_MSG_CHECKING(for version and release date)

	vc_version=`head -1 ./version`
	vc_reldate=`head -2 ./version | tail -1`

	AC_DEFINE_UNQUOTED(VERSION, "$vc_version", Defined to the vchat client version.)
	AC_DEFINE_UNQUOTED(REL_DATE, "$vc_reldate", Defined to the release date of the client.)

	AC_MSG_RESULT($vc_version / $vc_reldate)
])

dnl
dnl AC_VCHAT_OS_INFO_CHK
dnl

AC_DEFUN(AC_VCHAT_OSINFO_CHECK,
[
	AC_MSG_CHECKING(how to get system information)

	AC_DEFINE_UNQUOTED(SI_UNAME, 1, Use uname() to get system information.)
	AC_DEFINE_UNQUOTED(SI_SYSINFO, 2, Use sysinfo() to get system information.)
	AC_DEFINE_UNQUOTED(SI_SYSCTL, 3, Use sysctl() to get system information.)

	case $host_os in
		freebsd2*)	SI_TYPE="SI_SYSCTL"
				;;

		freebsd3*)	SI_TYPE="SI_SYSCTL"
				;;

		freebsd4*)	SI_TYPE="SI_SYSCTL"
				;;

		freebsd5*)	SI_TYPE="SI_SYSCTL"
				;;

		freebsd*)	SI_TYPE="SI_UNAME"
				;;

		netbsd*)	SI_TYPE="SI_UNAME"
				;;

		openbsd*)	SI_TYPE="SI_UNAME"
				;;

		linux*)		SI_TYPE="SI_UNAME"
				;;

		solaris2*)	SI_TYPE="SI_SYSINFO"
				;;

		bsdi*)		SI_TYPE="SI_UNAME"
				;;

		next*)		SI_TYPE="SI_UNAME"
				;;

		irix5*)		SI_TYPE="SI_UNAME"
				;;

		irix6*)		SI_TYPE="SI_UNAME"
				;;

		aix*)		SI_TYPE="SI_UNAME"
				;;

		sun*)		SI_TYPE="SI_UNAME"
				;;

		*)		SI_TYPE="SI_UNAME"
				;;
	esac

	case $SI_TYPE in

		SI_UNAME)	SI_NAME="uname"
				;;
		SI_SYSINFO)	SI_NAME="sysinfo"
				;;
		SI_SYSCTL)	SI_NAME="sysctl"
				;;
	esac

	AC_MSG_RESULT($SI_NAME)

	AC_DEFINE_UNQUOTED(SI_TYPE, ${SI_TYPE},The method how to get system information.)
	AC_DEFINE_UNQUOTED(SI_NAME, "${SI_NAME}",The name of the method how to get system information.)
])

dnl
dnl AC_VCHAT_SUNLEN_CHECK
dnl

AC_DEFUN(AC_VCHAT_SUNLEN_CHECK,
[
	AC_MSG_CHECKING(wheter struct sockaddr_un has a sun_len field)

	AC_TRY_RUN(
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
int main(int argc, char *argv[])
{
	struct sockaddr_un su;
	su.sun_len = 23;
	return 0;
}
, SUNLEN="1" , SUNLEN="0" , SUNLEN="0")

	AC_DEFINE_UNQUOTED(SUN_LEN_FIELD, ${SUNLEN}, Wheter struct sockaddr_un has a sun_len field.)

	case ${SUNLEN} in
		0)	RESULT="no"
			;;
		*)	RESULT="yes"
			;;
	esac

	AC_MSG_RESULT(${RESULT})
])
