/* $Id: vctl_html.c,v 1.1 2001/04/09 18:30:43 mickey Exp $ */

#include <vchatctl.h>
#include <proto.h>
#include "../vserver/include/vctlproto.h"

#include <sys/socket.h>
#include <sys/un.h>

/*** Externals ***/

extern	char			*prog_name;
extern	char			*ctlsockname;
extern	int			ctlsock;
extern	struct	sockaddr_un	sa;
extern	struct	sockaddr_un	connsa;

/*** Defines ***/

#define ADDRESS "/tmp/vcontrol.%d"

#ifndef AF_LOCAL
  #define AF_LOCAL AF_UNIX
#endif

#ifndef PF_LOCAL
  #define PF_LOCAL PF_UNIX
#endif

#ifndef SUN_LEN
  #define SUN_LEN(x) (strlen((x)->sun_path) + sizeof((x)->sun_family))
#endif

/*** Code ***/

int op_html(void)
{
	char	*query;

	/*
	 * Get CGI query string
	 */

	if((query = (char *)getenv("QUERY_STRING")) == NULL)
	{
		printf("Content-type: text/html\n\n");
		printf("<HTML>\n<HEAD><TITLE>FAILURE</TITLE></HEAD>\n");
		printf("<BODY BGCOLOR=#ff0000 TEXT=#ffffff LINK=#ffff00>\n");
		printf("No QUERY_STRING environment present - check script call\n");
		printf("</BODY>\n</HTML>\n");

		return(-1);
	}

	/*
	 * Initialize control socket
	 */

	if(init_ctlsock() == -1)
	{
		printf("Content-type: text/html\n\n");
		printf("<HTML>\n<HEAD><TITLE>FAILURE</TITLE></HEAD>\n");
		printf("<BODY BGCOLOR=#ff0000 TEXT=#ffffff LINK=#ffff00>\n");
		printf("A failure has occured while trying to initialize the\n");
		printf("VChat control socket.\n");
		printf("</BODY>\n</HTML>\n");

		return(-1);
	}

	/*
	 * Determine operation mode from query string
	 */

	if(!strncmp(query, "kill=", 5))
	{
		/*
		 * Kill user mode
		 */

		printf("Content-type: text/html\n\n");
		printf("<HTML>\n<HEAD><TITLE>VChat control panel: kill user</TITLE></HEAD>\n");
		printf("<BODY BGCOLOR=#cc3300 TEXT=#ffffff LINK=#0000ff>\n");
		printf("<CENTER><H2>VChat Control Panel</H2></CENTER>\n");
		printf("<HR><P><DL><DL>\n");

		query += 5;

		html_kill(query);

		printf("</DL></DL>\n");
		printf("<HR><A HREF=\"vchatctl.cgi\">[Zur&uuml;ck zum VChat control panel]</A>\n");
		printf("</BODY>\n</HTML>\n");
	}
	else if(!strlen(query))
	{
		/*
		 * Ulist mode
		 */

		printf("Content-type: text/html\n\n");
		printf("<HTML>\n<HEAD>\n<TITLE>VChat control panel:</TITLE>\n");
		printf("<META HTTP-EQUIV=\"Refresh\" CONTENT=\"60\">\n</HEAD>\n");
		printf("<BODY BGCOLOR=#cc3300 TEXT=#ffffff LINK=#0000ff>\n");
		printf("<CENTER><H2>VChat Control Panel</H2></CENTER>\n");
		printf("<HR><P>\n");
		printf("Folgende Benutzer befinden sich augenblicklich im VChat (Klicken auf User entfernt diesen):\n");
		printf("<P><DL><DL>\n");

		html_ulist();

		printf("</DL></DL>\n</BODY>\n</HTML>\n");
	}
	else
	{
		/*
		 * Garbled query string
		 */

		printf("Content-type: text/html\n\n");
		printf("<HTML>\n<HEAD><TITLE>ERROR: invalid query string supplied.</TITLE></HEAD>\n");
		printf("<BODY BGCOLOR=#ff0000 TEXT=#ffffff LINK=#ffff00>\n");
		printf("The query string you have supplied is invalid.\n");
		printf("</BODY>\n</HTML>\n");
	}

	/*
	 * Remove control socket and exit
	 */

	unlink(sa.sun_path);

	return(0);
}

int html_ulist(void)
{
	VCTL			vctl;
	struct	sockaddr_un	fsa;

	int			fsalen	= sizeof(fsa);
	int			users	= 0;

	vctl.type = VCTL_ULIST;
	vctl.flags = 0;
	vctl.data[0] = '\0';
	vctl.len = VCTL_LEN(&vctl);

	if(SENDTO(ctlsock, &vctl, vctl.len, 0, &connsa, SUN_LEN(&connsa)) == -1)
	{
		printf("<P><BR><P><BR>ERROR sending control packet - aborted<P><BR><P><BR>\n");
		return(-1);
	}

	BZERO(&vctl, sizeof(VCTL));

	while(RECVFROM(ctlsock, &vctl, sizeof(VCTL), 0, &fsa, &fsalen) != -1)
	{
		switch(vctl.type)
		{
			case VCTL_USER:
			{
				char *s;

				if((s = (char *)strchr(vctl.data, ' ')) != NULL)
					*s = '\0';

				printf("<LI><A HREF=\"vchatctl.cgi?kill=%s\">%s</A><BR>\n",
					vctl.data, vctl.data);
				++users;
				break;
			}
			case VCTL_INFO:	printf("<P>%s<BR>\n", vctl.data);
					break;
			default:	printf("<P><BR>--unknown server response $%02x--<P><BR>\n", vctl.type);
					break;
		}

		if(vctl.flags == VCTL_MSGCT)
			break;

		BZERO(&vctl, sizeof(VCTL));
	}

	return(users);
}

void html_kill(char *user)
{
	VCTL			vctl;
	struct sockaddr_un	fsa;

	int			fsalen	= sizeof(fsa);

	vctl.type = VCTL_KILL;
	vctl.flags = 0;
	sprintf(vctl.data, "%s", user);
	vctl.len = VCTL_LEN(&vctl);

	if(SENDTO(ctlsock, &vctl, vctl.len, 0, &connsa, SUN_LEN(&connsa)) == -1)
	{
		printf("<P><BR><P><BR>ERROR sending control packet - aborted<P><BR><P><BR>\n");
		return;
	}

	BZERO(&fsa, sizeof(fsa));
	BZERO(&vctl, sizeof(VCTL));

	while(RECVFROM(ctlsock, &vctl, sizeof(VCTL), 0, &fsa, &fsalen) != -1)
	{
		switch(vctl.type)
		{
			case	VCTL_INFO:	printf("%s<BR>\n", vctl.data);
						break;
			default:		printf("<P><BR>--unknown server response $%02x--<P><BR>\n", vctl.type);
						break;
		}

		if(vctl.flags == VCTL_MSGCT)
			break;

		BZERO(&vctl, sizeof(VCTL));
		BZERO(&fsa, sizeof(fsa));
	}
}
