/* $Id: vctl_subr.c,v 1.1 2001/04/09 18:30:43 mickey Exp $ */

#include <vchatctl.h>
#include <proto.h>
#include "../vserver/include/vctlproto.h"

#include <pwd.h>
#include <sys/socket.h>
#include <sys/un.h>

/*** Defines ***/

#define	ADDRESS	"/tmp/vc.%ld"

#ifndef AF_LOCAL
  #define AF_LOCAL AF_UNIX
#endif

#ifndef PF_LOCAL
  #define PF_LOCAL PF_UNIX
#endif

#ifndef SUN_LEN
  #define SUN_LEN(x) (strlen((x)->sun_path) + sizeof((x)->sun_family))
#endif

/*** Externals ***/

extern	char			*prog_name;
extern	char			*connectsockname;
extern	int			ctlsock;
extern	struct sockaddr_un	sa;
extern	struct sockaddr_un	connsa;

/*** Code ***/

void kill(char *nick, int mode)
{
	VCTL	vctl;
	struct	sockaddr_un fsa;

	int	fsalen = sizeof(fsa);

	vctl.type = (mode == 1) ? VCTL_KILLSILENT : VCTL_KILL;
	vctl.flags = 0;
	sprintf(vctl.data, "%s", nick);
	vctl.len = VCTL_LEN(&vctl);

	if(SENDTO(ctlsock, &vctl, vctl.len, 0, &connsa, SUN_LEN(&connsa)) == -1)
	{
		fprintf(stderr, "%s: failed to send control message to VChat server process.\n",
			prog_name);
		return;
	}

	BZERO(&fsa, sizeof(fsa));
	BZERO(&vctl, sizeof(VCTL));

	while(RECVFROM(ctlsock, &vctl, sizeof(VCTL), 0, &fsa, &fsalen) != -1)
	{
		switch(vctl.type)
		{
			case VCTL_INFO:	printf("%s\n", vctl.data);
					break;
			default:	printf("Unknown response type $%02x received from server.\n", vctl.type);
					break;
		}

		if(vctl.flags == VCTL_MSGCT)
			break;

		BZERO(&vctl, sizeof(VCTL));
		BZERO(&fsa, sizeof(fsa));
	}
}

void ulist(void)
{
	VCTL	vctl;
	struct	sockaddr_un fsa;

	int	fsalen = sizeof(fsa);

	vctl.type = VCTL_ULIST;
	vctl.flags = 0;
	vctl.data[0] = '\0';
	vctl.len = VCTL_LEN(&vctl);

	if(SENDTO(ctlsock, &vctl, vctl.len, 0, &connsa, SUN_LEN(&connsa)) == -1)
	{
		fprintf(stderr, "%s: failed to send control message to VChat server process.\n",
			prog_name);
		return;
	}

	BZERO(&fsa, sizeof(fsa));
	BZERO(&vctl, sizeof(VCTL));

	while(RECVFROM(ctlsock, &vctl, sizeof(VCTL), 0, &fsa, &fsalen) != -1)
	{
		switch(vctl.type)
		{
			case VCTL_USER:
			case VCTL_INFO:	printf("%s\n", vctl.data);
					break;
			default:	printf("Unknown response type $%02x received from server.\n", vctl.type);
					break;
		}

		if(vctl.flags == VCTL_MSGCT)
			break;

		BZERO(&vctl, sizeof(VCTL));
		BZERO(&fsa, sizeof(fsa));
	}
}

void suppress(void)
{
	VCTL	vctl;
	struct	sockaddr_un fsa;

	int	fsalen = sizeof(fsa);

	vctl.type = VCTL_SUPPRESS;
	vctl.flags = 0;
	vctl.data[0] = '\0';
	vctl.len = VCTL_LEN(&vctl);

	if(SENDTO(ctlsock, &vctl, vctl.len, 0, &connsa, SUN_LEN(&connsa)) == -1)
	{
		fprintf(stderr, "%s: failed to send control message to VChat server process.\n",
			prog_name);
		return;
	}

	BZERO(&fsa, sizeof(fsa));
	BZERO(&vctl, sizeof(VCTL));

	while(RECVFROM(ctlsock, &vctl, sizeof(VCTL), 0, &fsa, &fsalen) != -1)
	{
		switch(vctl.type)
		{
			case VCTL_INFO:	printf("%s\n", vctl.data);
					break;
			default:	printf("Unknown response type $%02x received from server.\n", vctl.type);
					break;
		}

		if(vctl.flags == VCTL_MSGCT)
			break;

		BZERO(&vctl, sizeof(VCTL));
		BZERO(&fsa, sizeof(fsa));
	}
}

void wall(char *msg)
{
	VCTL			vctl;
	struct passwd		*pwe;
	struct sockaddr_un	fsa;

	char			*opname;
	char			*optty	= NULL;
	int			fsalen	= sizeof(fsa);
	uid_t			uid	= getuid();

	/*
	 * Get the users name from the password database
	 */

	if((pwe = getpwuid(uid)) == NULL)
	{

	}

	opname = pwe->pw_name;

	/*
	 * Get the users login terminal
	 */

	if(isatty(0))
		optty = ttyname(0);
	else if(isatty(1))
		optty = ttyname(1);
	else if(isatty(2))
		optty = ttyname(2);

	if(optty == NULL)
		optty = "<stdin>";

	/*
	 * Send the control message to the conference server
	 */

	vctl.type = VCTL_WALL;
	vctl.flags = 0;
	sprintf(vctl.data, "%s %s %s", opname, optty, msg);
	vctl.len = VCTL_LEN(&vctl);

	if(SENDTO(ctlsock, &vctl, vctl.len, 0, &connsa, SUN_LEN(&connsa)) == -1)
	{
		fprintf(stderr, "%s: failed to send control message to VChat server process.\n",
			prog_name);
		return;
	}

	BZERO(&fsa, sizeof(fsa));
	BZERO(&vctl, sizeof(VCTL));

	while(RECVFROM(ctlsock, &vctl, sizeof(VCTL), 0, &fsa, &fsalen) != -1)
	{
		switch(vctl.type)
		{
			case VCTL_INFO:	printf("%s\n", vctl.data);
					break;
			default:	printf("Unknown response type $%02x received from server.\n", vctl.type);
					break;
		}

		if(vctl.flags == VCTL_MSGCT)
			break;

		BZERO(&vctl, sizeof(VCTL));
		BZERO(&fsa, sizeof(fsa));
	}
}

void server_shutdown(void)
{
	VCTL	vctl;
	struct	sockaddr_un fsa;

	int	fsalen = sizeof(fsa);

	vctl.type = VCTL_SHUTDOWN;
	vctl.flags = 0;
	vctl.data[0] = '\0';
	vctl.len = VCTL_LEN(&vctl);

	if(SENDTO(ctlsock, &vctl, vctl.len, 0, &connsa, SUN_LEN(&connsa)) == -1)
	{
		fprintf(stderr, "%s: failed to send control message to VChat server process.\n",
			prog_name);
		return;
	}

	BZERO(&fsa, sizeof(fsa));
	BZERO(&vctl, sizeof(VCTL));

	while(RECVFROM(ctlsock, &vctl, sizeof(VCTL), 0, &fsa, &fsalen) != -1)
	{
		switch(vctl.type)
		{
			case VCTL_INFO:	printf("%s\n", vctl.data);
					break;
			default:	printf("Unknown response type $%02x received from server.\n", vctl.type);
					break;
		}

		if(vctl.flags == VCTL_MSGCT)
			break;

		BZERO(&vctl, sizeof(VCTL));
		BZERO(&fsa, sizeof(fsa));
	}	
}

int init_ctlsock(void)
{
	int s;

	if((s = socket(PF_LOCAL, SOCK_DGRAM, 0)) == -1)
	{
		fprintf(stderr, "%s: socket failure (%s)", prog_name,
			strerror(errno));
		return(-1);
	}

	sa.sun_family = AF_LOCAL;
	sprintf(sa.sun_path, ADDRESS, (u_long)getpid());
#ifdef HAVE_SUN_LEN
	sa.sun_len = SUN_LEN(&sa);
#endif
	connsa.sun_family = AF_LOCAL;
	sprintf(connsa.sun_path, "%s", connectsockname);
#ifdef HAVE_SUN_LEN
	connsa.sun_len = SUN_LEN(&connsa);
#endif
	unlink(sa.sun_path);

	if(bind(s, (struct sockaddr *) &sa, SUN_LEN(&sa)) < 0)
	{
		fprintf(stderr, "%s: bind failure (%s).\n", prog_name,
			strerror(errno));
		return(-1);
	}

	ctlsock = s;

	return(0);
}
