/* $Id: cmd_acl_add.c,v 1.3 2002/06/23 23:16:04 mickey Exp $ */

#include <vserver.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>

#include <limits.h>

#ifndef WIN32
  #include <arpa/inet.h>
#endif

/*
 * void cmd_acl_add(CLDATA *ptr, u_char *dat, size_t len);
 *
 * Process ACL commands
 */

void cmd_acl_add(CLDATA *ptr, u_char *dat, size_t len)
{
	VACL	rule;
	char	act;
	int	ruleidx;
	time_t	expire;

	/*
	 * Check if the client is permitted to use ACL commands
	 */

	if(!(ptr->accperm & AUTH_ACL))
	{
		snd_svmsg(ptr, "* You are not authorized to use ACL commands");
		return;
	}

	/*
	 * Check for correct argument length depending on client proto rev.
	 */

	if(ptr->protocol >= 0x00040011 && len == sizeof(VACL))
	{
		/*
		 * Protocol 4-11 and above use 16 byte VACL struct with TTL.
		 */

		BZERO(&rule, sizeof(VACL));
		BCOPY(dat, &rule, sizeof(VACL));
	}
	else if(ptr->protocol < 0x00040011 && len == 12)
	{
		/*
		 * Protocol 4-10 and below use 12 byte VACL without TTL.
		 */

		BZERO(&rule, sizeof(VACL));
		BCOPY(dat, &rule, 12);
	}
	else
	{	
		/*
		 * Wrong packet length.
		 */

		snd_svmsg(ptr, "* Argument supplied for ACL_ADD has invalid size (%d instead of %d)",
			len, ptr->protocol < 0x00040011 ? 12 : sizeof(VACL));

		return;
	}

	/*
	 * Check if rule number within range.
	 */

	if(ntohs(rule.rule) > 65534)
	{
		snd_svmsg(ptr, "* Supplied rule number out of range - use a value between 0 and 65534");
		return;
	}

	/*
	 * Check specified action.
	 */

	switch(ntohs(rule.action))
	{
		case ACL_ACT_ACCEPT:	act = 'a';
					break;
		case ACL_ACT_REJECT:	act = 'r';
					break;
		default:		snd_svmsg(ptr, "* Supplied action is invalid");
					return;
	}

	/*
	 * Check TTL.
	 */

	if(ntohl(rule.ttl) > DYNACL_MAX_TTL)
	{
		snd_svmsg(ptr, "* Invalid TTL specified; maximum=%d", DYNACL_MAX_TTL);
		return;
	}

	/*
	 * Calculate expiration time for this rule.
	 */

	if(ntohl(rule.ttl) > 0)
	{
		time(&expire);
		expire += ntohl(rule.ttl);
	}
	else
	{
		expire = 0;	// Never expires.
	}

	/*
	 * Add the ACL rule.
	 */

	if((ruleidx = acl_add_rule(ntohs(rule.rule), act, rule.address, rule.netmask, expire)) == -1)
	{
		snd_svmsg(ptr, "* Memory reallocation failure");
		return;
	}

	/*
	 * Display the rule that has just been added.
	 */

	{
		u_char	buf[256];
		ACLRULE *nr = RULE(ruleidx);

		acl_print_rule(nr, buf);

		snd_svmsg(ptr, "* %s", buf);
	}
}
