/* $Id: cmd_addnick.c,v 1.1 2001/04/09 18:30:43 mickey Exp $ */

#include <vserver.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>

/*
 * void addnick(CLDATA *ptr, u_char *dat);
 *
 * Add a new authority record to the authority database
 */

void cmd_addnick(CLDATA *ptr, u_char *dat)
{
	AUTHREC	*auth;
	u_char	*passwd;
	int	plen;

	/*
	 * Check if the client is permitted to execute the ADDNICK command
	 */

	if(!(ptr->accperm & AUTH_ADD))
	{
		snd_svmsg(ptr, "* You are not authorized to create new accounts");
		return;
	}

	/*
	 * Check if a valid password has been supplied
	 */

	if((passwd = (u_char *)strchr(dat, ' ')) == NULL)
	{
		snd_svmsg(ptr, "* Must supply password for new account");
		return;
	}

	*passwd++ = '\0';

	if(*passwd == '\0')
	{
		snd_svmsg(ptr, "* Empty passwords are not allowed");
		return;
	}

	plen = strlen(passwd);

	if(plen < MINPASSWDSIZE)
	{
		snd_svmsg(ptr, "* Please choose a password with at least %d characters", MINPASSWDSIZE);
		return;
	}

	if(plen > VPPASSWDSIZE)
	{
		snd_svmsg(ptr, "* Please choose a password with at most %d characters", VPPASSWDSIZE);
		return;
	}

	/*
	 * Check if a valid nickname was supplied
	 */

	if(*dat == '\0')
	{
		snd_svmsg(ptr, "* No nickname was specified");
		return;
	}

	if(!nick_ok(dat))
	{
		snd_svmsg(ptr, "* Nickname '%s' is invalid", dat);
		return;
	}

	/*
	 * Check if the specified nickname is already registered
	 */

	if((auth = auth_getbynick(dat)) != NULL)
	{
		snd_svmsg(ptr, "* Nickname '%s' is already registered in the authority database",
			auth->nick);
		return;
	}

	/*
	 * Check if the specified nickname is currently in use
	 */

	if(nick_dup(dat))
	{
		snd_svmsg(ptr, "* Nickname '%s' is currently in use", dat);
		return;
	}

	/*
	 * Add the new authority record to the database and sync database file
	 */

	if(auth_add_record(dat, passwd) == -1)
	{
		snd_svmsg(ptr, "* Failed to create new authority record - database unchanged");
		return;
	}

	if(auth_save() == -1)
	{
		(void) auth_delete_record(dat);
		snd_svmsg(ptr, "* Failed to update authority database file - database unchanged");
		return;
	}

	snd_svmsg(ptr, "* Successfully created new account for '%s'", dat);
}
