/* $Id: cmd_chaccperm.c,v 1.2 2002/06/23 23:16:05 mickey Exp $ */

#include <vserver.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>

/*
 * void chaccperm(CLDATA *ptr, u_char *dat, size_t len);
 *
 * Change the access permissions of registered nicknames
 */

void cmd_chaccperm(CLDATA *ptr, u_char *dat, size_t len)
{
	/*
	 * Check if the client is authorized to execute CHACCPERM command
	 */

	if(!(ptr->accperm & AUTH_CHACCPERM))
	{
		snd_svmsg(ptr, "* You are not authorized to change access permissions");
		return;
	}

	change_perm(ptr, 'p', dat, len);
}

void change_perm(CLDATA *ptr, u_char which, u_char *dat, size_t len)
{
	char			*xargv[VCMD_MAXARGS];
	char			**aptr;
	register AUTHREC	*auth;
	register CLDATA		*cl;
	register int		i;
	register int		y;
	u_int32_t		md;
	int			xargc		= 0;
	u_int32_t		givemask	= 0;
	u_int32_t		takemask	= 0;
	u_char			modified	= 0;
	u_char			*nicklist;
	u_char			*arg;

	/*
	 * Check for correct permission set specifier
	 */

	switch(which)
	{
		case 'p':
		case 'm':	break;
		default:	vlog(VLOG_ERR, "ERROR: unknown permission set specifier '%c' encountered", which);
				return;
	}

	/*
	 * Check for minimum argument size
	 */

	if(len < (2 * sizeof(u_int32_t)))
	{
		snd_svmsg(ptr, "* Supplied datasize is too short");
		return;
	}
	else if(len == (2 * sizeof(u_int32_t)))
	{
		snd_svmsg(ptr, "* No nickname(s) specified");
		return;
	}

	/*
	 * Get givemask and takemask from supplied data packet
	 * and set the pointer for the start of the nicknamelist.
	 */

	givemask = ntohl(*((u_int32_t *)(dat)));
	takemask = ntohl(*((u_int32_t *)(dat + sizeof(u_int32_t))));
	nicklist = dat + (2 * sizeof(u_int32_t));

	/*
	 * Parse nickname list into argument vector
	 */

#if HAVE_STRSEP
	for(aptr = xargv; (*aptr = (char *) strsep((char **)&nicklist, " \t")) != NULL && xargc < VCMD_MAXARGS;)
#else
	aptr = xargv;

	*aptr = (char *)strtok((char *)nicklist, " \t");

	while(*aptr != NULL && xargc < VCMD_MAXARGS)
#endif
	{
		y = 0;

		for(i = 0; i < xargc; i++)
		{
			if(!strcmp(xargv[i], *aptr))
			{
				y = 1;
				break;
			}
		}

		if(**aptr != '\0' && !y)
		{
			++aptr;
			++xargc;
		}

#ifndef HAVE_STRSEP
		*aptr = (char *)strtok(NULL, " \t");
#endif
	}

	/*
	 * Check if any nickname(s) given
	 */

	if(!xargc)
	{
		snd_svmsg(ptr, "* No nickname(s) specified");
		return;
	}

	/*
	 * Check for permissions compensating each other
	 */

	md = (givemask ^ takemask);
	givemask &= md;
	takemask &= md;

	/*
	 * Check if any mode change would take place
	 */

	if(givemask == 0 && takemask == 0)
	{
		snd_svmsg(ptr, "* Null permission change specified");
		return;
	}

	/*
	 * Check if the client is allowed to change the specified modes
	 */

	md = (~ptr->modperm & givemask);
	md |= (~ptr->modperm & takemask);

	if(md)
	{
		TABLE	cltab;

		table_init(&cltab, ptr);

		table_printf(&cltab, "* %sYou are not authorized to change the following %s permission(s):%s\n*%s",
			S_UL, (which == 'p') ? "access" : "modify", S_OFF, S_FG_RED);

		if(md & AUTH_ADD)
			table_printf(&cltab, " ADD");
		if(md & AUTH_DELETE)
			table_printf(&cltab, " DELETE");
		if(md & AUTH_CHACCPERM)
			table_printf(&cltab, " CHACCPERM");
		if(md & AUTH_CHMODPERM)
			table_printf(&cltab, " CHMODPERM");
		if(md & AUTH_KILL)
			table_printf(&cltab, " KILL");
		if(md & AUTH_BROADCAST)
			table_printf(&cltab, " BCAST");
		if(md & AUTH_ACL)
			table_printf(&cltab, " ACL");
		if(md & AUTH_QUERY)
			table_printf(&cltab, " QUERY");
		if(md & AUTH_SOPT)
			table_printf(&cltab, " SOPT");
		if(md & AUTH_FIXCHAN)
			table_printf(&cltab, " FIX");
		if(md & AUTH_STATUS)
			table_printf(&cltab, " STATUS");
		if(md & AUTH_EXCLUDE)
			table_printf(&cltab, " EXCLUDE");

		table_printf(&cltab, "%s", S_OFF);
		table_flush(&cltab);

		return;
	}

	/*
	 * Process nickname list
	 */

	for(i = 0; i < xargc; i++)
	{
		arg = xargv[i];

		/*
		 * Deny modifications to root's permissions
		 */

		if(!strcmp(arg, "root"))
		{
			snd_svmsg(ptr, "* You may not change root's %s permissions - ignored",
				(which == 'p') ? "access" : "modify");
			continue;
		}

		/*
		 * Deny modifications to own permissions
		 */

		if(!strcmp(arg, ptr->nick))
		{
			snd_svmsg(ptr, "* You may not modify your own %s permissions - ignored",
				(which == 'p') ? "access" : "modify");
			continue;
		}

		/*
		 * Get authority record for current nickname
		 */

		if((auth = auth_getbynick(arg)) == NULL)
		{
			snd_svmsg(ptr, "* No authority record for nickname '%s' found",
				arg);
			continue;
		}

		if(which == 'p')
		{
			md = auth->accperm;

			md |= givemask;
			md &= ~takemask;

			if(auth->accperm != md)
			{
				auth->accperm = md;
				modified = 1;

				/*
				 * Check if the nickname is currently signed on
				 * and change access permissions accordingly.
				 */

				for(y = 0; y < clients; y++)
				{
					cl = cl_addr(y);

					if(!strncmp(cl->nick, arg, VPNICKSIZE))
						cl->accperm = md;
				}
			}
		}
		else if(which == 'm')
		{
			md = auth->modperm;
			md |= givemask;
			md &= ~takemask;

			if(auth->modperm != md)
			{
				auth->modperm = md;
				modified = 1;

				/*
				 * Check if the nickname is currently signed on
				 * and change modify permissions accordingly.
				 */

				for(y = 0; y < clients; y++)
				{
					cl = cl_addr(y);

					if(!strncmp(cl->nick, arg, VPNICKSIZE))
						cl->modperm = md;
				}
			}
		}
		else
		{
			vlog(VLOG_ERR, "ERROR: Unknown permission set specifier '%c' encountered", which);
			return;
		}
	}

	/*
	 * Update authority database file
	 */

	if(modified)
	{
		if(auth_save() == 0)
			snd_svmsg(ptr, "* Successfully updated authority database file");
		else
			snd_svmsg(ptr, "* Failure updating authority database file - inform system operator");
	}
	else
	{
		snd_svmsg(ptr, "* No changes made");
	}
}
