/* $Id: cmd_chnick.c,v 1.1 2001/04/09 18:30:43 mickey Exp $ */

#include <vserver.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>

/*
 * void cmd_chnick(CLDATA *ptr, u_char *dat);
 *
 * Change to another nickname
 */

void cmd_chnick(CLDATA *ptr, u_char *dat)
{
	register int	i;
	register CLDATA	*pt;
	AUTHREC		*auth;
	u_char		*nick;
	u_char		*passwd;
	u_int32_t	oldflags	= 0;
	u_int32_t	old_opts	= 0;
	time_t		last_in		= 0;

	/*
	 * Split password from nickname
	 */

	nick = dat;

	if((passwd = (u_char *)strchr(nick, ' ')) == NULL)
		passwd = "";
	else
		*passwd++ = '\0';

	/*
	 * Check against current nickname.
	 */

	if(!strcmp(nick, ptr->nick))
	{
		snd_svmsg(ptr, "* Nickname unchanged");
		return;
	}

	/*
	 * Reject empty or too large nicknames
	 */

	if(*nick == '\0' || strlen(nick) > VPNICKSIZE)
	{
		snd_svmsg(ptr, "* Nickname '%s' has invalid size", nick);
		return;
	}

	/*
	 * Reject nicknames that contain invalid characters
	 */

	if(!nick_ok(nick))
	{
		snd_svmsg(ptr, "* Nickname '%s' is invalid", nick);
		return;
	}

	/*
	 * Check if there is an authority record assigned
	 * to the specified nickname
	 */

	if((auth = auth_getbynick(nick)) != NULL)
	{
		/*
		 * Yes - Check if the supplied password matches
		 */

		if(*passwd == '\0')
		{
			snd_svmsg(ptr, "* Permission denied");
			return;
		}

		if(strncmp(auth->passwd, passwd, VPPASSWDSIZE))
		{
			snd_svmsg(ptr, "* Permission denied");
			return;
		}

		/*
		 * Switch to new access and modify permissions
		 */

		ptr->accperm = auth->accperm;
		ptr->modperm = auth->modperm;

		/*
		 * Get saved client options from authority record.
		 */

		old_opts = ptr->cl_opts;
		ptr->cl_opts = auth->cl_opts;

		/*
		 * Save old client flags value for later use
		 * and set the client registered flag.
		 */

		oldflags = ptr->flags;
		ptr->flags |= CFLAG_REGISTERED;

		/*
		 * Save time of last signon to display it later and
		 * update the authority record with the current time.
		 */

		last_in = auth->last_signon;
		time(&auth->last_signon);
		auth->last_signoff = 0xffffffff;
		++auth->sessioncnt;
		auth_save();
	}
	else
	{
		/*
		 * No authority record found. If the server is in closed
		 * mode then reject any unregistered nicknames.
		 */

		if(cfdata.flags & CF_REG_ONLY)
		{
			snd_svmsg(ptr, "* Nickname '%s' is not registered", nick);
			return;
		}

		/*
		 * Check for duplicated nickname
		 */

		if(nick_dup(nick))
		{
			snd_svmsg(ptr, "* Nickname '%s' is already in use", nick);
			return;
		}

		/*
		 * Unauthorized nickname gets no access or modify permissions
		 */

		ptr->accperm = ptr->modperm = 0;

		/*
		 * Reset client options to default.
		 */

		old_opts = ptr->cl_opts;
		ptr->cl_opts = DEF_CLOPTMASK;

		/*
		 * Clear client registered flag.
		 */

		oldflags = ptr->flags;
		ptr->flags &= ~CFLAG_REGISTERED;
	}

	/*
	 * Update authority record when changing from a registered nickname.
	 */

	if(oldflags & CFLAG_REGISTERED)
	{
		if((auth = auth_getbynick(ptr->nick)) == NULL)
		{
			vlog(VLOG_NOTICE, "Oops: Registered nickname without database record ?");
		}
		else
		{
			time(&auth->last_signoff);

			auth->pubmsgcnt += ptr->pubmsgcnt;
			auth->prvmsgcnt	+= ptr->prvmsgcnt;

			auth->cl_opts = old_opts;

			auth_save();
		}
	}

	/*
	 * Make an entry in the last log file if
	 * the lastlog feature is enabled.
	 */

	if(cfdata.llog_entries > 0)
		lastlog_addentry(ptr, "nickname changed");

	/*
	 * Make a new login and activity timestamp
	 * in the client data and reset all counters.
	 */

	time(&ptr->login);
	ptr->stamp = ptr->login;

	ptr->pubmsgcnt = 0;
	ptr->prvmsgcnt = 0;

	/*
	 * Acknowledge the new nickname
	 */

	snd2client(ptr, NICK_ACK, NULL, 0);	

	/*
	 * Notify clients of name change.
	 */

	for(i = 0; i < clients; i++)
	{
		pt = cl_addr(i);

		/*
		 * Client with a protocol revision of 4-10 and above
		 * receive ULIST update messages to keep their userlist
		 * in sync with the server.
		 */

		if(MAJOR_REV(pt->protocol) >= 0x4 && MINOR_REV(pt->protocol) >= 0x10)
		{
			snd2client(pt, ULIST_REMOVE, ptr->nick, strlen(ptr->nick));
			snd2client(pt, ULIST_ADD, nick, strlen(nick));
		}

		if(pt->channel == ptr->channel)
		{
			snd_svmsg(pt, "* %s%s%s is now known as %s%s%s",
				S_FG_CYN, ptr->nick, S_OFF,
				S_FG_YEL, nick, S_OFF);
		}
	}

	vlog(VLOG_INFO, "Rename client USER: %s OLD-NICK: %s NEW-NICK: %s",
		ptr->user, ptr->nick, nick);

	/*
	 * Actually change the nickname
	 */

	strncpy(ptr->nick, nick, VPNICKSIZE);

	/*
	 * If the last signon time is known, display it now.
	 */

	if(last_in)
	{
		u_char	tstr[80];

		strftime(tstr, 80, "%a %b %d %H:%M:%S %Z %Y", localtime(&last_in));
		snd_svmsg(ptr, "* Last login %s", tstr);
	}
}
