/* $Id: cmd_chpasswd.c,v 1.1 2001/04/09 18:30:44 mickey Exp $ */

#include <vserver.h>
#include <ascii.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>

/*
 * void chpasswd(CLDATA *ptr, u_char *dat);
 *
 * Change the password of the calling user.
 */

void cmd_chpasswd(CLDATA *ptr, u_char *dat)
{
	AUTHREC	*auth;
	u_char	*newpass;
	int	plen;
	u_char	ignore	= 0;

	u_char	curpwd[VPPASSWDSIZE+1];

	/*
	 * Find the space separator character in the supplied argument
	 * ignoring any characters escaped by a CSI character.
	 */

	newpass = dat;

	for(;;)
	{
		if(newpass == NULL)
		{
			break;
		}
		else if(*newpass == CSI)
		{
			ignore = (ignore) ? 0 : 1;
		}
		else if(*newpass == ' ')
		{
			if(!ignore)
				break;
			else
				ignore = 0;
		}
		else if(*newpass == '\0')
		{
			break;
		}
		else
		{
			ignore = 0;
		}

		++newpass;
	}

	if(newpass == NULL || dat == NULL || *newpass == '\0')
	{
		snd_svmsg(ptr, "* Invalid arguments for CHPASSWD operation");
		vlog(VLOG_WARNING, "WARNING: Client sent invalid CHPASSWD data USER: %s NICK: %s HOST: %s",
			ptr->user, ptr->nick, ptr->host);
		return;
	}

	*newpass++ = '\0';

	/*
	 * Unescape the current password string
	 */

	unescape_str(dat, curpwd, VPPASSWDSIZE+1);

	/*
	 * Check for empty passwords
	 */

	if(curpwd[0] == '\0' || *newpass == '\0')
	{
		snd_svmsg(ptr, "* Empty passwords are not allowed");
		return;
	}

	/*
	 * Get authority record
	 */

	if((auth = auth_getbynick(ptr->nick)) == NULL)
	{
		/*
		 * No authority record found in database
		 */

		snd_svmsg(ptr, "%s\n%s",
			"* Your nickname is currently not registered",
			"* Only registered users may change their password");
		return;
	}

	/*
	 * Check the supplied current password against database
	 */

	if(strncmp(auth->passwd, curpwd, VPPASSWDSIZE))
	{
		snd_svmsg(ptr, "* Permission denied");
		return;
	}

	plen = strlen(newpass);

	if(plen < MINPASSWDSIZE)
	{
		snd_svmsg(ptr, "* Please choose a password with at least %d characters", MINPASSWDSIZE);
		return;
	}

	if(plen > VPPASSWDSIZE)
	{
		snd_svmsg(ptr, "* Please choose a password with at most %d characters", VPPASSWDSIZE);
		return;
	}

	if(!strncmp(auth->passwd, newpass, VPPASSWDSIZE))
	{
		snd_svmsg(ptr, "* Please choose a password different from your old one");
		return;
	}

	/*
	 * Set the new password and sync database file
	 */

	BZERO(auth->passwd, VPPASSWDSIZE+1);
	strncpy(auth->passwd, newpass, VPPASSWDSIZE);

	if(auth_save() == -1)
	{
		BZERO(auth->passwd, VPPASSWDSIZE+1);
		strncpy(auth->passwd, curpwd, VPPASSWDSIZE);
		snd_svmsg(ptr, "* Failed to update authority database file - password unchanged");
	}
	else
	{
		snd_svmsg(ptr, "* Password successfully changed");
	}
}
