/* $Id: cmd_clientinfo.c,v 1.4 2002/06/24 03:04:21 mickey Exp $ */

#include <vserver.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>

/*
 * void cmd_clientinfo(CLDATA *ptr, u_char *dat);
 *
 * Report information about a client
 */

void cmd_clientinfo(CLDATA *ptr, u_char *dat)
{
	register int	i;
	register int	y;
	register int	n;
	register CLDATA	*cl;
	TABLE		cltab;
	char		buf1[96];
	char		buf2[96];
	char		*xargv[VCMD_MAXARGS];
	char		**aptr;
	int		cnt	= 0;
	int		xargc	= 0;

	/*
	 * Check if registered
	 */

	if(!(ptr->flags & CFLAG_REGISTERED) && ptr->accperm != AUTH_ROOT)
	{
		snd_svmsg(ptr, "* Only registered users may use the CLIENTINFO command");
		return;
	}

	table_init(&cltab, ptr);

	/*
	 * Parse nicknames
	 */

#if HAVE_STRSEP
	for(aptr = xargv; (*aptr = (char *) strsep((char **)&dat, " \t")) != NULL && xargc < VCMD_MAXARGS;)
#else
	aptr = xargv;

	*aptr = (char *)strtok((char *)dat, " \t");

	while(*aptr != NULL && xargc < VCMD_MAXARGS)
#endif
	{
		/*
		 * Check for invalid nicknames.
		 */

		y = 0;

		if(!nick_ok(*aptr))
		{
			table_xprintf(&cltab, "\n* Invalid nickname '%s%s%s' ignored",
				S_FG_RED, *aptr, S_OFF);
			y = 1;
		}
		else
		{
			/*
			 * Check for duplicated nicknames.
			 */

			for(i = 0; i < xargc; i++)
			{
				if(!strcmp(xargv[i], *aptr))
				{
					table_xprintf(&cltab, "\n* Duplicated nickname '%s%s%s' ignored",
						S_FG_RED, *aptr, S_OFF);

					y = 1;
					break;
				}
			}
		}

		if(**aptr != '\0' && !y)
		{
			++aptr;
			++xargc;
		}

#ifndef HAVE_STRSEP
		*aptr = (char *)strtok(NULL, " \t");
#endif
	}

	if(!xargc)
	{
		table_xprintf(&cltab, "\n* No nickname(s) to show");
		table_flush(&cltab);
		return;
	}

	/*
	 * Lookup client entries
	 */

	for(y = 0; y < xargc; y++)
	{
		for(i = n = 0; i < clients; i++)
		{
			cl = cl_addr(i);

			if(!strcmp(cl->nick, xargv[y]))
			{
				/*
				 * Print header along with first listed entry.
				 */

				if(!cnt)
				{
					table_printf(&cltab, "\n*%s%s                                %s CLIENTINFO %s                                 %s",
						S_BG_BLU, S_FG_WHT, S_BG_RED, S_BG_BLU, S_OFF);
				}

				if(cnt > 0)
					table_printf(&cltab, "\n*");

				/*
				 * Line 1: username, nickname, channel
				 */

				if(cl->type == CLIENT_GATEWAY || !strcmp(cl->user, cl->nick))
					sprintf(buf1, "%s", cl->nick);
				else
					sprintf(buf1, "%s (%s)", cl->user, cl->nick);

				if(cl->channel >= 0 || cl->channel == ptr->channel)
					sprintf(buf2, " on channel %ld", cl->channel);
				else
					buf2[0] = '\0';

				table_printf(&cltab, "\n*     user: %s%s, %s",
						buf1, buf2,
						(cl->flags & CFLAG_REGISTERED)?
						"registered" : "unregistered");

				/*
				 * Line 2: messages total, public, private.
				 */

				{
					time_t	now;
					time_t	online;

					time(&now);
					online = now - cl->login;

					if(online <= 0)
						online = 1;

					sprintf(buf1, "%.4f", ((double)(cl->pubmsgcnt + cl->prvmsgcnt) / online));
				}

				table_printf(&cltab, "\n* messages: %d total, %d public, %d private, avg. %s msgs/sec",
					(cl->pubmsgcnt + cl->prvmsgcnt),
					cl->pubmsgcnt, cl->prvmsgcnt, buf1);

				/*
				 * Line 3: login@
				 */

				strftime(buf1, sizeof(buf1), "%a, %d %b %Y %H:%M:%S %z", localtime(&cl->login));

				table_printf(&cltab, "\n*   login@: %s", buf1);

				/*
				 * Line 4: time online, idle.
				 */

				sprintf(buf1, "%s", pftime(cl->login));
				sprintf(buf2, "%s", pftime(cl->stamp));

				table_printf(&cltab, "\n*     time: %s online, %s idle",
					buf1, buf2);

				/*
				 * Line 5: client ident, type, proto, conn-id,
				 *         dnslvl.
				 */

				table_printf(&cltab, "\n*   client: %s, type %s, proto %x-%x, conn 0x%X, dnslvl %d",
					cl->ident,
					(cl->type == CLIENT_FRONTEND) ?
					"FRONTEND" : "GATEWAY",
					MAJOR_REV(cl->protocol),
					MINOR_REV(cl->protocol),
					cl->fd, cl->dnschklvl);

				/*
				 * Line 6: host ip-address, hostname.
				 */

				if(!(cfdata.flags & CF_SUPPRESS_IP))
				{
					table_printf(&cltab, "\n*     from: %s (%s)",
						iptostr(cl->ip_addr), cl->host);
				}
				else
				{
					table_printf(&cltab, "\n*     from: -concealed-");
				}

				/*
				 * Line 7: gateway ip-address, hostname.
				 */

				if(cl->type == CLIENT_GATEWAY)
				{
					table_printf(&cltab, "\n*      via: %s (%s)",
						iptostr(cl->gw_addr),
						cl->gw_name);
				}

				++n;
				++cnt;
			}
		}

		if(!n)
		{
			table_xprintf(&cltab, "\n* Nickname '%s%s%s' not found",
				S_FG_RED, xargv[y], S_OFF);
		}
	}

	/*
	 * Flush output table.
	 */

	table_flush(&cltab);
}
