/* $Id: cmd_clopt.c,v 1.2 2002/06/23 23:16:05 mickey Exp $ */

#include <vserver.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>

/*
 * void cmd_clopt(CLDATA *ptr, void *dat);
 *
 * Get/Set client options.
 */

void cmd_clopt(CLDATA *ptr, u_char *dat, size_t len)
{
	TABLE		tab;
	GSOPT		*opt;
	u_int32_t	givemask;
	u_int32_t	takemask;
	u_int32_t	md;

	/*
	 * Check for correct argument length.
	 */

	if(len != sizeof(GSOPT))
	{
		snd_svmsg(ptr, "* Invalid packet length encountered");
		return;
	}

	/*
	 * Retrieve arguments
	 */

	opt = (GSOPT *)dat;

	givemask = ntohl(opt->givemask);
	takemask = ntohl(opt->takemask);
	md = 0;

	/*
	 * Check for give/take options that will compensate each other
	 */

	md = (givemask ^ takemask);
	givemask &= md;
	takemask &= md;

	/*
	 * Modify client options (unknown options are silently ignored)
	 */

	if(givemask & CL_OPT_WAKEUP)
		ptr->cl_opts |= CL_OPT_WAKEUP;
	if(givemask & CL_OPT_SGNMSG)
		ptr->cl_opts |= CL_OPT_SGNMSG;
	if(givemask & CL_OPT_PRVECHO)
		ptr->cl_opts |= CL_OPT_PRVECHO;

	if(takemask & CL_OPT_WAKEUP)
		ptr->cl_opts &= ~CL_OPT_WAKEUP;
	if(takemask & CL_OPT_SGNMSG)
		ptr->cl_opts &= ~CL_OPT_SGNMSG;
	if(takemask & CL_OPT_PRVECHO)
		ptr->cl_opts &= ~CL_OPT_PRVECHO;

	/*
	 * Print the current option set up
	 */

	table_init(&tab, ptr);

	table_printf(&tab, "* Client options: ");

	display_clopt(&tab, ptr->cl_opts);

	table_flush(&tab);
}

void display_clopt(TABLE *cltab, u_int32_t opts)
{
	if(opts & CL_OPT_WAKEUP)
		table_printf(cltab, "%s+%sWAKEUP", S_FG_GRN, S_OFF);
	else
		table_printf(cltab, "%s-%sWAKEUP", S_FG_RED, S_OFF);

	if(opts & CL_OPT_SGNMSG)
		table_printf(cltab, " %s+%sSGNMSG", S_FG_GRN, S_OFF);
	else
		table_printf(cltab, " %s-%sSGNMSG", S_FG_RED, S_OFF);

	if(opts & CL_OPT_PRVECHO)
		table_printf(cltab, " %s+%sPRVECHO", S_FG_GRN, S_OFF);
	else
		table_printf(cltab, " %s-%sPRVECHO", S_FG_RED, S_OFF);

}
