/* $Id: cmd_delnick.c,v 1.2 2002/06/23 23:16:05 mickey Exp $ */

#include <vserver.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>

/*
 * void delnick(CLDATA *ptr, u_char *dat);
 *
 * Delete the authority record for a given nickname
 * from the authority database.
 */

void cmd_delnick(CLDATA *ptr, u_char *dat)
{
	register int	i;
	register int	y;
	AUTHREC		*auth;
	TABLE		cltab;
	char		*xargv[VCMD_MAXARGS];
	char		**aptr;
	int		xargc		= 0;
	u_char		modified	= 0;

	/*
	 * Check if the client is permitted to execute the DELNICK command
	 */

	if(!(ptr->accperm & AUTH_DELETE))
	{
		snd_svmsg(ptr, "* You are not authorized to delete accounts");
		return;
	}

	/*
	 * Initialize output table.
	 */

	table_init(&cltab, ptr);

	/*
	 * Parse nicknames
	 */

#if HAVE_STRSEP
	for(aptr = xargv; (*aptr = (char *) strsep((char **)&dat, " \t")) != NULL && xargc < VCMD_MAXARGS;)
#else
	aptr = xargv;

	*aptr = (char *)strtok((char *)dat, " \t");

	while(*aptr != NULL && xargc < VCMD_MAXARGS)
#endif
	{
		/*
		 * Check for invalid nicknames.
		 */

		y = 0;

		if(!strcmp(*aptr, "root"))
		{
			table_xprintf(&cltab, "\n* You may not delete the 'root' entry - this incident will be reported");
			y = 1;
		}
		else if(!nick_ok(*aptr))
		{
			table_xprintf(&cltab, "\n* Invalid nickname '%s%s%s' ignored",
				S_FG_RED, *aptr, S_OFF);
			y = 1;
		}

		for(i = 0; i < xargc; i++)
		{
			if(!strcmp(xargv[i], *aptr))
			{
				table_xprintf(&cltab, "\n* Duplicated nickname '%s%s%s' ignored",
					S_FG_RED, *aptr, S_OFF);

				y = 1;
				break;
			}
		}

		if(**aptr != '\0' && !y)
		{
			++aptr;
			++xargc;
		}

#ifndef HAVE_STRSEP
		*aptr = (char *)strtok(NULL, " \t");
#endif
	}

	/*
	 * Check if a valid nickname has been specified
	 */

	if(!xargc)
	{
		table_xprintf(&cltab, "\n* No nickname(s) to delete");
		table_flush(&cltab);
		return;
	}

	/*
	 * Process nickname list
	 */

	for(i = 0; i < xargc; i++)
	{
		/*
		 * Check if the nickname is registered
		 */

		if((auth = auth_getbynick(xargv[i])) == NULL)
		{
			table_xprintf(&cltab, "\n* Nickname '%s%s%s' not registered",
				S_FG_RED, xargv[i], S_OFF);
			continue;
		}

		/*
		 * Check if the nickname is currently in use
		 */

		if(nick_dup(xargv[i]))
		{
			table_xprintf(&cltab, "\n* Nickname '%s%s%s' is currently in use",
				S_FG_RED, xargv[i], S_OFF);
			continue;
		}

		/*
		 * Delete the specified nickname from the authority database
		 */

		if(auth_delete_record(xargv[i]) == -1)
		{
			table_xprintf(&cltab, "\n* Failed to delete authority record for '%s%s%s'",
				S_FG_RED, xargv[i], S_OFF);
			table_flush(&cltab);
			return;
		}

		modified = 1;

		table_xprintf(&cltab, "\n* Nickname '%s%s%s' deleted",
			S_FG_YEL, xargv[i], S_OFF);
	}

	/*
	 * Sync authority database file if modifications where made
	 */

	if(modified)
	{
		if(auth_save() == 0)
			table_xprintf(&cltab, "\n* Successfully updated authority database file");
		else
			table_xprintf(&cltab, "\n* Failure updating authority database file - inform system operator");
	}

	/*
	 * Flush output table.
	 */

	table_flush(&cltab);
}
