/* $Id: cmd_exclude.c,v 1.1 2002/06/23 23:16:54 mickey Exp $ */

#include <vserver.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>

/*
 * void cmd_exclude(CLDATA *ptr, void *dat);
 *
 * Exclude users from chat by adding a dynamic ACL first and then kicking
 * them from the server.
 */

void cmd_exclude(CLDATA *ptr, u_char *dat, size_t len)
{
	register int	i;
	register int	y;
	register int	n;
	register CLDATA	*cl;
	TABLE		cltab;
	char		*xargv[VCMD_MAXARGS];
	time_t		xttlv[VCMD_MAXARGS];
	time_t		ttl;
	char		**aptr;
	char		*s;
	int		xargc	= 0;
	struct in_addr	mask;
	time_t		now;

	/*
	 * Check if permitted to use the exclude command.
	 */

	if(!(ptr->accperm & AUTH_EXCLUDE))
	{
		snd_svmsg(ptr, "* You are not authorized to use the EXCLUDE command");
		return;
	}

	/*
	 * Initialize output table.
	 */

	table_init(&cltab, ptr);

	/*
	 * Parse arguments into vector.
	 */

#if HAVE_STRSEP
	for(aptr = xargv; (*aptr = (char *) strsep((char **)&dat, " \t")) != NULL && xargc < VCMD_MAXARGS;)
#else
	aptr = xargv;

	*aptr = (char *)strtok((char *)dat, " \t");

	while(*aptr != NULL && xargc < VCMD_MAXARGS)
#endif
	{
		/*
		 * Check for optional TTL.
		 */

		if((s = (char *)strchr(*aptr, ':')) != NULL)
		{
			/*
			 * Parse TTL.
			 */

			*s++ = '\0';
			ttl = strtol(s, NULL, 0);
		}
		else
		{
			/*
			 * No TTL given, use default TTL.
			 */

			ttl = DYNACL_DEFAULT_TTL;
		}

		/*
		 * Check for invalid nicknames.
		 */

		if(!nick_ok(*aptr))
		{
			/*
			 * Nickname invalid.
			 */

			table_xprintf(&cltab, "\n* Invalid nickname '%s%s%s' ignored",
				S_FG_RED, *aptr, S_OFF);
			y = 1;
		}
		else
		{
			/*
			 * Check for duplicated nicknames.
			 */

			y = 0;

			for(i = 0; i < xargc; i++)
			{
				if(!strcmp(xargv[i], *aptr))
				{
					table_xprintf(&cltab, "\n* Duplicated nickname '%s%s%s' ignored",
						S_FG_RED, *aptr, S_OFF);

					y = 1;
					break;
				}
			}
		}

		if(**aptr != '\0' && !y)
		{
			++aptr;
			xttlv[xargc++] = ttl;
		}

#ifndef HAVE_STRSEP
		*aptr = (char *)strtok(NULL, " \t");
#endif
	}

	if(!xargc)
	{
		table_xprintf(&cltab, "\n* No nickname(s) to exclude");
		table_flush(&cltab);
		return;
	}

	/*
	 * Get current time for calculation of rule expiration time
	 * and set netmask used.
	 */

	time(&now);
	mask.s_addr = htonl(0xffffffff);

	/*
	 * Lookup client entries
	 */

	for(y = 0; y < xargc; y++)
	{
		/*
		 * Check if TTL specification is valid.
		 */

		if(xttlv[y] > DYNACL_MAX_TTL)
		{
			table_xprintf(&cltab, "\n* TTL for nickname '%s%s%s' (%d) invalid, using max TTL (%d)",
				S_FG_RED, xargv[y], S_OFF, xttlv[y],
				DYNACL_MAX_TTL);

			xttlv[y] = DYNACL_MAX_TTL;
		}

		/*
		 * Lookup matching clients.
		 */

		for(i = 0, n = 0; i < clients; i++)
		{
			cl = cl_addr(i);

			if(!strcmp(cl->nick, xargv[y]))
			{
				int ridx;

				/*
				 * Mark the client to be killed.
				 */

				cl->flags |= CFLAG_EXCLUDE;
				BZERO(cl->killinfo, VPNICKSIZE + 1);
				strncpy(cl->killinfo, ptr->nick, VPNICKSIZE);

				/*
				 * Create a new dynamic ACL that excludes
				 * the user's IP address for the given TTL.
				 */

				if(dynacl_next >= 65534)
					dynacl_next = DYNACL_FIRST_RULE;

				if((ridx = acl_add_rule(dynacl_next++, 'r', cl->ip_addr, mask, now + xttlv[y])) == -1)
				{
					table_xprintf(&cltab, "\n* ACL: %smemory allocation failure%s",
						S_FG_RED, S_OFF);
					return;
				}
				else
				{
					u_char msgbuf[256];
					ACLRULE *rl = RULE(ridx);

					acl_print_rule(rl, msgbuf);

					table_xprintf(&cltab, "\n* Excluded '%s%s%s' by ACL %s",
						S_FG_YEL, xargv[y], S_OFF, msgbuf);
				}

				++n;
			}
		}

		if(!n)
		{
			table_xprintf(&cltab, "\n* Nickname '%s%s%s' not found",
				S_FG_RED, xargv[y], S_OFF);
		}
	}

	/*
	 * Flush output table.
	 */

	table_flush(&cltab);
}
