/* $Id: cmd_fixchan.c,v 1.1 2001/04/09 18:30:44 mickey Exp $ */

#include <vserver.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>

/*
 * void cmd_fixchan(CLDATA *ptr, void *dat, size_t len);
 *
 * Delete/Create fixed conference channels
 */

void cmd_fixchan(CLDATA *ptr, u_char *dat, size_t len)
{
	/*
	 * Sanity checks
	 */

	if(dat == NULL)
		return;
	/*
	 * Check access permission
	 */

	if(!(ptr->accperm & AUTH_FIXCHAN))
	{
		snd_svmsg(ptr, "* You are not authorized to administrate fixed channels");
		return;
	}

	/*
	 * Determine action from length of argument
	 */

	if(len >= sizeof(int32_t))
	{
		int32_t			ch	= ntohl(*(int32_t *)dat);
		int			tidx	= -1;
		struct chantopic	*ct	= (struct chantopic *)0;

		/*
		 * Check if the specified channel already exists
		 * as a non-fixed conference channel.
		 */

		if((tidx = topic_idx(ch)) != -1)
		{
			ct =  TOPADDR(tidx);

			if(ct->flags & CHFLG_REMOVABLE)
			{
				snd_svmsg(ptr, "* Channel %s%d%s already exists as non-fixed channel",
					S_FG_BLU, ch, S_OFF);
				return;
			}
		}

		if(len > sizeof(int32_t))
		{
			/*
			 * Create/Update a fixed channel
			 */

			char	*top	= (char *)(dat + sizeof(int32_t));

			if(strlen(top) > VPTOPICSIZE)
				*(top + VPTOPICSIZE) = '\0';

			if(store_topic(ch, top, CHFLG_FIXED))
			{
				snd_svmsg(ptr, "* Failed to create/update fixed channel %s%d%s",
					S_FG_BLU, ch, S_OFF);
				return;
			}

			touch_channel(ch);

			if(tidx == -1)
			{
				snd_svmsg(ptr, "* Created fixed channel %s%d%s with topic '%s'",
					S_FG_BLU, ch, S_OFF, top);
			}
			else
			{
				register int		i;
				register CLDATA		*cp;

				/*
				 * Inform users on this channel
				 * anout the topic change.
				 */

				for(i = 0; i < clients; i++)
				{
					cp = cl_addr(i);

					if(cp->channel == ch)
					{
						snd_svmsg(cp, "* Topic changed to '%s%s%s' by %s%s%s",
							S_BOLD, top, S_OFF, S_FG_YEL, ptr->nick, S_OFF);
					}
				}

				snd_svmsg(ptr, "* Updated fixed channel %s%d%s with topic '%s'",
					S_FG_BLU, ch, S_OFF, top);
			}
		}
		else
		{
			/*
			 * Remove a fixed channel
			 */

			int ret;

			if(tidx == -1)
			{
				snd_svmsg(ptr, "* Channel %s%d%s not currently active",
					S_FG_BLU, ch, S_OFF);
				return;
			}

			/*
			 * If there are still users on the specified
			 * channel, then convert the channel to a removable
			 * type.
			 */

			if(chatters(ch) > 0)
			{
				ct->flags &= ~CHFLG_FIXED;
				ct->flags |= CHFLG_REMOVABLE;

				snd_svmsg(ptr, "* Converted channel %s%d%s to a removable type channel",
					S_FG_BLU, ch, S_OFF);
				return;
			}

			/*
			 * Else go ahead and delete it from the topic database
			 */

			switch((ret = remove_topic(ch, CHFLG_FIXED)))
			{
				case 0:		snd_svmsg(ptr, "* Removed fixed channel %s%d%s",
							S_FG_BLU, ch, S_OFF);
						break;
				case -1:	snd_svmsg(ptr, "* Channel %s%d%s not currently active",
							S_FG_BLU, ch, S_OFF);
						break;
				case -2:	snd_svmsg(ptr, "* Channel %s%d%s is not of fixed type",
							S_FG_BLU, ch, S_OFF);
						break;
				default:	snd_svmsg(ptr, "* Undefined error %d", ret);
						break;
				
			}
		}
	}
	else
	{
		snd_svmsg(ptr, "* Malformed FIXCHAN request");
	}
}
