/* $Id: cmd_invite.c,v 1.2 2002/06/23 23:16:05 mickey Exp $ */

#include <vserver.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>

/*
 * void cmd_invite(CLDATA *ptr, u_char *dat);
 *
 * Invite user to join a channel
 */

void cmd_invite(CLDATA *ptr, u_char *dat)
{
	register int	i;
	register int	y;
	register CLDATA	*cl;
	TABLE		cltab;
	char		*xargv[VCMD_MAXARGS];
	char		**aptr;
	u_char		ok;
	u_char		n	= 0;
	int		xargc	= 0;

	/*
	 * Initialize output table.
	 */

	table_init(&cltab, ptr);

	/*
	 * Parse nickname list into argument vector.
	 */

#if HAVE_STRSEP
	for(aptr = xargv; (*aptr = (char *) strsep((char **)&dat, " \t")) != NULL && xargc < VCMD_MAXARGS;)
#else
	aptr = xargv;

	*aptr = (char *)strtok((char *)dat, " \t");

	while(*aptr != NULL && xargc < VCMD_MAXARGS)
#endif
	{
		/*
		 * Check for invalid nicknames.
		 */

		y = 0;

		if(!nick_ok(*aptr))
		{
			table_xprintf(&cltab, "\n* Invalid nickname '%s%s%s' ignored",
				S_FG_RED, *aptr, S_OFF);

			y = 1;
		}
		else
		{
			/*
			 * Check for duplicated nicknames.
			 */

			for(i = 0; i < xargc; i++)
			{
				if(!strcmp(xargv[i], *aptr))
				{
					table_xprintf(&cltab, "\n* Duplicated nickname '%s%s%s' ignored",
						S_FG_RED, *aptr, S_OFF);

					y = 1;
					break;
				}
			}
		}

		if(**aptr != '\0' && !y)
		{
			++aptr;
			++xargc;
		}

#ifndef HAVE_STRSEP
		*aptr = (char *)strtok(NULL, " \t");
#endif
	}

	if(!xargc)
	{
		table_xprintf(&cltab, "\n* No nickname(s) to invite");
		table_flush(&cltab);
		return;
	}

	/*
	 * Lookup client entries
	 */

	for(y = 0; y < xargc; y++)
	{
		ok = 0;

		for(i = 0; i < clients; i++)
		{
			cl = cl_addr(i);

			if(!strcmp(cl->nick, xargv[y]))
			{
				if(cl->channel != ptr->channel)
				{
					snd_svmsg(cl, "* %s%s%s invites you to join channel %s%d%s",
						S_FG_YEL, ptr->nick, S_OFF,
						S_FG_BLU, ptr->channel, S_OFF);
					ok = 1;
				}
				else
				{
					if(!strcmp(cl->nick, ptr->nick))
						table_xprintf(&cltab, "\n* No need to invite yourself");
					else
						table_xprintf(&cltab, "\n* No need to invite %s%s%s - (s)he is already on your channel",
							S_FG_RED, cl->nick, S_OFF);
					ok = 2;
					break;
				}
			}
		}

		switch(ok)
		{
			case 0:		table_xprintf(&cltab, "\n* Nickname '%s%s%s' not found",
						S_FG_RED, xargv[y], S_OFF);
					break;
			case 1:		++n;
			case 2:		break;
		}
	}

	/*
	 * Report status
	 */

	if(n > 1)
	{
		table_xprintf(&cltab, "\n* %s%d%s users have been invited",
			S_FG_MAG, n, S_OFF);
	}
	else if(n == 1)
	{
		table_xprintf(&cltab, "\n* %s1%s user has been invited",
			S_FG_MAG, S_OFF);
	}

	/*
	 * Flush output table.
	 */

	table_flush(&cltab);
}
