/* $Id: cmd_kill.c,v 1.2 2002/06/23 23:16:05 mickey Exp $ */

#include <vserver.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>

/*
 * void cmd_kill(CLDATA *ptr, u_char *dat);
 *
 * Enforce signoff of the specified users.
 */

void cmd_kill(CLDATA *ptr, u_char *dat)
{
	register int	i;
	register int	y;
	register CLDATA	*cl;
	TABLE		cltab;
	char		*xargv[VCMD_MAXARGS];
	char		**aptr;
	int		xargc	= 0;
	int		found	= 0;

	/*
	 * Check if the client is permitted to execute KILL command
	 */

	if(!(ptr->accperm & AUTH_KILL))
	{
		snd_svmsg(ptr, "* You are not authorized to kill others");
		return;
	}

	/*
	 * Initialize output table.
	 */

	table_init(&cltab, ptr);

	/*
	 * Parse nicknamelist into argument vector
	 */

#if HAVE_STRSEP
	for(aptr = xargv; (*aptr = (char *) strsep((char **)&dat, " \t")) != NULL && xargc < VCMD_MAXARGS;)
#else

	aptr = xargv;

	*aptr = (char *)strtok((char *)dat, " \t");

	while(*aptr != NULL && xargc < VCMD_MAXARGS)
#endif
	{
		/*
		 * Check if the nickname is valid
		 */

		y = 0;

		if(!strcmp(*aptr, "root"))
		{
			table_xprintf(&cltab, "\n* The 'root' user may not be killed - this incident will be reported");
			y = 1;
		}
		else if(!nick_ok(*aptr))
		{
			table_xprintf(&cltab, "\n* Invalid nickname '%s%s%s' ignored",
				S_FG_RED, *aptr, S_OFF);
			y = 1;
		}
		else
		{
			/*
			 * Check for duplicated nicknames.
			 */

			for(i = 0; i < xargc; i++)
			{
				if(!strcmp(xargv[i], *aptr))
				{

					table_xprintf(&cltab, "\n* Duplicated nickname '%s%s%s' ignored",
						S_FG_RED, *aptr, S_OFF);

					y = 1;
					break;
				}
			}
		}

		if(**aptr != '\0' && !y)
		{
			++aptr;
			++xargc;
		}

#ifndef HAVE_STRSEP
		*aptr = (char *)strtok(NULL, " \t");
#endif
	}

	/*
	 * Check if any arguments remain
	 */

	if(!xargc)
	{
		table_xprintf(&cltab, "\n* No nickname(s) to kill");
		table_flush(&cltab);
		return;
	}

	/*
	 * Kill all users matching the specified nicknames
	 */

	for(i = 0; i < xargc; i++)
	{
		/*
		 * Lookup all clients matching the given nickname
		 */

		found = 0;

		for(y = (clients - 1); y >= 0; y--)
		{
			cl = cl_addr(y);

			if(!strncmp(cl->nick, xargv[i], VPNICKSIZE) && cl->accperm != AUTH_ROOT)
			{
				/*
				 * Found one - Kill it!
				 */

				cl->flags |= CFLAG_KILL;
				BZERO(cl->killinfo, VPNICKSIZE+1);
				strncpy(cl->killinfo, ptr->nick, VPNICKSIZE);
				++found;
			}
		}

		if(!found)
		{
			table_xprintf(&cltab, "\n* Nickname '%s%s%s' not found",
				S_FG_RED, xargv[i], S_OFF);
		}
		else if(found == 1)
		{
			table_xprintf(&cltab, "\n* Marked '%s%s%s' for removal",
				S_FG_YEL, xargv[i], S_OFF);
		}
		else
		{
			table_xprintf(&cltab, "\n* Marked %d instances of '%s%s%s' for removal",
				found, S_FG_YEL, xargv[i], S_OFF);
		}
	}

	/*
	 * Flush output table.
	 */

	table_flush(&cltab);
}
