/* $Id: cmd_lastlog.c,v 1.1 2001/04/09 18:30:44 mickey Exp $ */

#include <vserver.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>

/*
 * void cmd_lastlog(CLDATA *ptr, u_char *dat);
 *
 * Show list of users that last logged on.
 */

void cmd_lastlog(CLDATA *ptr, u_char *dat)
{
	LLOGDAT		*ldat;
	char		buf1[64];
	char		buf2[64];
	char		**aptr;
	char		*xargv[VCMD_MAXARGS];
	int		xargc = 0;
	int		count = 0;
	u_int32_t	num		= ntohl(*(u_int32_t *)dat);
	char		*nicklist	= (char *)(dat + sizeof(u_int32_t));
	char		display;
	register int	v;
	register int	i;
	register int	q;

	/*
	 * Check if registered user or root
	 */

	if(!(ptr->flags & CFLAG_REGISTERED) && ptr->accperm != AUTH_ROOT)
	{
		snd_svmsg(ptr, "* Only registered users may use the LASTLOG command");
		return;
	}

	/*
	 * Apply maximum list entries per request
	 */

	if(num > LASTLOG_MAX_LIST)
		num = LASTLOG_MAX_LIST;

	/*
	 * Check if lastlog is enabled
	 */

	if(llog_base == NULL || llog_header == NULL || cfdata.llog_entries <= 0)
	{
		snd_svmsg(ptr, "* Lastlog feature is disabled");
		return;
	}

	/*
	 * Parse nickname list
	 */

#if HAVE_STRSEP
	for(aptr = xargv; (*aptr = (char *) strsep((char **)&nicklist, " \t")) !=NULL && xargc < VCMD_MAXARGS;)
	{
		v = 0;

		for(i = 0; i < xargc; i++)
		{
			if(!strcmp(xargv[i], *aptr))
			{
				v = 1;
				break;
			}
		}

		if(**aptr != '\0' && !v)
		{
			++aptr;
			++xargc;
		}
	}
#else
	aptr = xargv;

	*aptr = (char *)strtok((char *)nicklist, " \t");

	while(*aptr != NULL && xargc < VCMD_MAXARGS)
	{
		v = 0;

		for(i = 0; i < xargc; i++)
		{
			if(!strcmp(xargv[i], *aptr))
			{
				v = 1;
				break;
			}
		}

		if(!v)
		{
			++aptr;
			++xargc;
		}

		*aptr = (char *)strtok(NULL, " \t");
	}
#endif
	/*
	 * Lookup matching lastlog entries
	 */

	v = llog_header->next_record;

	snd_svmsg(ptr, "\n*%s%s%s NICKNAME        FROM                     LOGIN@         OUT   TIME          %s",
		S_BG_BLU, S_FG_WHT, S_BOLD, S_OFF);

	for(i = 0; (i < llog_header->used_records) && (count < num); i++)
	{
		if(v == 0)
			v = llog_header->total_records;

		--v;

		ldat = LLOG_RECORD(v);

		display = 0;

		if(!(display = (!xargc) ? 1 : 0))
		{
			for(q = 0; q < xargc; q++)
			{
				if(!strcmp(ldat->nickname, xargv[q]))
				{
					display = 1;
					break;
				}
			}
		}

		if(display)
		{
			char	htbuf[64];

			strftime(buf1, 64, "%b %d %H:%M", localtime(&ldat->signon_stamp));
			strftime(buf2, 64, "%H:%M", localtime(&ldat->signoff_stamp));

			if(strlen(ldat->hostname) > 24)
				sprintf(htbuf, "%s", iptostr(ldat->ipaddr));
			else
				sprintf(htbuf, "%s", ldat->hostname);

			snd_svmsg(ptr, "* %-15s %-24s %s - %s %s",
				ldat->nickname,
				htbuf,
				buf1, buf2,
				pf2time(ldat->signon_stamp, ldat->signoff_stamp));

			++count;
		}
	}

	/*
	 * Determine the login time of the first record in the database
	 */

	ldat = LLOG_RECORD(llog_header->first_record);

	strftime(buf1, 64, "%a %b %d %H:%M:%S %Z %Y",
		localtime(&ldat->signon_stamp));

	snd_svmsg(ptr, "* Lastlog begins: %s%s%s",
		S_BOLD, buf1, S_OFF);
}
