/* $Id: cmd_page.c,v 1.1 2001/04/09 18:30:44 mickey Exp $ */

#include <vserver.h>
#include <proto.h>
#include <extern.h>

#include <errno.h>

#ifndef WIN32
  #include <arpa/inet.h>
#endif

/*
 * void cmd_page(CLDATA *ptr, u_char *dat);
 *
 * Send a chat request to a user on a remote system via e-mail
 */

void cmd_page(CLDATA *ptr, u_char *dat)
{
#ifndef WIN32
	FILE *pipe;
	FILE *mf;
	char comline[255];
	char line[255];
	char *p;

	/*
	 * Page command is only permitted if OPFL_ENABLE_MAIL is set.
	 */

	if(!(cfdata.flags & CF_ENABLE_MAIL))
	{
#endif
		snd_svmsg(ptr, "* Sorry, page command is disabled");
		return;
#ifndef WIN32
	}

	/*
	 * Construct mail commandline
	 */

	sprintf(comline, "mail -s \"VChat invitation.\" %s 1>&2 2>/dev/null", dat);

	/*
	 * Execute mail request
	 */

	if((pipe = popen(comline, "w")))
	{
		fprintf(pipe, "                             VChat invitation\n");
		fprintf(pipe, "                             ================\n\n");

		if(ptr->type == CLIENT_GATEWAY)
		{
			fprintf(pipe, " %s from %s [%s] would like to chat to you on\n",
				ptr->nick, ptr->host, inet_ntoa(ptr->ip_addr));
			fprintf(pipe, " the conference server at %s [%s], channel %ld.\n\n",
				ptr->if_name, inet_ntoa(ptr->if_addr), ptr->channel);
		}
		else
		{
			fprintf(pipe, " %s@%s [%s] would like to chat to you on\n",
				ptr->user, ptr->host, inet_ntoa(ptr->ip_addr));
			fprintf(pipe, " the conference server at %s [%s], channel %ld.\n\n",
				ptr->if_name, inet_ntoa(ptr->if_addr), ptr->channel);
		}

		if(cfdata.mail_file[0] != '\0')
		{
			if((mf = fopen(cfdata.mail_file, "r")) != NULL)
			{
				while(fgets(line, 255, mf) != NULL)
				{
					while((p = (char *)strrchr(line, '\n')) != NULL)
						*p = '\0';

					while((p = (char *)strrchr(line, '\r')) != NULL)
						*p = '\0';

					fprintf(pipe, "%s\n", line);
				}

				fclose(mf);
			}
			else
			{
				vlog(VLOG_ERR, "ERROR: Failed to open mail invitation file '%s' (%s) USER: %s NICK: %s TO: %s",
					cfdata.mail_file, strerror(errno), ptr->user, ptr->nick, dat);
			}
		}

		if(pclose(pipe) && errno != EINTR)
		{
			snd_svmsg(ptr, "* Mail failed.");
			vlog(VLOG_ERR, "ERROR: Mail command returned status %d (%s) USER: %s NICK: %s TO: %s",
				errno, strerror(errno), ptr->user, ptr->nick, dat);
		}
		else
		{
			snd_svmsg(ptr, "* Page sent.");
			vlog(VLOG_INFO, "Page request sent USER: %s NICK: %s TO: %s",
				ptr->user, ptr->nick, dat);
		}
	}
	else
	{
		snd_svmsg(ptr, "* Sorry, mail is out of order.");
		vlog(VLOG_ERR, "ERROR: Attempt to open pipe to mail command returned status %d (%s) USER: %s NICK: %s TO: %s",
			errno, strerror(errno), ptr->user, ptr->nick, dat);
	}
#endif /* !WIN32 */
}
