/* $Id: cmd_query.c,v 1.2 2002/06/23 23:16:05 mickey Exp $ */

#include <vserver.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>
#include <auth.h>

/*
 * void cmd_query(CLDATA *ptr, u_char *dat);
 *
 * Query information about registered nicknames
 */

void cmd_query(CLDATA *ptr, u_char *dat)
{
	register int		i;
	register int		y;
	register AUTHREC	*auth;
	TABLE			cltab;
	char			buf[128];
	char			*xargv[VCMD_MAXARGS];
	char			**aptr;
	int			xargc	= 0;

        /*
         * Check if the client is permitted to query user data
         */

        if(!(ptr->accperm & AUTH_QUERY))
        {
                snd_svmsg(ptr, "* You are not authorized to query user data");
                return;
        }

	/*
	 * Initialize output table.
	 */

	table_init(&cltab, ptr);

	/*
	 * Default to own nickname if none specified.
	 */

	if(dat == NULL || *dat == '\0')
		dat = ptr->nick;

	/*
	 * Parse nicknames into argument vector.
	 */

#if HAVE_STRSEP
	for(aptr = xargv; (*aptr = (char *) strsep((char **)&dat, " \t")) != NULL && xargc < VCMD_MAXARGS;)
#else
	aptr = xargv;

	*aptr = (char *)strtok((char *)dat, " \t");

	while(*aptr != NULL && xargc < VCMD_MAXARGS)
#endif
	{
		/*
		 * Check for invalid nicknames.
		 */

		y = 0;

		if(!nick_ok(*aptr))
		{
			table_xprintf(&cltab, "\n* Invalid nickname '%s%s%s' ignored",
				S_FG_RED, *aptr, S_OFF);
			y = 1;
		}
		else
		{
			/*
			 * Check for duplicated nicknames.
			 */

			for(i = 0; i < xargc; i++)
			{
				if(!strcmp(xargv[i], *aptr))
				{
					table_xprintf(&cltab, "\n* Duplicated nickname '%s%s%s' ignored",
						S_FG_RED, *aptr, S_OFF);

					y = 1;
					break;
				}
			}
		}

		if(**aptr != '\0' && !y)
		{
			++aptr;
			++xargc;
		}

#ifndef HAVE_STRSEP
		*aptr = (char *)strtok(NULL, " \t");
#endif
	}

	if(!xargc)
	{
		table_xprintf(&cltab, "\n* No nickname(s) to query");
		table_flush(&cltab);
		return;
	}

	/*
	 * Lookup authority record for the given nicknames
	 */

	for(i = 0; i < xargc; i++)
	{
		if((auth = auth_getbynick(xargv[i])) == NULL)
		{
			table_xprintf(&cltab, "\n* Nickname '%s%s%s' not registered",
				S_FG_RED, xargv[i], S_OFF);

			continue;
		}

		/*
		 * Display data from authority record
		 */

		if(i)
			table_printf(&cltab, "\n");

		sprintf(buf, " %s:", xargv[i]);

		table_printf(&cltab, "\n*%s%s%s%s%s%s%*s%s\n", S_BG_BLU, S_FG_WHT,
			S_BOLD, buf, S_OFF, S_BG_BLU, (77 - strlen(buf)), " ", S_OFF);

		/*
		 * Display the time when the account was created, the time
		 * of last login and logout as well as the duration of session.
		 */

		table_printf(&cltab, "*          Sessions: %d\n", auth->sessioncnt);

		table_printf(&cltab, "*          Messages: %d public / %d private\n",
			auth->pubmsgcnt, auth->prvmsgcnt);

		if(auth->last_signon)
			strftime(buf, 128, "%a %b %d %H:%M:%S %Z %Y", localtime(&auth->last_signon));
		else
			sprintf(buf, "<never logged in>");

		table_printf(&cltab, "*        Last login: %s\n", buf);

		if(auth->last_signoff == 0xffffffff)
		{
			time_t	now;

			time(&now);
				
			table_printf(&cltab, "*       Last logout: <still logged in>  (%s)\n",
				pf2time(auth->last_signon, now));
		}
		else if(auth->last_signoff)
		{
			strftime(buf, 128, "%a %b %d %H:%M:%S %Z %Y",
				localtime(&auth->last_signoff));

			table_printf(&cltab, "*       Last logout: %s  (%s)\n",
				buf, pf2time(auth->last_signon, auth->last_signoff));
		}
		else
		{
			sprintf(buf, "<never logged in>");
			table_printf(&cltab, "*       Last logout: %s\n", buf);
		}

		strftime(buf, 128, "%a %b %d %H:%M:%S %Z %Y", localtime(&auth->account_created));
		table_printf(&cltab, "*   Account created: %s\n", buf);

		/*
		 * Display access privileges
		 */

		table_printf(&cltab, "* Access permission: ",
			S_UL, S_OFF);

		display_permission1(&cltab, auth->accperm);

		table_printf(&cltab, "\n*                    ");

		display_permission2(&cltab, auth->accperm);

		if((auth->accperm & AUTH_CHACCPERM)
		|| (auth->accperm & AUTH_CHMODPERM))
		{
			table_printf(&cltab, "\n* Modify permission: ",
				S_UL, S_OFF);

			display_permission1(&cltab, auth->modperm);

			table_printf(&cltab, "\n*                    ");

			display_permission2(&cltab, auth->modperm);
		}

	}

	table_flush(&cltab);
}

void display_permission1(TABLE *tab, u_long perm)
{
	if(perm & AUTH_ADD)
		table_printf(tab, "%s+%sADD", S_FG_GRN, S_OFF);
	else
		table_printf(tab, "%s-%sADD", S_FG_RED, S_OFF);	

	if(perm & AUTH_DELETE)
		table_printf(tab, " %s+%sDELETE", S_FG_GRN, S_OFF);
	else
		table_printf(tab, " %s-%sDELETE", S_FG_RED, S_OFF);

	if(perm & AUTH_CHACCPERM)
		table_printf(tab, " %s+%sCHACC", S_FG_GRN, S_OFF);
	else
		table_printf(tab, " %s-%sCHACC", S_FG_RED, S_OFF);

	if(perm & AUTH_CHMODPERM)
		table_printf(tab, " %s+%sCHMOD", S_FG_GRN, S_OFF);
	else
		table_printf(tab, " %s-%sCHMOD", S_FG_RED, S_OFF);

	if(perm & AUTH_KILL)
		table_printf(tab, " %s+%sKILL", S_FG_GRN, S_OFF);
	else
		table_printf(tab, " %s-%sKILL", S_FG_RED, S_OFF);

	if(perm & AUTH_BROADCAST)
		table_printf(tab, " %s+%sBCAST", S_FG_GRN, S_OFF);
	else
		table_printf(tab, " %s-%sBCAST", S_FG_RED, S_OFF);

	if((perm & AUTH_ACL) == AUTH_ACL)
		table_printf(tab, " %s+%sACL", S_FG_GRN, S_OFF);
	else
		table_printf(tab, " %s-%sACL", S_FG_RED, S_OFF);

	if((perm & AUTH_QUERY) == AUTH_QUERY)
		table_printf(tab, " %s+%sQUERY", S_FG_GRN, S_OFF);
	else
		table_printf(tab, " %s-%sQUERY", S_FG_RED, S_OFF);

	if((perm & AUTH_SOPT) == AUTH_SOPT)
		table_printf(tab, " %s+%sSOPT", S_FG_GRN, S_OFF);
	else
		table_printf(tab, " %s-%sSOPT", S_FG_RED, S_OFF);
}

void display_permission2(TABLE *tab, u_long perm)
{
	if(perm & AUTH_FIXCHAN)
		table_printf(tab, "%s+%sFIX", S_FG_GRN, S_OFF);
	else
		table_printf(tab, "%s-%sFIX", S_FG_RED, S_OFF);	

	if(perm & AUTH_STATUS)
		table_printf(tab, " %s+%sSTATUS", S_FG_GRN, S_OFF);
	else
		table_printf(tab, " %s-%sSTATUS", S_FG_RED, S_OFF);

	if(perm & AUTH_EXCLUDE)
		table_printf(tab, " %s+%sEXCLUDE", S_FG_GRN, S_OFF);
	else
		table_printf(tab, " %s-%sEXCLUDE", S_FG_RED, S_OFF);
}
