/* $Id: cmd_setpasswd.c,v 1.1 2001/04/09 18:30:44 mickey Exp $ */

#include <vserver.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>

/*
 * void setpasswd(CLDATA *ptr, u_char *dat);
 *
 * Set the password of the specified user.
 */

void cmd_setpasswd(CLDATA *ptr, u_char *dat)
{
	AUTHREC	*auth;
	u_char	*pass;
	int	plen;

	/*
	 * Only the superuser may change arbitrary passwords
	 */

	if(strcmp(ptr->nick, "root") || ptr->accperm != AUTH_ROOT)
	{
		snd_svmsg(ptr, "* Only the super-user may change arbitrary passwords");
		return;
	}

	/*
	 * Check if nickname and password were supplied
	 */

	if((pass = (char *)strchr(dat, ' ')) == NULL)
	{
		snd_svmsg(ptr, "* Invalid arguments for SETPASSWD operation");
		vlog(VLOG_WARNING, "WARNING: Client sent invalid SETPASSWD data USER: %s NICK: %s HOST: %s",
			ptr->user, ptr->nick, ptr->host);
		return;
	}

	*pass++ = '\0';

	/*
	 * May not change own password this way - use CHPASSWD instead
	 */

	if(!strcmp(dat, ptr->nick))
	{
		snd_svmsg(ptr, "* You may not change your own password using SETPASSWD");
		return;
	}

	/*
	 * Get authority record
	 */

	if((auth = auth_getbynick(dat)) == NULL)
	{
		/*
		 * No authority record found in database
		 */

		snd_svmsg(ptr, "* Nickname '%s' is currently not registered",
			dat);

		return;
	}

	/*
	 * Check the supplied password
	 */

	if(*pass == '\0')
	{
		snd_svmsg(ptr, "* Empty passwords are not allowed");
		return;
	}

	plen = strlen(pass);

	if(plen < MINPASSWDSIZE)
	{
		snd_svmsg(ptr, "* Please choose a password with at least %d characters", MINPASSWDSIZE);
		return;
	}

	if(plen > VPPASSWDSIZE)
	{
		snd_svmsg(ptr, "* Please choose a password with at most %d characters", VPPASSWDSIZE);
		return;
	}

	/*
	 * Set the new password and sync database file
	 */

	strncpy(auth->passwd, pass, VPPASSWDSIZE);

	if(auth_save() == -1)
	{
		snd_svmsg(ptr, "* Failed to update authority database file - inform operator");
	}
	else
	{
		snd_svmsg(ptr, "* Password for '%s' successfully changed", dat);
	}
}
