/* $Id: cmd_signoff.c,v 1.3 2002/06/24 03:04:21 mickey Exp $ */

#include <vserver.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>

/*
 * Sigoff user from the conference.
 */

void cmd_signoff(CLDATA *ptr, u_char *reason, u_char *errmsg)
{
	register CLDATA	*cl;
	register int	i;
	u_char		sgnoffmsg[256];

	/*
	 * Truncate messages that are too long.
	 */

	if(reason && strlen(reason) > 60)
		*(reason + 59) = '\0';
	if(errmsg && strlen(errmsg) > 60)
		*(errmsg + 59) = '\0';

	/*
	 * Fill in the signoff message
	 */

	if(reason && !errmsg)
	{
		sprintf(sgnoffmsg, "* Signoff %s%s%s [%s]",
			S_FG_YEL, ptr->nick, S_OFF, reason);
	}
	else if(errmsg)
	{
		sprintf(sgnoffmsg, "* Signoff %s%s%s (%s%s%s)",
			S_FG_YEL, ptr->nick, S_OFF, S_FG_RED, errmsg, S_OFF);
	}
	else
	{
		sprintf(sgnoffmsg, "* Signoff %s%s%s", S_FG_YEL, ptr->nick,
			S_OFF);
	}

	/*
	 * Inform others about the
	 * user who is signing off
	 */

	for(i=0; i<clients; i++)
	{
		cl = cl_addr(i);

		/*
		 * Clients with protocol revision of 4-10 and above
		 * receive ulist update messages to keep their userlist
		 * in sync with the server.
		 */

		if(MAJOR_REV(cl->protocol) >= 0x4 && MINOR_REV(cl->protocol) >= 0x10)
		{
			snd2client(cl, ULIST_REMOVE, ptr->nick, strlen(ptr->nick));
		}

		if(cl != ptr && (cl->cl_opts & CL_OPT_SGNMSG))
			snd_svmsg(cl, "%s", sgnoffmsg);
		else if(cl != ptr && cl->channel == ptr->channel)
			snd_svmsg(cl, "* %s%s%s signed off",
				S_FG_YEL, ptr->nick, S_OFF);
	}

	vlog(VLOG_INFO, "Signoff USER: %s NICK: %s HOST: %s REASON: %s ERRMSG: %s",
		ptr->user, ptr->nick, ptr->host,
		(reason) ? (char *)reason : "(null)",
		(errmsg) ? (char *)errmsg : "(null)");
	/*
	 * If nobody else is on the channel,
	 * the channeltopic expires by signing
	 * off this user.
	 */

	if(chatters(ptr->channel) == 1)
		remove_topic(ptr->channel, CHFLG_REMOVABLE);

	/*
	 * For registered nicknames, update the signoff
	 * timestamp and public- and private message counters.
	 */

	if(ptr->flags & CFLAG_REGISTERED)
	{
		AUTHREC		*auth;

		if((auth = auth_getbynick(ptr->nick)) == NULL)
		{
			/*
			 * Oops - registered client without database record?
			 */

			vlog(VLOG_NOTICE, "OOps: Registered nickname without database record?");
		}
		else
		{
			time(&auth->last_signoff);

			auth->pubmsgcnt += ptr->pubmsgcnt;
			auth->prvmsgcnt += ptr->prvmsgcnt;

			auth->cl_opts = ptr->cl_opts;

			auth_save();
		}
	}

	/*
	 * When the lastlog feature is enabled,
	 * log an entry to the user last login file.
	 */

	if(cfdata.llog_entries > 0)
		lastlog_addentry(ptr, (errmsg) ? errmsg : reason);

	/*
	 * Finally go on and delete
	 * the client from the table.
	 */

	delete_client(ptr->fd);
}
