/* $Id: cmd_signon.c,v 1.2 2002/06/23 23:16:06 mickey Exp $ */

#include <vserver.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>

/*
 * void cmd_signon(CLDATA *ptr);
 *
 * Signon a user to the conference
 */

void cmd_signon(CLDATA *ptr, time_t last_in)
{
	register CLDATA	*cl;
	register int	i;
	int		px;
	u_char		sgnonmsg[256];
	signed long	chan = htonl(ptr->channel);

	/*
	 * If the client that signs on has a protocol revision
	 * of 4-10 or above, then feed it with an initial username list.
	 */

	if(MAJOR_REV(ptr->protocol) >= 0x4 && MINOR_REV(ptr->protocol) >= 0x10)
	{
		snd2client(ptr, ULIST_CLEAR, NULL, 0);

		for(i = 0; i < clients; i++)
		{
			cl = cl_addr(i);

			snd2client(ptr, ULIST_ADD, cl->nick, strlen(cl->nick));
		}
	}

	/*
	 * Send a ulist update to all clients that have a protocol
	 * revision 4-10 or above.
	 */

	for(i = 0; i < clients; i++)
	{
		cl = cl_addr(i);

		if(cl != ptr && MAJOR_REV(cl->protocol) >= 0x4
		&& MINOR_REV(cl->protocol) >= 0x10)
		{
			snd2client(cl, ULIST_ADD, ptr->nick, strlen(ptr->nick));
		}
	}

	/*
	 * Spit out server info
	 */

	snd_svmsg(ptr, "%s %s%s%s\n%s %s%s%s / protocol revision %s%x-%x%s\n",
		"* Connected to VChat server at",
		S_FG_GRN, ptr->if_name, S_OFF,
		"* VChat TCP/IP conference server",
		S_FG_CYN, VERSION, S_OFF,
		S_FG_CYN, MAJOR_REV(PROTO_REV),	MINOR_REV(PROTO_REV), S_OFF);

	/*
	 * Inform the client of the current conference channel.
	 */

	snd2client(ptr, CHAN_ACK, (char *)&chan, sizeof(signed long));

	/*
	 * Show date of last login if available
	 */

	if(last_in != 0)
	{
		u_char	tstr[256];

		strftime(tstr, 255, "%a %b %d %H:%M:%S %Z %Y", localtime(&last_in));
		snd_svmsg(ptr, "* Last login %s\n", tstr);
	}

	/*
	 * Construct signon message
	 */

	if(!(cfdata.flags & CF_SUPPRESS_IP))
	{
		/*** Display client IP address ***/

		if(ptr->type == CLIENT_GATEWAY)
		{
			sprintf(sgnonmsg, "* Signon %s%s%s from %s%s%s via %s%s%s",
				S_FG_YEL, ptr->nick, S_OFF,
				S_FG_GRN, ptr->host, S_OFF,
				S_FG_GRN, ptr->gw_name, S_OFF);
		}
		else
		{
			if(!strcmp(ptr->user, ptr->nick))
			{
				sprintf(sgnonmsg, "* Signon %s%s%s from %s%s%s",
					S_FG_YEL, ptr->nick, S_OFF,
					S_FG_GRN, ptr->host, S_OFF);
			}
			else
			{
				sprintf(sgnonmsg, "* Signon %s%s%s (%s%s%s) from %s%s%s",
					S_FG_MAG, ptr->user, S_OFF,
					S_FG_YEL, ptr->nick, S_OFF,
					S_FG_GRN, ptr->host, S_OFF);
			}
		}
	}
	else
	{
		/*** Suppress display of client IP addr ***/

		if(ptr->type == CLIENT_GATEWAY)
		{
			sprintf(sgnonmsg, "* Signon %s%s%s via %s%s%s",
				S_FG_YEL, ptr->nick, S_OFF,
				S_FG_GRN, ptr->gw_name, S_OFF);
		}
		else
		{
			if(!strcmp(ptr->user, ptr->nick))
			{
				sprintf(sgnonmsg, "* Signon %s%s%s",
					S_FG_YEL, ptr->nick, S_OFF);
			}
			else
			{
				sprintf(sgnonmsg, "* Signon %s%s%s (%s%s%s)",
					S_FG_MAG, ptr->user, S_OFF,
					S_FG_YEL, ptr->nick, S_OFF);
			}
		}
	}

	/*
	 * Inform others about the user signing on.
	 */

	for(i = 0; i < clients; i++)
	{
		cl = cl_addr(i);

		if(cl->cl_opts & CL_OPT_SGNMSG)
			snd_svmsg(cl, "%s", sgnonmsg);

		if(cl != ptr && cl->channel == ptr->channel)
			snd_svmsg(cl, "* %s%s%s joined this channel",
				S_FG_YEL, ptr->nick, S_OFF);
	}

	/*
	 * Write log info
	 */

	vlog(VLOG_INFO, "Signon USER: %s NICK: %s HOST: %s GATEWAY: %s DNSLVL: %d",
		ptr->user, ptr->nick, ptr->host,
		(ptr->type == CLIENT_GATEWAY) ?
		(char *)ptr->gw_name : "-none-",
		ptr->dnschklvl);

	/*
	 * If there is currently no topic set for this channel
	 * then go ahead and request one from the client.
	 */

	if((px = topic_idx(ptr->channel)) == -1)
	{
		store_topic(ptr->channel, "-none-", CHFLG_REMOVABLE);
		touch_channel(ptr->channel);
		snd2client(ptr, SUBMIT_TOPIC, NULL, 0);
	}
}
