/* $Id: cmd_status.c,v 1.2 2002/06/23 23:16:06 mickey Exp $ */

#include <vserver.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>

#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>

#ifdef WIN32
  #include <process.h>
#else
  #include <arpa/inet.h>
#endif

/*
 * void cmd_status(CLDATA *ptr, u_char *dat, size_t len);
 *
 * Send server status information
 */

void cmd_status(CLDATA *ptr, u_char *dat, size_t len)
{
	u_int32_t	page	= ntohl(*(u_int32_t *)dat);

	/*
	 * Check access permission
	 */

	if(!(ptr->accperm & AUTH_STATUS))
	{
		snd_svmsg(ptr, "* You are not authorized to request server status pages");
		return;
	}

	/*
	 * Display status page
	 */

	switch(page)
	{
		case 0x00:	status_page0(ptr);
				break;
		case 0x01:	status_page1(ptr);
				break;
		case 0x02:	status_page2(ptr);
				break;
		case 0x03:	status_page3(ptr);
				break;
		default:	snd_svmsg(ptr, "* Invalid server status page number %d", page);
				break;
	}
}

/*
 * STATUS PAGE 1:
 */

void status_page0(CLDATA *ptr)
{
	TABLE		tab;
	TMREPORT	*tmr;
	OSINFO		*osinf;
	time_t		now;
	time_t		uptime;
	struct timeval	*tm;
	char		tmbuf[32];
	VIDENT		*sid = ident(0);

	table_init(&tab, ptr);

	/*
	 * Get server load average if possible
	 */

#if HAVE_GETLOADAVG
	getloadavg(vs.vs_loadavg, 3);
#endif

	/*
	 * Show status report
	 */

	table_printf(&tab, "\n*%s%s%s SERVER STATUS               %s GENERAL INFORMATION %s                    PAGE 0 %s\n*\n",
		S_FG_WHT, S_BG_BLU, S_BOLD, S_BG_RED, S_BG_BLU, S_OFF);

	time(&now);
	uptime = (now - vs.vs_start);
	tm = (struct timeval *) localtime(&vs.vs_start);
#ifdef WIN32
	strftime((char *)&tmbuf, 32, "%a, %d %b %Y %H:%M:%S", (struct tm *)tm);
#else
	strftime((char *)&tmbuf, 32, "%a, %d %b %Y %H:%M:%S %Z", (struct tm *)tm);
#endif
	table_printf(&tab, "*   Server location: %s%s%s (%s%s%s)\n", S_BOLD, inet_ntoa(ptr->if_addr), S_OFF, S_UL, ptr->if_name, S_OFF);

	osinf = get_os_info();

	if(osinf->info_ok)
	{
		table_printf(&tab, "*  Operating system: %s%s%s\n",
			S_BOLD, osinf->info, S_OFF);
	}

	table_printf(&tab, "*   Server up since: %s (%s)\n", tmbuf, pftime(vs.vs_start));
#ifdef WIN32
	table_printf(&tab, "*        Process ID: %s%s%d%s\n", S_BOLD, S_BLINK, _getpid(), S_OFF);
#else
	table_printf(&tab, "*        Process ID: %s%s%d%s\n", S_BOLD, S_BLINK, getpid(), S_OFF);
#endif

#if HAVE_GETLOADAVG
	table_printf(&tab, "*     Load averages: %s%.2f%s / %s%.2f%s / %s%.2f%s (5/10/15 min)\n",
		S_FG_YEL, vs.vs_loadavg[0], S_OFF,
		S_FG_CYN, vs.vs_loadavg[1], S_OFF,
		S_FG_MAG, vs.vs_loadavg[2], S_OFF);
#endif
	tmr = report_stat();

	table_printf(&tab, "*  Command duration: %s%s%s / %s%s%s / %s%s%s (min/avg/max)\n",
		S_FG_YEL, pr_time(&tmr->_min), S_OFF,
		S_FG_CYN, pr_time(&tmr->_avg), S_OFF,
		S_FG_MAG, pr_time(&tmr->_max), S_OFF);

	table_printf(&tab, "*       Connections: %s%d%s / %s%d%s / %s%d%s (active/total/dropped)\n",
		S_FG_GRN, vs.vs_active_conn, S_OFF,
		S_FG_CYN, vs.vs_total_conn, S_OFF,
		S_FG_RED, vs.vs_err_conn, S_OFF);

#ifdef MSGDB
	table_printf(&tab, "* Software revision: (%svserver%s %s%s%s - %s%s%s) %s[%s]%s msgdb\n",
		S_BOLD, S_OFF, S_FG_CYN, VERSION, S_OFF,
		S_FG_RED, REL_DATE, S_OFF,
		S_BOLD, sid->ident, S_OFF);
#else
	table_printf(&tab, "* Software revision: (%svserver%s %s%s%s - %s%s%s) %s[%s]%s",
		S_BOLD, S_OFF, S_FG_CYN, VERSION, S_OFF,
		S_FG_RED, REL_DATE, S_OFF,
		S_BOLD, sid->ident, S_OFF);
#endif

	table_flush(&tab);
}

/*
 * STATUS PAGE 1:
 */

void status_page1(CLDATA *ptr)
{
	TABLE	tab;

	/*
	 * Status report of processed server requests
	 */

	table_init(&tab, ptr);

	table_printf(&tab, "\n*%s%s%s SERVER STATUS               %s REQUEST STATISTICS %s                     PAGE 1 %s\n*\n",
		S_FG_WHT, S_BG_BLU, S_BOLD, S_BG_RED, S_BG_BLU, S_OFF);

	table_printf(&tab, "*      %stotal%s: %s%-38d%s  %sunknown%s: %s%d%s\n",
		S_BOLD, S_OFF, S_FG_MAG, vs.vs_requests_processed, S_OFF,
		S_BOLD, S_OFF, S_FG_RED, vs.vs_requests_unknown, S_OFF);

	table_printf(&tab, "*     signon: %s%-13d%s   pubmsg: %s%-13d%s   version: %s%d%s\n",
		S_FG_GRN, vs.vs_requests_signon, S_OFF,
		S_FG_GRN, vs.vs_requests_pubmsg, S_OFF,
		S_FG_GRN, vs.vs_requests_snd_id, S_OFF);
	table_printf(&tab, "*    signoff: %s%-13d%s   prvmsg: %s%-13d%s    status: %s%d%s\n",
		S_FG_GRN, vs.vs_requests_signoff, S_OFF,
		S_FG_GRN, vs.vs_requests_prvmsg, S_OFF,
		S_FG_GRN, vs.vs_requests_sv_stat, S_OFF);
	table_printf(&tab, "*     chnick: %s%-13d%s   action: %s%-13d%s      page: %s%d%s\n",
		S_FG_GRN, vs.vs_requests_chnick, S_OFF,
		S_FG_GRN, vs.vs_requests_action, S_OFF,
		S_FG_GRN, vs.vs_requests_page, S_OFF);
	table_printf(&tab, "*     chchan: %s%-13d%s   invite: %s%-13d%s    wakeup: %s%d%s\n",
		S_FG_GRN, vs.vs_requests_chchan, S_OFF,
		S_FG_GRN, vs.vs_requests_invite, S_OFF,
		S_FG_GRN, vs.vs_requests_wakeup, S_OFF);
	table_printf(&tab, "*    chtopic: %s%-13d%s    ulist: %s%-13d%s     rtcmp: %s%d%s\n",
		S_FG_GRN, vs.vs_requests_chtopic, S_OFF,
		S_FG_GRN, vs.vs_requests_ulist, S_OFF,
		S_FG_GRN, vs.vs_requests_rtcmp, S_OFF);
	table_printf(&tab, "*   asstopic: %s%-13d%s chanlist: %s%-13d%sclientinfo: %s%d%s\n",
		S_FG_GRN, vs.vs_requests_asstopic, S_OFF,
		S_FG_GRN, vs.vs_requests_chanlist, S_OFF,
		S_FG_GRN, vs.vs_requests_clientinfo, S_OFF);
	table_printf(&tab, "*   chpasswd: %s%-13d%ssetpasswd: %s%-13d%s   addnick: %s%d%s\n",
		S_FG_GRN, vs.vs_requests_chpasswd, S_OFF,
		S_FG_GRN, vs.vs_requests_setpasswd, S_OFF,
		S_FG_GRN, vs.vs_requests_addnick, S_OFF);
	table_printf(&tab, "*    delnick: %s%-13d%schaccperm: %s%-13d%s chmodperm: %s%d%s\n",
		S_FG_GRN, vs.vs_requests_delnick, S_OFF,
		S_FG_GRN, vs.vs_requests_chaccperm, S_OFF,
		S_FG_GRN, vs.vs_requests_chmodperm, S_OFF);
	table_printf(&tab, "*       kill: %s%-13d%sbroadcast: %s%-13d%s   acl_add: %s%d%s\n",
		S_FG_GRN, vs.vs_requests_kill, S_OFF,
		S_FG_GRN, vs.vs_requests_broadcast, S_OFF,
		S_FG_GRN, vs.vs_requests_acl_add, S_OFF);
	table_printf(&tab, "* acl_delete: %s%-13d%sacl_flush: %s%-13d%s  acl_list: %s%d%s\n",
		S_FG_GRN, vs.vs_requests_acl_delete, S_OFF,
		S_FG_GRN, vs.vs_requests_acl_flush, S_OFF,
		S_FG_GRN, vs.vs_requests_acl_list, S_OFF);

	table_printf(&tab, "*    lastlog: %s%-13d%s    clopt: %s%-13d%s     svopt: %s%d%s\n",
		S_FG_GRN, vs.vs_requests_lastlog, S_OFF,
		S_FG_GRN, vs.vs_requests_clopt, S_OFF,
		S_FG_GRN, vs.vs_requests_svopt, S_OFF);

	table_printf(&tab, "*    fixchan: %s%-13d%s svdnsopt: %s%-13d%s   exclude: %s%d%s",
		S_FG_GRN, vs.vs_requests_fixchan, S_OFF,
		S_FG_GRN, vs.vs_requests_svdnsopt, S_OFF,
		S_FG_GRN, vs.vs_requests_exclude, S_OFF);

	table_flush(&tab);
}

/*
 * STATUS PAGE 2:
 */

void status_page2(CLDATA *ptr)
{
	TABLE		tab;

	table_init(&tab, ptr);

	table_printf(&tab, "\n*%s%s%s SERVER STATUS                  %s MEMORY USAGE %s                        PAGE 2 %s\n*\n",
		S_FG_WHT, S_BG_BLU, S_BOLD, S_BG_RED, S_BG_BLU, S_OFF);

	table_printf(&tab, "*                %s RECORDS    RECSIZE    TOTAL         %s\n",
		S_UL, S_OFF);
	table_printf(&tab, "*    Client data: %-10d %-10d %d\n",
		clients, sizeof(CLDATA), (clients * sizeof(CLDATA)));

	table_printf(&tab, "*   Channel data: %-10d %-10d %d\n",
		vtop.entries, sizeof(struct chantopic),
		(vtop.entries * sizeof(struct chantopic)));

	table_printf(&tab, "* Authority data: %-10d %-10d %d\n",
		auth_numrec, sizeof(AUTHREC), (auth_numrec * sizeof(AUTHREC)));

	table_printf(&tab, "*   Lastlog data: %-10d %-10d %d",
		llog_header->total_records, sizeof(LLOGDAT), (llog_header->total_records * sizeof(LLOGDAT)));



	table_flush(&tab);
}

/*
 * STATUS PAGE 3:
 */

void status_page3(CLDATA *ptr)
{
	TABLE		tab;
	struct rusage	ru;

	table_init(&tab, ptr);

	table_printf(&tab, "\n*%s%s%s SERVER STATUS                 %s RESOURCE USAGE %s                       PAGE 3 %s\n*\n",
		S_FG_WHT, S_BG_BLU, S_BOLD, S_BG_RED, S_BG_BLU, S_OFF);

	if(!getrusage(RUSAGE_SELF, &ru))
	{
		table_printf(&tab, "*     %s%s%s real %s%ld.%03ld%s user %s%ld.%03ld%s sys\n*\n",
			S_FG_YEL, pftime(vs.vs_start), S_OFF,
			S_FG_CYN, ru.ru_utime.tv_sec,
			ru.ru_utime.tv_usec / 10000, S_OFF,
			S_FG_MAG, ru.ru_stime.tv_sec,
			ru.ru_stime.tv_usec / 10000, S_OFF);

		table_printf(&tab, "*    Maximum resident set size: %s%-16d%s       Page faults: %s%d%s\n",
			S_FG_GRN, ru.ru_maxrss, S_OFF,
			S_FG_GRN, ru.ru_majflt, S_OFF);
		table_printf(&tab, "*     Average shared text size: %s%-16d%s     Page reclaims: %s%d%s\n",
			S_FG_GRN, ru.ru_ixrss, S_OFF,
			S_FG_GRN, ru.ru_minflt, S_OFF);
		table_printf(&tab, "*   Average unshared data size: %s%-16d%s   Swap operations: %s%d%s\n",
			S_FG_GRN, ru.ru_idrss, S_OFF,
			S_FG_GRN, ru.ru_nswap, S_OFF);
		table_printf(&tab, "*  Average unshared stack size: %s%-16d%s     Messages sent: %s%d%s\n",
			S_FG_GRN, ru.ru_isrss, S_OFF,
			S_FG_GRN, ru.ru_msgsnd, S_OFF);
		table_printf(&tab, "*    Blocking input operations: %s%-16d%s Messages received: %s%d%s\n",
			S_FG_GRN, ru.ru_inblock, S_OFF,
			S_FG_GRN, ru.ru_msgrcv, S_OFF);
		table_printf(&tab, "*   Blocking output operations: %s%-16d%s  Signals received: %s%d%s\n",
			S_FG_GRN, ru.ru_oublock, S_OFF,
			S_FG_GRN, ru.ru_nsignals, S_OFF);
		table_printf(&tab, "*   Voluntary context switches: %s%d%s\n", S_FG_GRN, ru.ru_nvcsw, S_OFF);
		table_printf(&tab, "* Involuntary context switches: %s%d%s", S_FG_GRN, ru.ru_nivcsw, S_OFF);
	}
	else
	{
		table_printf(&tab, "* No resource usage statistics available");
	}

	table_flush(&tab);
}
