/* $Id: cmd_svdnsopt.c,v 1.1 2002/06/23 23:16:54 mickey Exp $ */

#include <vserver.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>

/*
 * void cmd_svdnsopt(CLDATA *ptr, void *dat);
 *
 * Get/Set server DNS check options.
 */

void cmd_svdnsopt(CLDATA *ptr, u_char *dat, size_t len)
{
	/*
	 * Check if the client is permitted to alter server options.
	 */

	if(!(ptr->accperm & AUTH_SOPT))
	{
		snd_svmsg(ptr, "* You are not authorized to alter server options");
		return;
	}

	/*
	 * Check for correct data size.
	 */

	if(len == 0)
	{
		/*
		 * Only display current DNS checking level.
		 */

		goto displaystatus;
	}
	else if(len != 1)
	{
		snd_svmsg(ptr, "* Invalid packet length encountered");
		return;
	}

	/*
	 * Check for correct DNS check level.
	 */

	if(*dat > SV_DNS_CHK_STRICT)
	{
		snd_svmsg(ptr, "* Invalid DNS checking level specification");
		return;
	}

	/*
	 * When the new DNS checking level specification is more secure
	 * then the old level setting, then kick off all clients not
	 * fullfilling the new requirements.
	 */

	if(*dat > cfdata.dnschklvl)
	{
		register int	i;
		CLDATA		*cl;

		for(i = 0; i < clients; i++)
		{
			cl = cl_addr(i);

			if(cl->dnschklvl < *dat)
			{
				cl->flags |= CFLAG_SIGNOFF;
				BZERO(&cl->offmsg, sizeof(cl->offmsg));
				sprintf(cl->offmsg, "level %d DNS clearance required", *dat);
			}
		}
	}

	/*
	 * Set new DNS checking level.
	 */

	cfdata.dnschklvl = *dat;	

	/*
	 * Display current setting.
	 */

displaystatus:

	snd_svmsg(ptr, "* DNS checking: %s%s%s",
			S_FG_GRN, cfdata.dnschklvl == DNS_CHK_STRICT ?
			"strict" : cfdata.dnschklvl == DNS_CHK_REV ?
			"reverse" : cfdata.dnschklvl == DNS_CHK_FWD ?
			"forward" : "none", S_OFF);
}
