/* $Id: cmd_svopt.c,v 1.3 2002/06/23 23:16:06 mickey Exp $ */

#include <vserver.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>

/*
 * void cmd_svopt(CLDATA *ptr, void *dat);
 *
 * Get/Set server options.
 */

void cmd_svopt(CLDATA *ptr, u_char *dat, size_t len)
{
	TABLE		tab;
	GSOPT		*opt;
	u_int32_t	givemask;
	u_int32_t	takemask;
	u_int32_t	md;

	/*
	 * Check if the client is permitted to alter server options
	 */

	if(!(ptr->accperm & AUTH_SOPT))
	{
		snd_svmsg(ptr, "* You are not authorized to alter server options");
		return;
	}

	/*
	 * Check for correct data size.
	 */

	if(len != sizeof(GSOPT))
	{
		snd_svmsg(ptr, "* Invalid packet length encountered.");
		return;
	}

	/*
	 * Retrieve arguments.
	 */

	opt = (GSOPT *)dat;
	givemask = ntohl(opt->givemask);
	takemask = ntohl(opt->takemask);
	md = 0;

	/*
	 * Check for give/take options that will compensate each other
	 */

	md = (givemask ^ takemask);
	givemask &= md;
	takemask &= md;

	/*
	 * Modify server options (unknown options are silently ignored)
	 */

	if(givemask & SV_OPT_SHOW_IP)
		cfdata.flags &= ~CF_SUPPRESS_IP;

	if(givemask & SV_OPT_MAIL_INV)
		cfdata.flags |= CF_ENABLE_MAIL;

	if(givemask & SV_OPT_REG_ONLY)
	{
		cfdata.flags |= CF_REG_ONLY;
		killall_by_flag(FLG_NOTSET, CFLAG_REGISTERED, "not registered");
	}

	if(givemask & SV_OPT_FLOOD_GUARD)
		cfdata.flags |= CF_FLOOD_GUARD;


	if(takemask & SV_OPT_SHOW_IP)
		cfdata.flags |= CF_SUPPRESS_IP;

	if(takemask & SV_OPT_MAIL_INV)
		cfdata.flags &= ~CF_ENABLE_MAIL;

	if(takemask & SV_OPT_REG_ONLY)
		cfdata.flags &= ~CF_REG_ONLY;

	if(takemask & SV_OPT_FLOOD_GUARD)
		cfdata.flags &= ~CF_FLOOD_GUARD;

	/*
	 * Print the current option set up
	 */

	table_init(&tab, ptr);

	table_printf(&tab, "* Server options: ");

	display_svopt(&tab);

	table_flush(&tab);
}

void display_svopt(TABLE *cltab)
{
	if(cfdata.flags & CF_SUPPRESS_IP)
		table_printf(cltab, "%s-%sSHOW_IP", S_FG_RED, S_OFF);
	else
		table_printf(cltab, "%s+%sSHOW_IP", S_FG_GRN, S_OFF);

	if(cfdata.flags & CF_ENABLE_MAIL)
		table_printf(cltab, " %s+%sMAIL_INV", S_FG_GRN, S_OFF);
	else
		table_printf(cltab, " %s-%sMAIL_INV", S_FG_RED, S_OFF);

	if(cfdata.flags & CF_REG_ONLY)
		table_printf(cltab, " %s+%sREG_ONLY", S_FG_GRN, S_OFF);
	else
		table_printf(cltab, " %s-%sREG_ONLY", S_FG_RED, S_OFF);

	if(cfdata.flags & CF_FLOOD_GUARD)
		table_printf(cltab, " %s+%sFLOOD_GUARD", S_FG_GRN, S_OFF);
	else
		table_printf(cltab, " %s-%sFLOOD_GUARD", S_FG_RED, S_OFF);
}

/*
 * void killall_by_flag(u_int8_t sc, u_int32_t flag, char *reason);
 *
 * Accordning to the <sc> parameter, all clients that have or do
 * not have set the client flag specified by the <flag> parameter
 * get signoff with the reason specified by the string pointed to
 * by the <reason> parameter.
 */

void killall_by_flag(u_int8_t sc, u_int32_t flag, char *reason)
{
	register CLDATA	*cl;
	register int	i;

	switch(sc)
	{
		case FLG_SET:		for(i = 0; i < clients; i++)
					{
						cl = cl_addr(i);

						if(cl->flags & flag && cl->accperm != AUTH_ROOT)
						{
							/*
							 * Flag set
							 */

							cl->flags |= CFLAG_SIGNOFF;
							BZERO(&cl->offmsg, sizeof(cl->offmsg));
							strncpy(cl->offmsg, reason, sizeof(cl->offmsg) - 1);
						}
					}
					break;

		case FLG_NOTSET:	for(i = 0; i < clients; i++)
					{
						cl = cl_addr(i);

						if(!(cl->flags & flag) && cl->accperm != AUTH_ROOT)
						{
							/*
							 * Flag not set
							 */

							cl->flags |= CFLAG_SIGNOFF;
							BZERO(&cl->offmsg, sizeof(cl->offmsg));
							strncpy(cl->offmsg, reason, sizeof(cl->offmsg) - 1);
						}
					}
					break;

		default:		return;
	}
}
