/* $Id: cmd_wakeup.c,v 1.2 2002/06/23 23:16:06 mickey Exp $ */

#include <vserver.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>

#include <errno.h> 

#if HAVE_SYS_UIO_H
  #include <sys/uio.h>
#endif

/*
 * void cmd_wakeup(CLDATA *ptr, u_char *dat);
 *
 * Send wakeup signal to user
 */

void cmd_wakeup(CLDATA *ptr, u_char *dat)
{
	register CLDATA	*cl;
	register int	i;
	register int	y;
	TABLE		cltab;
	char		*xargv[VCMD_MAXARGS];
	char		**aptr;
	int		n	= 0;
	int		xargc	= 0;

#if HAVE_SYS_UIO_H
	struct iovec iov[2];
	VMSG	svmsg;
#else
	u_char	sndbuf[256];
	VMSG	*svmsg;
#endif

	/*
	 * Initialize output table.
	 */

	table_init(&cltab, ptr);

	/*
	 * Parse nickname list into argument vector.
	 */

#if HAVE_STRSEP
	for(aptr = xargv; (*aptr = (char *) strsep((char **)&dat, " \t")) != NULL && xargc < VCMD_MAXARGS;)
#else
	aptr = xargv;

	*aptr = (char *)strtok((char *)dat, " \t");

	while(*aptr != NULL && xargc < VCMD_MAXARGS)
#endif
	{
		/*
		 * Check for invalid nicknames.
		 */

		y = 0;

		if(!nick_ok(*aptr))
		{
			table_xprintf(&cltab, "\n* Invalid nickname '%s%s%s' ignored",
				S_FG_RED, *aptr, S_OFF);
			y = 1;
		}
		else
		{
			/*
			 * Check for duplicated nicknames.
			 */

			for(i = 0; i < xargc; i++)
			{
				if(!strcmp(xargv[i], *aptr))
				{
					table_xprintf(&cltab, "\n* Duplicated nickname '%s%s%s' ignored",
						S_FG_RED, *aptr, S_OFF);

					y = 1;
					break;
				}
			}
		}

		/*
		 * Check for own nickname.
		 */

		if(!strcmp(ptr->nick, *aptr))
		{
			table_xprintf(&cltab, "\n* No need to wake yourself up");
			y = 1;
		}

		if(**aptr != '\0' && !y)
		{
			++aptr;
			++xargc;
		}

#ifndef HAVE_STRSEP
		*aptr = (char *)strtok(NULL, " \t");
#endif

	}

	/*
	 * Check if there are any nicknames in list
	 */

	if(!xargc)
	{
		table_xprintf(&cltab, "\n* No nickname(s) to wake up");
		table_flush(&cltab);
		return;
	}

	/*
	 * Notify list of users
	 */

	for(y = 0; y < xargc; y++)
	{
		for(i = n = 0; i < clients; i++)
		{
			cl = cl_addr(i);

			if(!strcmp(cl->nick, xargv[y])
			&& !(cl->flags & CFLAG_ERROR))
			{
				if(cl->cl_opts & CL_OPT_WAKEUP)
				{
					/* Send wakeup signal */
#if HAVE_SYS_UIO_H
					svmsg.cmd = htonl(WAKEUP);
					svmsg.len = htonl(strlen(ptr->nick));

					iov[0].iov_base = (char *)&svmsg;
					iov[1].iov_base = (char *)ptr->nick;
					iov[0].iov_len = sizeof(VMSG);
					iov[1].iov_len = strlen(ptr->nick);

					if(vio_writev(cl->fd, (struct iovec *)&iov, 2, CONN_TIMEOUT) == -1)
#else
					char	*datbuf;
					svmsg = (VMSG *)&sndbuf;
					datbuf = (char *)&sndbuf[sizeof(VMSG)];

					svmsg->cmd = htonl(WAKEUP);
					svmsg->len = htonl(strlen(ptr->nick));

					sprintf(datbuf, "%s", ptr->nick);

					if(vio_write(cl->fd, (char *)&sndbuf, sizeof(VMSG) + strlen(ptr->nick), CONN_TIMEOUT) == -1)
#endif
					{
						cl->flags |= CFLAG_ERROR;
						cl->error = errno;
					}

					if(n >= 0)
					{
						++n;
					}
					else
					{
						n = 1;
					}
				}
				else
				{
					if(n == 0)
						n = -1;
				}
			}
		}

		if(n > 0)
		{
			table_xprintf(&cltab, "\n* Sent wakeup signal to %s%s%s",
				S_FG_YEL, xargv[y], S_OFF);

			vlog(VLOG_INFO, "Wakeup USER: %s NICK: %s TO: %s",
				ptr->user, ptr->nick, xargv[y]);
		}
		else if(n == 0)
		{
			table_xprintf(&cltab, "\n* Nickname '%s%s%s' not found",
				S_FG_RED, xargv[y], S_OFF);
		}
		else if(n == -1)
		{
			table_xprintf(&cltab, "\n* Cannot wake up '%s%s%s' (wakeup disabled)",
				S_FG_RED, xargv[y], S_OFF);
		}
	}

	/*
	 * Flush output table.
	 */

	table_flush(&cltab);
}
