/* $Id: dnschk.c,v 1.1 2002/06/23 23:16:54 mickey Exp $ */

#include <vserver.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>

#include <stdlib.h>
#include <stdarg.h>
#include <errno.h>
#include <signal.h>
#include <netdb.h>

#ifdef WIN32
  #include <winsock.h>
#else
  #include <sys/socket.h>
#endif

#include <arpa/inet.h>

/****** Code ******/

u_char dns_check(struct in_addr *ip, u_char *namebuf, size_t maxsize)
{
	struct	hostent		*ht;
	char			**p;

	/*
	 * Initialize name buffer.
	 */

	BZERO(namebuf, maxsize);

	/*
	 * Do a forward DNS lookup to get the name for the IP address.
	 */

	if((ht = gethostbyaddr((char *)ip, sizeof(ip), AF_INET)) == NULL)
	{
		/*
		 * No name for this IP address found.
		 */

		char *str = inet_ntoa(*ip);
		strncpy(namebuf, str, maxsize - 1);

		return DNS_CHK_NONE;
	}

	/*
	 * Found DNS name for address, copy it in.
	 */

	strncpy(namebuf, ht->h_name, maxsize - 1);

	/*
	 * Now check if the name maps back to an address.
	 */

	if((ht = gethostbyname(ht->h_name)) == NULL)
	{
		/*
		 * No reverse mapping.
		 */

		return DNS_CHK_FWD;
	}

	/*
	 * Reverse DNS mapping present, check for strict.
	 */

	for(p = ht->h_addr_list; *p != NULL; p++)
	{
		if(!BCMP(*p, ip, sizeof(ip)))
			return DNS_CHK_STRICT;
	}

	return DNS_CHK_REV;
}
