/* $Id: proto.h,v 1.3 2002/06/23 23:16:08 mickey Exp $ */

#ifndef __VSERVER_PROTO_H__
#define __VSERVER_PROTO_H__

#ifdef HAVE_SYS_UIO_H
  #include <sys/uio.h>
#endif

#ifndef WIN32
  #include <sys/socket.h>
  #include <sys/un.h>
  #include <unistd.h>
#else
  #include <windows.h>
#endif

#include "vserver.h"
#include "microtime.h"
#include "acl.h"
#include "auth.h"

/*-----------*
 | vserver.c |
 *-----------*/

#ifndef WIN32

int	main(int argc, char *argv[]);

#else

VOID _CRTAPI1	main(int argc, char *argv[]);

#endif

int		ServiceStart(void);

/*-------------*
 | win32util.c |
 *-------------*/

#ifdef WIN32
VOID WINAPI	svc_main(DWORD dwArgc, LPTSTR *lpszArgv);
VOID WINAPI	svc_ctl(DWORD dwCtlCode);
BOOL		ReportStatusToSCMgr(DWORD dwCurrentState, DWORD dwWin32ExitCode, DWORD dwWaitHint);
LPTSTR		GetLastErrorText(LPTSTR lpszBuf, DWORD dwSize);
void		LogMessage(int pri, char *fmt, ...);
int		GetRegistryData(char *cfgname);
VOID		ServiceStop(void);
char		*winerror(DWORD code);
#endif

/*-------------*
 | serv_subr.c |
 *-------------*/

#ifndef WIN32
int	process_cmdline(int argc, char *argv[]);
#endif

int	init_global(void);
void	exit_global(void);
u_long	chatters(signed long chan);
int	nick_ok(u_char *nick);
int	nick_dup(u_char *nick);
char	*iptostr(struct in_addr ip);
char	*iptoname(struct in_addr ip);
char	*dnsname(struct in_addr ip);
void	str_lower(char *str);
void	vs_init(void);
VIDENT	*ident(u_long type);
int	fix_topic(char *arg);
void	reread_config(void);
void	rotate_logfile(void);
void	pipesig(void);
int	unescape_str(u_char *str, u_char *buf, size_t len);

/*----------*
 | vtopic.c |
 *----------*/

int	store_topic(signed long channel, char *top, u_long flags);
int	remove_topic(signed long channel, u_int32_t flags);
int	topic_idx(signed long channel);
char	*topic_ptr(signed long channel);
void	touch_channel(signed long channel);
char	*pftime(time_t clock);
char	*pf2time(time_t clock1, time_t clock2);

/*-------*
 | acl.c |
 *-------*/

int	acl_add_rule(u_short rule, char action, struct in_addr address, struct in_addr netmask, time_t expires);
int	acl_check_access(struct in_addr address, char *msgbuf);
int	acl_delete_rule(u_short rule);
int	acl_delete_expired_rules();
int	acl_find_rule(u_short rule);
void	acl_flush(void);
int	acl_load_file(char *file);
void	acl_log_status(void);
int	acl_merge_file(char *file);
void	acl_print_rule(ACLRULE *rule, char *msgbuf);
int	acl_process_cmd(char *cmd, char *msgbuf);
int	netmask_bits(struct in_addr netmask);
void	netmask_create(int bits, struct in_addr *netmask);

/*--------*
 | auth.c |
 *--------*/

int	auth_load(void);
int	auth_save(void);

#ifndef WIN32
int	auth_reset(char *rootpasswd);
#endif

void	auth_flush(void);
int	auth_add_record(u_char *nick, u_char *passwd);
int	auth_delete_record(u_char *nick);
AUTHREC	*auth_getbynick(u_char *nick);

/*----------*
 | dnschk.c |
 *----------*/

u_char	dns_check(struct in_addr *ip, u_char *namebuf, size_t maxsize);

/*--------*
 | vlog.c |
 *--------*/

void	init_logfile(void);
void	exit_logfile(void);
void	vlog(int pri, char *fmt, ...);
void	init_lastlog(void);
void	exit_lastlog(void);
void	sync_lastlog(void *addr, size_t len);
void	lastlog_addentry(CLDATA *cl, char *info);

/*------------*
 | vcommand.c |
 *------------*/

void	handle_cmd(u_short cmd, CLDATA *cl, u_char *data, size_t len);

/*---------*
 | table.c |
 *---------*/

void	table_init(TABLE *table, CLDATA *cl);
void	table_printf(TABLE *table, char *fmt, ...);
void	table_xprintf(TABLE *table, char *fmt, ...);
void	table_flush(TABLE *table);

/*------------*
 | vchatctl.c |
 *------------*/

int	init_ctlsock(void);
void	exit_ctlsock(void);
void	process_ctlcmd(void);
void	ctlmsg(struct sockaddr_un *to, u_char type, u_char flags, char *fmt, ...);

/*---------------*
 | vclientconn.c |
 *---------------*/

void	assign_client(void);
void	delete_client(int cl_fd);
int	getclientnum(int fd);
void	snd_svmsg(CLDATA *cldat, char *fmt, ...);
void	snd2client(CLDATA *cldat, u_short packtype, char *data, size_t datalen);

/*-------------*
 | microtime.c |
 *-------------*/

void	timestamp(struct timeval *tm);
void	store_tm_value(struct timeval *tp);
char	*pr_time(struct timeval *tm);
TMREPORT *report_stat(void);
struct timeval *diff_t(struct timeval *t1, struct timeval *t2);
void	push_value(struct timeval *srctp, struct timeval *stacktp);
void	avgvalue(struct timeval *pt);

/*---------*
 | msgdb.c |
 *---------*/

#ifdef MSGDB
void	msg_storedb(char *msgfrom, char *msgto, char *msgtxt);
int	sqlstate_err(void);
void	disp_sqlstate_err(void);
int	whenexp_chk(void);
#endif

/*-----------*
 | os_info.c |
 *-----------*/

OSINFO	*get_os_info(void);

/*-------*
 | vio.c |
 *-------*/

ssize_t	vio_read(int fd, void *buf, size_t nbytes, time_t tout);
ssize_t	vio_write(int fd, void *buf, size_t nbytes, time_t tout);

#ifdef HAVE_SYS_UIO_H
ssize_t	vio_writev(int fd, struct iovec *iov, int iovcnt, time_t tout);
#endif

#ifdef WIN32
DWORD WINAPI	ReadOp(LPVOID arg);
DWORD WINAPI	WriteOP(LPVOID arg);
#else
void	conn_timeout(void);
#endif

/*---------*
 | usage.c |
 *---------*/

#ifndef WIN32
void	usage(char *fmt, ...);
#endif

/*---------------*
 | cmd_acl_add.c |
 *---------------*/

void	cmd_acl_add(CLDATA *ptr, u_char *dat, size_t len);

/*------------------*
 | cmd_acl_delete.c |
 *------------------*/

void	cmd_acl_delete(CLDATA *ptr, u_char *dat, size_t len);

/*-----------------*
 | cmd_acl_flush.c |
 *-----------------*/

void	cmd_acl_flush(CLDATA *ptr);

/*----------------*
 | cmd_acl_list.c |
 *----------------*/

void	cmd_acl_list(CLDATA *ptr);

/*--------------*
 | cmd_action.c |
 *--------------*/

void	cmd_action(CLDATA *ptr, u_char *dat);

/*---------------*
 | cmd_addnick.c |
 *---------------*/

void	cmd_addnick(CLDATA *ptr, u_char *dat);

/*----------------*
 | cmd_asstopic.c |
 *----------------*/

void	cmd_asstopic(CLDATA *ptr, u_char *dat);

/*-----------------*
 | cmd_broadcast.c |
 *-----------------*/

void	cmd_broadcast(CLDATA *ptr, u_char *dat);

/*----------------*
 | cmd_chanlist.c |
 *----------------*/

void	cmd_chanlist(CLDATA *ptr);

/*--------------*
 | cmd_chchan.c |
 *--------------*/

void	cmd_chchan(CLDATA *ptr, u_char *dat);

/*-----------------*
 | cmd_chaccperm.c |
 *-----------------*/

void	cmd_chaccperm(CLDATA *ptr, u_char *dat, size_t len);
void	change_perm(CLDATA *ptr, u_char which, u_char *dat, size_t len);

/*-----------------*
 | cmd_chmodperm.c |
 *-----------------*/

void	cmd_chmodperm(CLDATA *ptr, u_char *dat, size_t len);

/*--------------*
 | cmd_chnick.c |
 *--------------*/

void	cmd_chnick(CLDATA *ptr, u_char *dat);

/*----------------*
 | cmd_chpasswd.c |
 *----------------*/

void	cmd_chpasswd(CLDATA *ptr, u_char *dat);

/*---------------*
 | cmd_chtopic.c |
 *---------------*/

void	cmd_chtopic(CLDATA *ptr, u_char *dat);

/*------------------*
 | cmd_clientinfo.c |
 *------------------*/

void	cmd_clientinfo(CLDATA *ptr, u_char *dat);

/*-------------*
 | cmd_clopt.c |
 *-------------*/

void	cmd_clopt(CLDATA *ptr, u_char *dat, size_t len);
void	display_clopt(TABLE *cltab, u_int32_t opts);

/*---------------*
 | cmd_delnick.c |
 *---------------*/

void	cmd_delnick(CLDATA *ptr, u_char *dat);

/*---------------*
 | cmd_exclude.c |
 *---------------*/

void	cmd_exclude(CLDATA *ptr, u_char *dat, size_t len);

/*---------------*
 | cmd_fixchan.c |
 *---------------*/

void	cmd_fixchan(CLDATA *ptr, u_char *dat, size_t len);

/*--------------*
 | cmd_invite.c |
 *--------------*/

void	cmd_invite(CLDATA *ptr, u_char *dat);

/*------------*
 | cmd_kill.c |
 *------------*/

void	cmd_kill(CLDATA *ptr, u_char *dat);

/*-------------*
 | cmd_lastlog |
 *-------------*/

void	cmd_lastlog(CLDATA *ptr, u_char *dat);

/*------------*
 | cmd_page.c |
 *------------*/

void	cmd_page(CLDATA *ptr, u_char *dat);

/*--------------*
 | cmd_prvmsg.c |
 *--------------*/

void	cmd_prvmsg(CLDATA *ptr, u_char *dat);

/*--------------*
 | cmd_pubmsg.c |
 *--------------*/

void	cmd_pubmsg(CLDATA *ptr, u_char *dat);

/*-------------*
 | cmd_query.c |
 *-------------*/

void	cmd_query(CLDATA *ptr, u_char *dat);
void	display_permission1(TABLE *tab, u_long perm);
void	display_permission2(TABLE *tab, u_long perm);

/*-------------*
 | cmd_rtcmp.c |
 *-------------*/

void	cmd_rtcmp(CLDATA *ptr, u_char *dat);


/*-----------------*
 | cmd_setpasswd.c |
 *-----------------*/

void	cmd_setpasswd(CLDATA *ptr, u_char *dat);

/*---------------*
 | cmd_signmsg.c |
 *---------------*/

void	cmd_signmsg(CLDATA *ptr);

/*---------------*
 | cmd_signoff.c |
 *---------------*/

void	cmd_signoff(CLDATA *ptr, u_char *reason, u_char *errmsg);

/*--------------*
 | cmd_signon.c |
 *--------------*/

void	cmd_signon(CLDATA *ptr, time_t last_in);

/*--------------*
 | cmd_status.c |
 *--------------*/

void	cmd_status(CLDATA *ptr, u_char *dat, size_t len);
void	status_page0(CLDATA *ptr);
void	status_page1(CLDATA *ptr);
void	status_page2(CLDATA *ptr);
void	status_page3(CLDATA *ptr);

/*----------------*
 | cmd_svdnsopt.c |
 *----------------*/

void	cmd_svdnsopt(CLDATA *ptr, u_char *dat, size_t len);

/*-------------*
 | cmd_svopt.c |
 *-------------*/

void	cmd_svopt(CLDATA *ptr, u_char *dat, size_t len);
void	display_svopt(TABLE *cltab);
void	killall_by_flag(u_int8_t sc, u_int32_t flag, char *reason);

/*-------------*
 | cmd_ulist.c |
 *-------------*/

void	cmd_ulist(CLDATA *ptr, u_char *dat);

/*---------------*
 | cmd_version.c |
 *---------------*/

void	cmd_version(CLDATA *ptr);

/*--------------*
 | cmd_wakeup.c |
 *--------------*/

void	cmd_wakeup(CLDATA *ptr, u_char *dat);

#endif /* __VSERVER_PROTO_H__ */

