/******************************************************************************
 * This file is part of a software distribution, which is furnished under the *
 * terms of a license.  Use of this software  by any means is subject to this *
 * license  and  signifies  the  acceptance of  the  licensing  terms  stated *
 * therein. Please see  the file LICENSE in the  top-level directory  of this *
 * software  distribution  for detailed copyright  disclaimers  and licensing *
 * terms.                                                                     *
 ******************************************************************************
 * Copryight (c) by Andreas S. Wetzel - All rights reserved.                  *
 ******************************************************************************/

/* $Id: types.h,v 1.2 2002/06/21 16:59:08 mickey Exp $ */

#ifndef __VTYPES_H__
#define __VTYPES_H__

#ifndef __VPARAM_H__
#  include <vproto/param.h>
#endif

/**********************************
 * Software identification packet *
 **********************************/

typedef struct
{
	u_int8_t	ident[VPIDENTSIZE+1];	/* Indetification	*/
	u_int32_t	type;			/* Type specification	*/
	u_int32_t	proto;			/* Protocol revision	*/
	u_int32_t	magic;			/* Magic number		*/

} VIDENT;

/*******************************
 * Server login data structure *
 *******************************/

typedef struct
{
	u_int8_t	uname[VPUSERSIZE+1];	/* Username		*/
	u_int8_t	nick[VPNICKSIZE+1];	/* Initial nickname	*/
	struct in_addr	remote_user_ip;		/* addr of remote user	*/
						/* if type == GATEWAY	*/
} VLOGINDAT;

/*************************************
 * Password authentication structure *
 *************************************/

typedef struct
{
	u_int8_t	passwd[VPPASSWDSIZE+1];	/* Password		*/	

} VPASSWD;

/*******************************************************
 * Access Control List structure used with CMD_ACL_ADD *
 *******************************************************/

typedef struct
{
	struct in_addr	address;
	struct in_addr	netmask;
	u_short		rule;
#define	ACL_ACT_ACCEPT	0x0001
#define	ACL_ACT_REJECT	0x0000
	u_short		action;
	u_int32_t	ttl;

} VACL;

/************************************
 * General purpose messag structure *
 ************************************/

typedef struct
{
	u_int32_t	cmd;			/* server message byte	*/
	u_int32_t	len;			/* Length of data that	*/
						/* follows this message */
} VMSG;

/********************************
 * Ulist command data structure *
 ********************************/

struct list
{
	u_int32_t	cmd;			/* ulist command	*/
	int32_t		chan;			/* channel number	*/

};

/*******************************
 * RTCMP packet data structure *
 *******************************/

typedef struct
{
	struct	timeval stamp;			/* Timestamp		*/
	u_int32_t	seq;			/* Sequence number	*/
	u_int8_t	dummy[44];		/* Filler		*/

} RTCMP;

/************************
 * GSOPT data structure *
 ************************/

typedef struct
{
	u_int32_t	givemask;		/* Give mask		*/
	u_int32_t	takemask;		/* Take mask		*/
	u_int32_t	control;		/* Reserved		*/

} GSOPT;

#endif /* __VTYPES_H__ */
