/* $Id: vserver.h,v 1.4 2002/06/23 23:16:08 mickey Exp $ */

#ifndef __VSERVER_H__
#define __VSERVER_H__

/*
 * Suck in config.h
 */

#include <config.h>

/*
 * Include commonly used header files
 */

#include <stdio.h>

#if HAVE_STRING_H
#  include <string.h>
#endif

#if HAVE_STRINGS_H
#  include <strings.h>
#endif

#if HAVE_CTYPE_H
#  include <ctype.h>
#endif

#include <stdlib.h>
#include <stdarg.h>
#include <sys/types.h>
#include <limits.h>
#include <errno.h>

#if TIME_WITH_SYS_TIME
#  include <sys/time.h>
#  include <time.h>
#else
#  if HAVE_SYS_TIME_H
#    include <sys/time.h>
#  else
#    include <time.h>
#  endif
#endif

#ifndef WIN32
  #include <netinet/in.h>
#endif

#if HAVE_UNISTD_H
#  include <unistd.h>
#endif

/*
 * Include common VChat headers
 */

#include <options.h>
#include <vproto/types.h>

/*
 * Set up some handy macros
 */

#if HAVE_BCOPY
#  define       BCOPY(a,b,c)            bcopy(a,b,c)
#elif HAVE_MEMCPY
#  define       BCOPY(a,b,c)            memcpy(b,a,c)
#else
#  error "Neither bcopy nor memcpy are available"
#endif

#if HAVE_BZERO
#  define       BZERO(a,b)              bzero(a,b)
#elif HAVE_MEMSET
#  define       BZERO(a,b)              memset(a,0,b)
#else
#  error "Neither bzero nor memset are available"
#endif

#if HAVE_BCMP
#  define       BCMP(a,b,c)             bcmp(a,b,c)  
#elif HAVE_MEMCMP
#  define       BCMP(a,b,c)             memcmp(a,b,c)
#else 
#  error "Neither bcmp nor memcmp are available"
#endif

#define GETPEERNAME(a,b,c)	getpeername(a, (struct sockaddr *)b, (int *)c)
#define GETSOCKNAME(a,b,c)	getsockname(a, (struct sockaddr *)b, (int *)c)
#define RECVFROM(a,b,c,d,e,f)	recvfrom(a, (void *)b, (size_t)c, d, (struct sockaddr *)e, (int *)f)
#define SENDTO(a,b,c,d,e,f)	sendto(a, (void *)b, (size_t)c, d, (struct sockaddr *)e, f)

/*
 * Define the logging priorities. If syslog(3) is present, include
 * <syslog.h> and duplicate values, otherwise define them ourselves.
 */

#if HAVE_SYSLOG_H
  #include <syslog.h>
  #define	VLOG_EMERG	LOG_EMERG
  #define	VLOG_ALERT	LOG_ALERT
  #define	VLOG_CRIT	LOG_CRIT
  #define	VLOG_ERR	LOG_ERR
  #define	VLOG_WARNING	LOG_WARNING
  #define	VLOG_NOTICE	LOG_NOTICE
  #define	VLOG_INFO	LOG_INFO
  #define	VLOG_DEBUG	LOG_DEBUG
#else
  #define	VLOG_EMERG	0
  #define	VLOG_ALERT	1
  #define	VLOG_CRIT	2
  #define	VLOG_ERR	3
  #define	VLOG_WARNING	4
  #define	VLOG_NOTICE	5
  #define	VLOG_INFO	6
  #define	VLOG_DEBUG	7
#endif

/*
 * Windows Service name
 */

#ifdef WIN32
#define SZSERVICENAME		"VChatServer"	/* Windows Service name */
#endif

/*
 * Macro for accessing client data records
 */

#define	cl_addr(x)		(cl_buf + x)	/* Macro for client data address */

/*********************************************
 * Parameters to the kill_by_flag() function *
 *********************************************/

#define		FLG_SET		0x01
#define		FLG_NOTSET	0x02

/*************************************
 * Macro for accessing topic entries *
 *************************************/

#define		TOPADDR(i)	(struct chantopic *)(vtop.bufaddr + (i * sizeof(struct chantopic)))

/*********************************
 * define INADDR_NONE if missing *
 *********************************/

#ifndef INADDR_NONE
#  define	INADDR_NONE	0xffffffff
#endif

/********************
 * DNS check levels *
 ********************/

#define DNS_CHK_NONE		0x00
#define	DNS_CHK_FWD		0x01
#define	DNS_CHK_REV		0x02
#define	DNS_CHK_STRICT		0x03

/***************************************
 * Server configuration data structure *
 ***************************************/

typedef struct
{
	u_char		auth_file[_POSIX_PATH_MAX];
	u_char		acl_file[_POSIX_PATH_MAX];
#ifndef WIN32
	u_char		ctl_file[_POSIX_PATH_MAX];
#endif
	u_char		log_file[_POSIX_PATH_MAX];
	u_char		mail_file[_POSIX_PATH_MAX];
	u_char		pid_file[_POSIX_PATH_MAX];
	u_char		llog_file[_POSIX_PATH_MAX];
	u_char		servername[_POSIX_PATH_MAX];
	struct in_addr	bind_addr;	/* Interface address to bind to  */
	u_long		port;		/* TCP service port              */
	u_long		max_pending;	/* Max. # of pending requests    */
	u_long		keep_logfiles;	/* Number of logfile generations */
	u_long		llog_entries;	/* Number of lastlog entries     */
#define	VLOG_NONE	0x0000		/* No server log                 */
#define	VLOG_FILE	0x0001		/* Server log to logfile         */
#define	VLOG_SYSLOG	0x0002		/* Server log to syslog process  */
	u_short		logswitch;
#define	CF_DETACH	0x00000001	/* Detach from terminal on start */
#define	CF_SUPPRESS_IP	0x00000002	/* Suppress display of client IP */
#define	CF_ENABLE_MAIL	0x00000004	/* Enable e-mail invitation	 */
#define	CF_REG_ONLY	0x00000008	/* Reject unregistered nicknames */
#define CF_FLOOD_GUARD	0x00000010	/* For time based flood protect  */
	u_long		flags;
	u_char		dnschklvl;	/* Client DNS checking level     */

} CFDATA;

/***********************************
 * Server last log data structures *
 ***********************************/

typedef struct
{
	int32_t		first_record;
	int32_t		next_record;
	u_int32_t	used_records;
	u_int32_t	total_records;
	u_int8_t	padding[240];
} LLOGHEAD;

typedef struct
{
	time_t		signon_stamp;
	time_t		signoff_stamp;
	u_char		username[VPUSERSIZE+1];
	u_char		nickname[VPNICKSIZE+1];
	u_char		info[40];
	struct in_addr	ipaddr;
	u_char		hostname[64];
	u_char		padding[108];	/* Page aligned! */

} LLOGDAT;

#define	LLOG_HEADER	(LLOGHEAD *)llog_base
#define LLOG_RECORD(i)	(LLOGDAT *)((u_int)llog_base + sizeof(LLOGHEAD) + (i * sizeof(LLOGDAT)))
#define	LLOG_SIZE	((cfdata.llog_entries * sizeof(LLOGDAT)) + sizeof(LLOGHEAD))

/************************************
 * Server request counter structure *
 ************************************/

typedef struct
{
	time_t	vs_start;		/* Time server was started at.	*/

	u_long	vs_total_conn;		/* Total #conns  since startup	*/
	u_long	vs_active_conn;		/* Currently active conns	*/
	u_long	vs_err_conn;		/* #conns aborted due to error	*/

	u_long	vs_requests_processed;	/* Requests serviced since start*/
	u_long	vs_requests_unknown;	/* Unknown requests serviced	*/

	u_long	vs_requests_action;	/* action requests serviced	*/
	u_long	vs_requests_asstopic;	/* topic change requests servcd.*/
	u_long	vs_requests_chanlist;	/* chanlist requests serviced.	*/
	u_long	vs_requests_chchan;	/* chchan requests serviced	*/
	u_long	vs_requests_chnick;	/* chnick requests serviced	*/
	u_long	vs_requests_chtopic;	/* chtopic requests serviced	*/
	u_long	vs_requests_clientinfo;	/* clientinfo requests serviced */
	u_long	vs_requests_invite;	/* invite requests serviced	*/
	u_long	vs_requests_page;	/* page requests serviced	*/
	u_long	vs_requests_prvmsg;	/* prvmsg requests serviced	*/
	u_long	vs_requests_pubmsg;	/* pubmsg requests serviced	*/
	u_long	vs_requests_rtcmp;	/* rtcmp requests serviced	*/
	u_long	vs_requests_signon;	/* signon requests serviced	*/
	u_long	vs_requests_signoff;	/* signoff requests serviced	*/
	u_long	vs_requests_snd_id;	/* snd_id requests serviced	*/
	u_long	vs_requests_sv_stat;	/* sv_stat requests serviced	*/
	u_long	vs_requests_ulist;	/* ulist requests serviced	*/
	u_long	vs_requests_wakeup;	/* wakeup requests serviced	*/
	u_long	vs_requests_signmsg;	/* signmsg requests serviced	*/
	u_long	vs_requests_chpasswd;	/* chpasswd requests serviced	*/
	u_long	vs_requests_setpasswd;	/* setpasswd requests serviced	*/
	u_long	vs_requests_addnick;	/* addnick requests serviced	*/
	u_long	vs_requests_delnick;	/* delnick requests serviced	*/
	u_long	vs_requests_chmodperm;	/* chmodperm requests serviced	*/
	u_long	vs_requests_chaccperm;	/* chaccperm requests serviced	*/
	u_long	vs_requests_kill;	/* kill requests serviced	*/
	u_long	vs_requests_broadcast;	/* broadcast requests serviced	*/
	u_long	vs_requests_acl_add;	/* ACL add requests serviced	*/
	u_long	vs_requests_acl_delete;	/* ACL delete requests serviced	*/
	u_long	vs_requests_acl_flush;	/* ACL flush requests serviced	*/
	u_long	vs_requests_acl_list;	/* ACL list requests serviced	*/
	u_long	vs_requests_query;	/* query requests serviced	*/
	u_long	vs_requests_lastlog;	/* lastlog requests serviced	*/
	u_long	vs_requests_svopt;	/* svoptr requests serviced	*/
	u_long	vs_requests_clopt;	/* clopt requests serviced	*/
	u_long	vs_requests_fixchan;	/* fixchan requests serviced	*/
	u_long	vs_requests_svdnsopt;	/* svdnsopt requests serviced	*/
	u_long	vs_requests_exclude;	/* exclude requests serviced	*/

	double	vs_loadavg[3];		/* Load averages of server	*/

} VS_STAT;

typedef struct
{
	char	*bufaddr;
	u_long	entries;

} VTOPIC;

typedef struct
{
	u_char	info[80];
	u_long	info_ok;

} OSINFO;

/*
 * CLDATA:
 * =======
 *
 * Client data structure. The server keeps this data
 * for every connected client process.
 */

typedef struct
{
	int		fd;			/* Descriptor of I/O socket  */
	int		error;			/* Last errno if CFLAG_ERROR */
	u_long		type;			/* Type of client software   */
	u_long		protocol;		/* Client protocol revision  */
	u_long		flags;			/* Client flags              */
#define	CFLAG_REGISTERED	0x10000000	/* Nickname is registered    */
#define	CFLAG_SIGNOFF		0x20000000	/* Client is about signoff   */
#define	CFLAG_ERROR		0x40000000	/* Client had error          */
#define	CFLAG_KILL		0x80000000	/* Client has been killed    */
#define	CFLAG_EXCLUDE		0x01000000	/* Client has been excluded  */

	u_int32_t	cl_opts;		/* Client option flags       */
	u_long		accperm;		/* Access permission bits    */
	u_long		modperm;		/* CHMOD permission bits     */
	u_long		pubmsgcnt;		/* Public message counter    */
	u_long		prvmsgcnt;		/* Private message counter   */
	signed long	channel;		/* Current channel number    */
	time_t		login;			/* Login timestamp           */
	time_t		stamp;			/* Last activity timestamp   */
	struct in_addr	ip_addr;		/* Client IP address         */
	struct in_addr	gw_addr;		/* Gateway address (if any)  */
	struct in_addr	if_addr;		/* Server interface address  */
	u_char		host[128];		/* The client's hostname     */
	u_char		gw_name[128];		/* Gateway name (if any)     */
	u_char		if_name[128];		/* Server interface hostname */
	u_char		user[VPUSERSIZE+1];	/* Username if FRONTEND      */
	u_char		nick[VPNICKSIZE+1];	/* Current nickname          */
	u_char		ident[VPIDENTSIZE+1];	/* Client identification     */
	u_char		killinfo[VPNICKSIZE+1]; /* Nickname of killing user  */
	u_char		offmsg[64];		/* Signoff message if any    */
	struct timeval	lastsend;		/* Time of last packet sent  */
	char		dnschklvl;		/* DNS check level           */

} CLDATA;

/*******************************
 * ChannelTopic data structure *
 *******************************/

struct chantopic
{
	signed long	channel;
	u_char		topic[VPTOPICSIZE+1];
#define	CHFLG_REMOVABLE	0x02
#define	CHFLG_FIXED	0x01
	u_long		flags;
	time_t		stamp;
};

/************************
 * Table data structure *
 ************************/

#define TABLE_SIZE	2048
#define TABLE_TRESHOLD	(7 * (TABLE_SIZE / 8))

typedef struct
{
	CLDATA	*client;
	VMSG	svcmd;
	u_char	buf[TABLE_SIZE];

} TABLE;

#ifdef WIN32

typedef struct
{
	int	fd;
	void	*buf;
	size_t	len;

} IO_OP;

#endif

#endif /* __VSERVER_H__ */
