/* $Id: os_info.c,v 1.2 2001/09/30 18:04:51 mickey Exp $ */

#include <vserver.h>
#include <proto.h>
#include <extern.h>

#if HAVE_SYS_PARAM_H
  #include <sys/param.h>
#endif

#if (SI_TYPE == SI_SYSCTL)
  #include <sys/sysctl.h>
#elif (SI_TYPE == SI_SYSINFO)
  #include <sys/systeminfo.h>
#elif (SI_TYPE == SI_UNAME)
  #include <sys/utsname.h>
#else
  #ifndef WIN32
    #warning "unable to determine how to get system information."
  #endif
#endif

/*** Code ***/

OSINFO *get_os_info(void)
{
	static OSINFO os;

#if (SI_TYPE == SI_SYSCTL)

	#define SYBUFSIZE 24

	u_char os_type[SYBUFSIZE];
	u_char os_release[SYBUFSIZE];
	u_char hw_model[SYBUFSIZE];
	int ncpu;

	size_t len;
	int mib[2];

	os.info[0] = '\0';

	/*** Get OS type ***/

	mib[0] = CTL_KERN;
	mib[1] = KERN_OSTYPE;

	len = SYBUFSIZE;

	sysctl(mib, 2, os_type, &len, NULL, 0);

	/*** Get OS release ***/

	mib[0] = CTL_KERN;
	mib[1] = KERN_OSRELEASE;

	len = SYBUFSIZE;

	sysctl(mib, 2, &os_release, &len, NULL, 0);

	/*** Get hardware model ***/

	mib[0] = CTL_HW;
	mib[1] = HW_MODEL;

	len = SYBUFSIZE;

	sysctl(mib, 2, &hw_model, &len, NULL, 0);

	/**** Get number of CPUs installed ***/

	mib[0] = CTL_HW;
	mib[1] = HW_NCPU;

	len = sizeof(ncpu);

	sysctl(mib, 2, &ncpu, &len, NULL, 0);

	/*
	 * Create info
	 */

	if(ncpu <= 1)
		sprintf(os.info, "%s %s %s %d CPU", os_type, os_release, hw_model, ncpu);
	else
		sprintf(os.info, "%s %s %s %d CPUs", os_type, os_release, hw_model, ncpu);

	os.info_ok = 0xff;

#elif (SI_TYPE == SI_SYSINFO)

	#define SIBUFSIZE 80

	u_char sysname[SIBUFSIZE];
	u_char hostname[SIBUFSIZE];
	u_char release[SIBUFSIZE];
	u_char version[SIBUFSIZE];
	u_char machine[SIBUFSIZE];
	u_char architecture[SIBUFSIZE];

	if(sysinfo(SI_SYSNAME, sysname, SIBUFSIZE) == -1
	|| sysinfo(SI_HOSTNAME, hostname, SIBUFSIZE) == -1
	|| sysinfo(SI_RELEASE, release, SIBUFSIZE) == -1
	|| sysinfo(SI_VERSION, version, SIBUFSIZE) == -1
	|| sysinfo(SI_MACHINE, machine, SIBUFSIZE) == -1
	|| sysinfo(SI_ARCHITECTURE, architecture, SIBUFSIZE) == -1)
	{
		os.info_ok = 0;
	}
	else
	{
		sprintf(os.info, "%s %s %s %s %s %s",
			sysname, hostname, release, version,
			machine, architecture);

		os.info_ok = 0xff;
	}

#elif (SI_TYPE == SI_UNAME)

	struct utsname uts;

	if(uname(&uts))
	{
		os.info_ok = 0;
	}
	else
	{
		sprintf(os.info, "%s %s %s", uts.sysname,
			uts.release, uts.machine);

		os.info_ok = 0xff;
	}
#else
  #ifdef WIN32
	HKEY	hKey;
	DWORD	dwValIdx	= 0;
	TCHAR	valname[MAX_PATH];
	DWORD	valnamesize	= MAX_PATH;
	BYTE	valbuf[MAX_PATH];
	DWORD	valbufsize	= MAX_PATH;
	DWORD	valtype;

	char	osname[32]	= "WindowsNT";
	char	osrev[6]	= "???";
	char	svcpack[32]	= "\0";
	char	buildnum[10]	= "???";

#define	INFO_KEY1	"SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion"
#define	INFO_KEY2	"SYSTEM\\CurrentControlSet\\Control\\ProductOptions"

	/*
	 * Open the first registry key that contains
	 * some of the infos we want to know.
	 */

	if(RegOpenKeyEx(HKEY_LOCAL_MACHINE,INFO_KEY1,0,KEY_QUERY_VALUE,&hKey))
	{
		vlog(VLOG_ERR, "ERROR: Failed to open registry key %s", INFO_KEY1);
		goto regfail1;
	}

	/*
	 * Step through all values for this key and
	 * watch out for the ones we want to know.
	 */

	while(!RegEnumValue(hKey, dwValIdx, valname, &valnamesize, NULL,
		&valtype, valbuf, &valbufsize))
	{
		if(!strcmp(valname, "CSDVersion")
		&& (valtype == REG_EXPAND_SZ || valtype == REG_SZ))
		{
			BZERO(&svcpack, sizeof(svcpack));
			strncpy(svcpack, (char *)valbuf, sizeof(svcpack)-1);
		}
		else if(!strcmp(valname, "CurrentBuildNumber")
		&& (valtype == REG_EXPAND_SZ || valtype == REG_SZ))
		{
			BZERO(&buildnum, sizeof(buildnum));
			strncpy(buildnum, (char *)valbuf, sizeof(buildnum)-1);
		}
		else if(!strcmp(valname, "CurrentVersion")
		&& (valtype == REG_EXPAND_SZ || valtype == REG_SZ))
		{
			BZERO(&osrev, sizeof(osrev));
			strncpy(osrev, (char *)valbuf, sizeof(osrev)-1);
		}

		valnamesize = valbufsize = MAX_PATH;
		++dwValIdx;
	}

	/*
	 * Close the first registry key.
	 */

	RegCloseKey(hKey);

regfail1:

	/*
	 * Open the second registry key that contains
	 * some other infos we want to know.
	 */

	if(RegOpenKeyEx(HKEY_LOCAL_MACHINE,INFO_KEY2,0,KEY_QUERY_VALUE,&hKey))
	{
		vlog(VLOG_ERR, "Failed to open registry key %s", INFO_KEY2);
		goto regfail2;
	}

	/*
	 * Step through all values for this key and
	 * watch out for the ones we want to know.
	 */

	valnamesize = valbufsize = MAX_PATH;
	dwValIdx = 0;

	while(!RegEnumValue(hKey, dwValIdx, valname, &valnamesize, NULL,
		&valtype, valbuf, &valbufsize))
	{
		if(!strcmp(valname, "ProductType")
		&& (valtype == REG_EXPAND_SZ || valtype == REG_SZ))
		{
			BZERO(&osname, sizeof(osname));
			strncpy(osname, (char *)valbuf, sizeof(osname)-1);
		}

		valnamesize = valbufsize = MAX_PATH;
		++dwValIdx;
	}

	/*
	 * Close the key.
	 */

	RegCloseKey(hKey);

regfail2:

	sprintf(os.info, "%s %s Build: %s %s", osname, osrev, buildnum, svcpack);
	os.info_ok = 0xff;
  #else
	os.info_ok = 0;
        #warning os-info not supported on this system
  #endif
#endif
	return(&os);	
}
