/* $Id: serv_subr.c,v 1.3 2002/06/23 23:16:06 mickey Exp $ */

#include <vserver.h>
#include <ascii.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>

#include <stdlib.h>
#include <errno.h>
#include <signal.h>
#include <ctype.h>
#include <sys/stat.h>

#ifdef WIN32
  #include <windows.h>
  #include <process.h>
  #include <winsock.h>
#else
  #include <netdb.h> 
  #include <unistd.h>
  #include <sys/socket.h>
  #include <netinet/in.h>
  #include <arpa/inet.h>
#endif

/***** Externals *****/

extern int optind;
extern int optopt;
extern int opterr;
extern char *optarg;

#ifndef WIN32

/*
 * int process_cmdline(int argc, char *argv[]);
 *
 * Process all arguments specified on the commandline
 * and setup program defaults accordingly.
 */

int process_cmdline(int argc, char *argv[])
{
	char	*optstr;
	int	opt;

	/*
	 * Process commandline arguments
	 */
	
#if HAVE_SYSLOG
  #if HAVE_DAEMON
	optstr = ":a:A:C:dD:fk:lL:mM:n:No:p:w:W:P:R:sST:Z:";
  #else
	optstr = ":a:A:C:D:fk:lL:mM:n:No:p:w:W:P:R:sST:Z:";
  #endif
#else
  #if HAVE_DAEMON
	optstr = ":a:A:C:dD:fk:lL:mM:n:No:p:w:W:P:R:ST:Z:";
  #else
	optstr = ":a:A:C:D:fk:lL:mM:n:No:p:w:W:P:R:ST:Z:";
  #endif
#endif

	while((opt = getopt(argc, argv, optstr)) != -1)
	{
		switch(opt)
		{
			case 'a':	{
						struct hostent *ht;

						if((ht = gethostbyname(optarg)) == NULL)
						{
#if HAVE_HERROR
							herror(optarg);
#else
							fprintf(stderr, "%s: hostname lookup failure\n", optarg);
#endif
							return -1;
						}
						else
						{
							BCOPY(ht->h_addr, &cfdata.bind_addr, ht->h_length);
						}
					}
					break;
			case 'A':	BZERO(&cfdata.acl_file, sizeof(cfdata.acl_file));
					strncpy(cfdata.acl_file, optarg, sizeof(cfdata.acl_file)-1);
					break;
#ifndef WIN32
			case 'C':	BZERO(&cfdata.ctl_file, sizeof(cfdata.ctl_file));
					strncpy(cfdata.ctl_file, optarg, sizeof(cfdata.ctl_file)-1);
					break;
#endif
#if HAVE_DAEMON
			case 'd':	cfdata.flags |= CF_DETACH;
					break;
#endif
			case 'D':	if(!strcasecmp(optarg, "fwd"))
					{
						cfdata.dnschklvl = DNS_CHK_FWD;
					}
					else if(!strcasecmp(optarg, "rev"))
					{
						cfdata.dnschklvl = DNS_CHK_REV;
					}
					else if(!strcasecmp(optarg, "strict"))
					{
						cfdata.dnschklvl = DNS_CHK_STRICT;
					}
					else
					{
						usage("Argument '%s' invalid with option -D. Valid arguments are <fwd|rev|strict>.", optarg);
						return -1;
					}
					break;
			case 'f':	cfdata.flags |= CF_FLOOD_GUARD;
					break;
			case 'k':	cfdata.keep_logfiles = atoi(optarg);
					break;
			case 'l':	cfdata.logswitch = VLOG_FILE;
					break;
			case 'L':	BZERO(&cfdata.log_file, sizeof(cfdata.log_file));
					strncpy(cfdata.log_file, optarg, sizeof(cfdata.log_file)-1);
					break;
			case 'm':	cfdata.flags |= CF_ENABLE_MAIL;
					break;
			case 'M':	BZERO(&cfdata.mail_file, sizeof(cfdata.mail_file));
					strncpy(cfdata.mail_file, optarg, sizeof(cfdata.mail_file)-1);
					break;
			case 'n':	BZERO(&cfdata.servername, sizeof(cfdata.servername));
					strncpy(cfdata.servername, optarg, sizeof(cfdata.servername)-1);
					break;
			case 'N':	cfdata.flags |= CF_REG_ONLY;
					break;
			case 'o':	cfdata.max_pending = atoi(optarg);
					break;
			case 'p':	cfdata.port = htons((u_short)atoi(optarg));
					break;
			case 'P':	BZERO(&cfdata.pid_file, sizeof(cfdata.pid_file));
					strncpy(cfdata.pid_file, optarg, sizeof(cfdata.pid_file)-1);
					break;
			case 'R':	auth_reset(optarg);
					return -1;
#if HAVE_SYSLOG
			case 's':	cfdata.logswitch = VLOG_SYSLOG;
					break;
#endif
			case 'S':	cfdata.flags |= CF_SUPPRESS_IP;
					break;
			case 'T':	if(fix_topic(optarg))
					{
						usage("The -T option requires an argument of the form <channel:topic>.");
						return -1;
					}
					break;
			case 'w':	cfdata.llog_entries = strtol(optarg, NULL, 0);
					if(cfdata.llog_entries < 10)
						cfdata.llog_entries = 10;
					else if(cfdata.llog_entries > 6000)
						cfdata.llog_entries = 6000;
					break;
			case 'W':	BZERO(&cfdata.llog_file, sizeof(cfdata.llog_file));
					strncpy(cfdata.llog_file, optarg, sizeof(cfdata.llog_file)-1);
					break;
			case 'Z':	BZERO(&cfdata.auth_file, sizeof(cfdata.auth_file));
					strncpy(cfdata.auth_file, optarg, sizeof(cfdata.auth_file)-1);
					break;
			case ':':	usage("Option -%c requires an argument.", optopt);
					return -1;
			case '?':	usage("Unrecognized option: -%c", optopt);
					return -1;
		}
	}

	return 0;
}

#endif /* !WIN32 */

/*
 * int init_global(void);
 *
 * Main server initialization function.
 */

int init_global(void)
{
	FILE			*pidf;
	int			on		= 1;

#ifdef WIN32
	WSADATA			wsaData;
	WORD			version_requested;
#else
  #if HAVE_SIGACTION
	struct sigaction	sa;
	struct sigaction	sh;
	struct sigaction	sl;

	sa.sa_handler = (void *)pipesig;
	sh.sa_handler = (void *)reread_config;
	sl.sa_handler = (void *)rotate_logfile;

	sigemptyset(&sa.sa_mask);
	sigemptyset(&sh.sa_mask);
	sigemptyset(&sl.sa_mask);

    #ifdef SA_RESTART
	sa.sa_flags = SA_RESTART;
	sh.sa_flags = SA_RESTART;
	sl.sa_flags = SA_RESTART;
    #else
	sa.sa_flags = 0;
	sh.sa_flags = 0;
	sl.sa_flags = 0;
    #endif
  #else
	struct sigvec sv;
	struct sigvec sh;
	struct sigvec sl;

	sv.sv_handler = (void *)pipesig;
	sh.sv_handler = (void *)reread_config;
	sl.sv_handler = (void *)rotate_logfile;

	sv.sv_mask = sv.sv_flags = 0;
	sh.sv_mask = sh.sv_flags = 0;
	sl.sv_mask = sl.sv_flags = 0;
  #endif
#endif /* !WIN32 */

	/*
	 * Init logfile
	 */

	init_logfile();

	/*
	 * Init last log file
	 */

	init_lastlog();

	/*
	 * Initialize vserver status structure
	 */

	vs_init();

	/*
	 * Initialize access control data
	 */

	if(cfdata.acl_file[0] != '\0')
		acl_load_file(cfdata.acl_file);

	acl_log_status();

	/*
	 * Initialize authority record database
	 */

	if(auth_load() == -1)
	{
		vlog(VLOG_ERR, "FATAL: Authority database initialization failure");
		return -1;
	}

#ifndef WIN32 /* -------------- UNIX --------------- */

	/*
	 * Initialize server control socket
	 */

	if(init_ctlsock() == -1)
	{
		vlog(VLOG_CRIT, "FATAL: Failed to initialize server control socket");
		return -1;
	}

	/*
	 * Call reread_config() each time a SIGHUP is received
	 */

  #if HAVE_SIGACTION
	sigaction(SIGHUP, &sh, NULL);
  #else
	sigvec(SIGHUP, &sh, NULL);
  #endif

	/*
	 * Rotate logfiles upon receipt of SIGUSR1
	 */

  #if HAVE_SIGACTION
	sigaction(SIGUSR1, &sl, NULL);
  #else
	sigvec(SIGUSR1, &sl, NULL);
  #endif

#else /* --------- WIN32 --------- */

	/*
	 * Initialize WINSOCK
	 */

	version_requested = MAKEWORD(2, 0);

	while(WSAStartup(version_requested, &wsaData) != 0)
	{
		vlog(VLOG_CRIT, "FATAL: Unable to find a suitable winsock version");
		return -1;
	}

	wsa_is_init = 1;

#endif /* ------------- !WIN32 ----------- */

	/*
	 * Prepare and open server socket
	 */

#ifdef WIN32
	if((sock = socket(AF_INET, SOCK_STREAM, 0)) == INVALID_SOCKET)
	{
		int errcod = WSAGetLastError();

		vlog(VLOG_CRIT, "FATAL: socket() returned status %d (%s)",
			errcod, strerror(errcod));
		return -1;
	}
#else
	if((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1)
	{
		vlog(VLOG_CRIT, "FATAL: socket() returned status %d (%s)",
			errno, strerror(errno));
		return -1;
	}
#endif

#ifdef WIN32
	if(setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char *)&on, sizeof(on)) == -1)
#else
	if(setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (void *)&on, sizeof(on)) == -1)
#endif
	{
		vlog(VLOG_CRIT, "FATAL: setsockopt() returned status %d (%s)",
			errno, strerror(errno));
#ifdef WIN32
		closesocket(sock);
#else
		close(sock);
#endif
		return -1;
	}

	/*
	 * Bind socket
	 */

	BZERO((char *)&sock_in, sizeof(sock_in));

	sock_in.sin_family = AF_INET;
	sock_in.sin_port = (u_short)cfdata.port;
	sock_in.sin_addr = cfdata.bind_addr;

	if(bind(sock, (struct sockaddr *)&sock_in, sizeof(sock_in)) != 0)
	{
		vlog(VLOG_CRIT, "FATAL: bind() returned status %d (%s)",
			errno, strerror(errno));
#ifdef WIN32
		closesocket(sock);
#else
		close(sock);
#endif
		return -1;
	}

	/*
	 * Handle SIGPIPE
	 */

#ifndef WIN32
  #if HAVE_SIGACTION
	sigaction(SIGPIPE, &sa, NULL);
  #else
	sigvec(SIGPIPE, &sv, NULL);
  #endif
#endif

	/*
	 * Listen for incoming connections.
	 */

	if(listen(sock, cfdata.max_pending))
	{
		vlog(VLOG_CRIT, "FATAL: listen() returned status %d (%s)",
			errno, strerror(errno));
#ifdef WIN32
		closesocket(sock);
#else
		close(sock);
#endif
		return -1;
	}

	vlog(VLOG_INFO, "Listening for connections on interface %s:%d",
		inet_ntoa(cfdata.bind_addr), ntohs((u_short)cfdata.port));

	if(cfdata.pid_file[0] != '\0')
	{
		if((pidf = fopen(cfdata.pid_file, "w")) != NULL)
		{
#ifdef WIN32
			fprintf(pidf, "%d\n", _getpid());
#else
			fprintf(pidf, "%ld\n", (u_long)getpid());
#endif
			fclose(pidf);
		}
	}

	vlog(VLOG_INFO, "VChat conference server %s protocol %x-%x pid %d -- ready", VERSION,
		MAJOR_REV(PROTO_REV), MINOR_REV(PROTO_REV), getpid());

	return 0;
}

/*
 * void exit_global(void);
 *
 * Main server cleanup function
 */

void exit_global(void)
{
	exit_ctlsock();
	exit_lastlog();
	exit_logfile();
}

/*
 * u_long chatters(signed long chan);
 *
 * Computes the number of users on a conference channel.
 */

u_long chatters(signed long chan)
{
	register CLDATA	*cl;
	register int	i;
	register u_long	n	= 0;

	for(i = 0; i < clients; i++)
	{
		cl = cl_addr(i);

		if(cl->channel == chan)
			n++;
	}

	return(n);
}

/*
 * int nick_ok(u_char *nick);
 *
 * Check if the given nickname contains any invalid
 * characters. Return 0 if the specified nickname is
 * invalid, or 1 if the nickname is ok to use.
 */

int nick_ok(u_char *nick)
{
	register u_char	*nkptr;

	/*
	 * Check nickname character validity
	 */

	for(nkptr = nick; *nkptr != '\0'; nkptr++)
	{
		if(strchr("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789_.@", *nkptr) == NULL)
			return 0;
	}

	return(1);
}

/*
 * int nick_dup(u_char *nick);
 *
 * Check if the given nickname is already signed on.
 * Returns 1 if the given nickname is already assigned
 * or 0 if the nickname was not found and is ok to use.
 */

int nick_dup(u_char *nick)
{
	register int	i;
	register CLDATA	*tmp;
	
	for(i = 0; i < clients; i++)
	{
		tmp = cl_addr(i);

		if(!strncmp(tmp->nick, nick, VPNICKSIZE))
			return(1);
	}

	return 0;
}

/*
 * char *iptostr(struct in_addr ip);
 *
 * Return a string representation for the internet address <ip>
 * that is useable multiple times in printf style function calls
 * without clobbering the buffer of each previous call.
 */

char *iptostr(struct in_addr ip)
{
	#define SIZE 128

	static char	namebuf[1024];
	static char	*bufptr;
	static int	index		= 0;

	if(++index > 7)
		index = 0;

	bufptr = (char *)&namebuf[(index * SIZE)];

	sprintf(bufptr, "%s", inet_ntoa(ip));

	return(bufptr);
}

/*
 * char *iptoname(struct in_addr ip);
 *
 * Return the DNS name for the host specified by <ip>.
 * If the DNS name cannot be found, the ASCII represantation of the
 * internet address contained in <ip> is returned.
 */

char *iptoname(struct in_addr ip)
{
	static char namebuf[1024];
	static char *bufptr;
	static int index = 0;
	struct hostent *ht;

	if(++index > 7)
		index = 0;

	bufptr = (char *) &namebuf[(index * SIZE)];

	if((ht = gethostbyaddr((char *)&ip, sizeof(ip), AF_INET)) == NULL)
	{
		sprintf(bufptr, "%s", inet_ntoa(ip));
	}
	else
	{
		strcpy(bufptr, ht->h_name);
	}

	return(bufptr);
}

/*
 * char *dnsname(struct in_addr ip);
 *
 * Return the DNS name for the host specified by <ip>
 * or NULL if no matching DNS record could be found.
 */

char *dnsname(struct in_addr ip)
{
	static char namebuf[1024];
	static char *bufptr;
	static int index = 0;
	struct hostent *ht;

	if((ht = gethostbyaddr((char *)&ip, sizeof(ip), AF_INET)) == NULL)
		return(NULL);

	if(++index > 7)
		index = 0;

	bufptr = (char *) &namebuf[(index * SIZE)];

	strcpy(bufptr, ht->h_name);

	return(bufptr);
}

/*
 * str_lower(char *str);
 *
 * Convert the string pointed to by <str> into lower case.
 */

void str_lower(register char *str)
{
	while(*str != '\0')
	{
		*str = tolower(*str);
		++str;
	}
}

/*
 * void vs_init(void);
 *
 * Initialize the server request counters.
 */

void vs_init(void)
{
	int i;

	time(&vs.vs_start);

	vs.vs_total_conn = vs.vs_active_conn = vs.vs_err_conn = 0;

	for(i = 0; i < 3; i++)
		vs.vs_loadavg[i] = 0;
}

/*
 * VIDENT *ident(u_long type);
 *
 * Create the server's VIDENT packet with the type field set to <type>.
 */

VIDENT *ident(u_long type)
{
	static VIDENT vid;

	vid.magic = htonl(VSERVER_MAGIC);
	vid.proto = htonl(PROTO_REV);
	vid.type  = htonl(type);
	sprintf(vid.ident, "VCS-%s-%s", OS_TYPE, VERSION);

	return(&vid);
}

/*
 * int fix_topic(char *arg);
 *
 * Create a fixed conference channel from an ASCII
 * argument of the form "channel:topic".
 */

int fix_topic(char *arg)
{
	char buf[256];
	char *top;

	signed long channel;

	strncpy(buf, arg, 255);
	buf[255] = '\0';

	if((top = (char *)strchr(buf, ':')) == NULL)
		return -1;

	*top++ = '\0';

	channel = strtol(buf, NULL, 0);

	store_topic(channel, top, CHFLG_FIXED);
	touch_channel(channel);

	return 0;
}

/*
 * void reread_config(void);
 *
 * This is called upon reception of a SIGHUP signal
 * in order to have the server reload all config files.
 */

void reread_config(void)
{
	vlog(VLOG_NOTICE, "SIGHUP received - rereading configuration files");

	if(cfdata.acl_file[0] != '\0')
		acl_load_file(cfdata.acl_file);

	acl_log_status();
}

/*
 * void rotate_logfile(void);
 *
 * Rotate the logfile upon receipt of a SIGUSR1 signal.
 */

void rotate_logfile(void)
{
	register long	i;
	char		tmp1[4096];
	char		tmp2[4096];
	struct stat	st;

	/*
	 * Do nothing if logging disabled or in syslog mode.
	 */

	if(cfdata.logswitch != VLOG_FILE || lgf == NULL || cfdata.log_file[0] == '\0')
		return;

	/*
	 * Close logfile
	 */

	fclose(lgf);

	/*
	 * Shift logfile generations
	 */

	for(i = cfdata.keep_logfiles ; i >= 0 ; --i)
	{
		sprintf(tmp1, "%s.%ld", cfdata.log_file, i);
#ifndef WIN32
		if(lstat(tmp1, &st) == -1)
#else
		if(stat(tmp1, &st) == -1)
#endif
		{
			continue;
		}

		if(i == cfdata.keep_logfiles)
		{
			/*
			 * This file has hit the limit!
			 * -delete- it!
			 */

			unlink(tmp1);
			continue;
		}

		/*
		 * Shift file up one number
		 */

		sprintf(tmp2, "%s.%ld", cfdata.log_file, i+1);

		rename(tmp1, tmp2);
	}

	/*
	 * Rename current logfile to logfile.0
	 */

	sprintf(tmp2, "%s.0", cfdata.log_file);
	rename(cfdata.log_file, tmp2);

	/*
	 * Open new logfile
	 */

	if((lgf = fopen(cfdata.log_file, "a")) == NULL)
	{
		fprintf(stderr, "%s: unable to open logfile '%s' for writing.\n",  
			prog_name, cfdata.log_file);
		fprintf(stderr, "%s: logging disabled\n\n", prog_name);
			cfdata.logswitch = 0;
	}
	else
	{
		setvbuf(lgf, NULL, _IONBF, 0);
		vlog(VLOG_INFO, "Logfile started");
	}
}

/*
 * void pipesig(void);
 *
 * Catch a SIGPIPE signal and log the condition.
 */

void pipesig(void)
{
	vlog(VLOG_WARNING, "SIGPIPE received - ignored");
}

/*
 * int unescape_str(u_char *str, u_char *buf, size_t len);
 *
 * Read input from the string pointed to by <str> unescaping any
 * escape sequences beginning with the CSI indicator. At most <len>
 * bytes of output will be written to the buffer pointed to by <buf>.
 * The return value is the number of bytes placed into the buffer <buf>.
 */

int unescape_str(u_char *str, u_char *buf, size_t len)
{
	size_t	cnt	= 0;

	while(str != NULL && *str != '\0' && cnt < len-1)
	{
		if(*str == CSI)
		{
			++str;

			switch(*str)
			{
				case CSI:	*buf++ = CSI;
						++cnt;
						++str;
						break;

				case ' ':	*buf++ = ' ';
						++cnt;
						++str;
						break;

				case '\0':	break;

				default:	++str;
						break;
			}
		}
		else
		{
			*buf++ = *str++;
			++cnt;
		}
	}

	*buf = '\0';

	return(cnt);
}
