/* $Id: usage.c,v 1.3 2002/06/23 23:16:07 mickey Exp $ */

#include <vserver.h>
#include <proto.h>
#include <extern.h>

#include <stdarg.h>

/*
 * void usage(char *fmt, ...);
 *
 * Display the program usage information along with the error
 * string specified by the format string <fmt> and its arguments.
 */

#ifndef WIN32

void usage(char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);

	fprintf(stderr, "\n");
	fprintf(stderr, " ==============================================================================\n");
	fprintf(stderr, "                      VChat - Interactive conference system\n");
	fprintf(stderr, "               (vserver %s - %s) / Protocol revision: %x-%x\n", VERSION, REL_DATE, MAJOR_REV(PROTO_REV), MINOR_REV(PROTO_REV));
	fprintf(stderr, "                       (c) 1993-2002 by Andreas S. Wetzel\n");
	fprintf(stderr, "                           *** All rights reserved ***\n");
	fprintf(stderr, " ==============================================================================\n\n");
	vfprintf(stderr, fmt, ap);
	fprintf(stderr, "\n\n");
	fprintf(stderr, "USAGE:\n\n");
	fprintf(stderr, "  %s [-a <in_addr>] [-A <file>] [-C <file>]", prog_name);
#if HAVE_DAEMON
	fprintf(stderr, " [-d]");
#endif
	fprintf(stderr, " [-D <fwd|rev|strict>]\n");
	fprintf(stderr, "  %*s [-f] [-k <num>] [-l] [-L <file>] [-m] [-M <file>] [-n <name>] [-N]\n", strlen(prog_name), " ");

	fprintf(stderr, "  %*s [-o <num>] [-p <port>] [-P <file>] [-R <passwd>]", strlen(prog_name), " ");
#if HAVE_SYSLOG
	fprintf(stderr, " [-s]");
#endif
	fprintf(stderr, " [-S]\n");
	fprintf(stderr, "  %*s [-T <chan:topic>] [-w <num>] [-W <file>] [-Z <file>]\n\n", strlen(prog_name), " ");

	fprintf(stderr, "OPTIONS:\n\n");
	fprintf(stderr, "  -a <in_addr>    Address to bind server socket to (default=INADDR_ANY)\n");
	fprintf(stderr, "  -A <file>       File containing access control list (default=none)\n");
	fprintf(stderr, "  -C <file>       Server control socket name (default=/tmp/vctl)\n");
#if HAVE_DAEMON
	fprintf(stderr, "  -d              Detach from controlling terminal and run as daemon\n");
#endif
	fprintf(stderr, "  -D fwd          Enable forward DNS checking on connecting clients\n");
	fprintf(stderr, "  -D rev          Enable forward and reverse DNS checking on clients\n");
	fprintf(stderr, "  -D strict       Enable strict forward and reverse DNS checking on clients\n");
	fprintf(stderr, "  -f              Enable time based flood protection\n");
	fprintf(stderr, "  -k <num>        Number of logfile generations to keep (default=%d)\n", DEF_NUMLOGFILES);
	fprintf(stderr, "  -l              Enable logging (default=false)\n");
	fprintf(stderr, "  -L <file>       Log file name (default=vserver.log)\n");
	fprintf(stderr, "  -m              Enable e-mail invitations (default=disable)\n");
	fprintf(stderr, "  -M <file>       File to include in e-mail invitations (default=none)\n");
	fprintf(stderr, "  -n <name>       Override servername (default=name of if-addr)\n");
	fprintf(stderr, "  -N              Disallow access for unregistered users (default=allow)\n");
	fprintf(stderr, "  -o <num>        Max. number of pending connection requests (default=%d)\n", DEF_MAX_PENDING);
	fprintf(stderr, "  -p <port>       Use TCP port <port> (default=1168)\n");
	fprintf(stderr, "  -P <file>       File to write server PID to (default=none)\n");
	fprintf(stderr, "  -R <passwd>     Reset authority file to account 'root' with <passwd>\n");
#if HAVE_SYSLOG
	fprintf(stderr, "  -s              Log via LOG_LOCAL1 to syslog (default=false)\n");
#endif
	fprintf(stderr, "  -S              Suppress client address display (default=false)\n");
	fprintf(stderr, "  -T <chan:topic> Create fixed channel number <chan> with topic <topic>\n");
	fprintf(stderr, "  -w <num>        Enables server lastlog with a maximum of <num> entries\n");
	fprintf(stderr, "  -W <file>       Alternate lastlog filename (default=vserver.lastlog)\n");
	fprintf(stderr, "  -Z <file>       Alternate authorization filename (default=vserver.auth)\n");
	fprintf(stderr, "\n");

	va_end(ap);
}
#endif /* !WIN32 */
