/* $Id: vchatctl.c,v 1.4 2002/06/24 03:17:58 mickey Exp $ */

#include <vserver.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>
#include <vctlproto.h>

#include <stdarg.h>
#include <errno.h>

#ifndef WIN32
  #include <sys/socket.h>
  #include <sys/un.h>

/*** Defines ***/

#ifndef AF_LOCAL
  #define AF_LOCAL AF_UNIX
#endif

#ifndef PF_LOCAL
  #define PF_LOCAL PF_UNIX
#endif

#ifndef SUN_LEN
  #define SUN_LEN(x) (strlen((x)->sun_path) + sizeof((x)->sun_family))
#endif

/*** Code ***/

int init_ctlsock(void)
{
	int s;

	/*
	 * Initialize UDP socket
	 */

	if((s = socket(PF_LOCAL, SOCK_DGRAM, 0)) == -1)
	{
		vlog(VLOG_ERR, "ERROR: socket() returned status %d (%s)",
			errno, strerror(errno));
		return(-1);
	}

	saun.sun_family = AF_LOCAL;
	sprintf(saun.sun_path, "%s", cfdata.ctl_file);
#if (SUN_LEN_FIELD == 1)
	saun.sun_len = SUN_LEN(&saun);
#endif

	unlink(saun.sun_path);

	if(bind(s, (struct sockaddr *) &saun, SUN_LEN(&saun)) < 0)
	{
		vlog(VLOG_ERR, "ERROR: bind() returned status %d (%s)",
			errno, strerror(errno));
		return(-1);
	}

	ctlsock = s;

	return(0);
}

void exit_ctlsock(void)
{
	if(ctlsock)
	{
		close(ctlsock);
		unlink(cfdata.ctl_file);
		ctlsock = 0;
	}
}

void process_ctlcmd(void)
{
	VCTL vctl;
	CLDATA *cl;
	int msglen;

	int i;
	int cnt = 0;
	int found = 0;

	struct sockaddr_un fsa;

	int	flen	= sizeof(fsa);

	BZERO(&fsa, flen);
	BZERO(vctl.data, sizeof(vctl.data));

	if((msglen = RECVFROM(ctlsock, &vctl, sizeof(VCTL), 0, &fsa, &flen)) == -1)
	{
		vlog(VLOG_ERR, "ERROR: Failed to receive message on control socket (%s)", strerror(errno));
		return;
	}

	if(vctl.len != msglen)
	{
		vlog(VLOG_ERR, "ERROR: Truncated control message received - ignored");
		return;
	}

	switch(vctl.type)
	{
		case VCTL_KILL:
		case VCTL_KILLSILENT:
		{
			found = 0;

			for(i = (clients - 1); i >= 0; i--)
			{
				cl = cl_addr(i);

				if(!strcmp(cl->nick, vctl.data))
				{
					if(vctl.type == VCTL_KILLSILENT)
						cmd_signoff(cl, NULL, "Connection reset by peer");
					else
						cmd_signoff(cl, NULL, "Operator kill");

					ctlmsg(&fsa, VCTL_INFO, 0, "user %s has been killed.", vctl.data);

					++found;
				}
			}

			if(!found)
			{
				ctlmsg(&fsa, VCTL_INFO, VCTL_MSGCT, "user %s not found.", vctl.data);
			}
			else
			{
				ctlmsg(&fsa, VCTL_INFO, VCTL_MSGCT, "%d instances killed.", found);
			}

			break;
		}
		case VCTL_ULIST:
		{
			cnt = 0;

			for(i = 0; i < clients; i++)
			{
				cl = cl_addr(i);

				ctlmsg(&fsa, VCTL_USER, 0, "%-15s %s", cl->nick, cl->host);

				++cnt;
			}

			ctlmsg(&fsa, VCTL_INFO, VCTL_MSGCT, "%d user(s) listed.", cnt);
			break;
		}
		case VCTL_ULIST2:
		{
			cnt = 0;

			for(i = 0; i < clients; i++)
			{
				cl = cl_addr(i);

				if(cl->channel >= 0)
				{
					ctlmsg(&fsa, VCTL_UINFO, 0, "(%s)(%d)(#%d)",
						cl->nick, cl->stamp, cl->channel);
				}
				else
				{
					ctlmsg(&fsa, VCTL_UINFO, 0, "(%s)(%d)(#%s)",
						cl->nick, cl->stamp, "??????");
				}

				++cnt;
			}

			ctlmsg(&fsa, VCTL_INFO, VCTL_MSGCT, "");
			break;
		}
		case VCTL_SUPPRESS:
		{
			if(cfdata.flags & CF_SUPPRESS_IP)
			{
				cfdata.flags &= ~(CF_SUPPRESS_IP);
				ctlmsg(&fsa, VCTL_INFO, VCTL_MSGCT, "client IP addresses will be displayed.");
				vlog(VLOG_NOTICE, "Normal server mode selected");
			}
			else
			{
				cfdata.flags |= CF_SUPPRESS_IP;
				ctlmsg(&fsa, VCTL_INFO, VCTL_MSGCT, "suppressing display of client IP addresses.");
				vlog(VLOG_NOTICE, "Anonymous server mode selected");
			}

			break;
		}
		case VCTL_WALL:
		{
			TABLE	tbl;
			char	*opname;
			char	*optty;
			char	*message;

			/*** Get operator name from message data ***/

			opname = &vctl.data[0];

			/*** Get operator terminal from message data ***/

			if((optty = (char *)strchr(opname, ' ')) == NULL)
			{
				ctlmsg(&fsa, VCTL_INFO, VCTL_MSGCT, "ERROR: Invalid message data for wall command - check for version conflict.");
				break;
			}

			*optty++ = '\0';

			/*** Get the message ***/

			if((message = (char *)strchr(optty, ' ')) == NULL)
			{
				ctlmsg(&fsa, VCTL_INFO, VCTL_MSGCT, "ERROR: Invalid message data for wall command - check for version conflict.");
				break;
			}

			*message++ = '\0';

			if(!strlen(message))
			{
				ctlmsg(&fsa, VCTL_INFO, VCTL_MSGCT, "ERROR: No message text specified.");
				break;
			}

			/*** Notify all clients ***/

			cnt = 0;

			for(i = 0; i < clients; i++)
			{
				cl = cl_addr(i);

				table_init(&tbl, cl);

				table_printf(&tbl, "\n* Broadcast message from %s%s%s%s@%s%s%s%s on %s%s%s:\n",
					S_FG_MAG, opname, S_OFF,
					S_BOLD, S_OFF,
					S_FG_GRN, cl->if_name, S_OFF,
					S_FG_CYN, optty, S_OFF);

				table_printf(&tbl, "%s%s%s\n",
					S_BOLD, message, S_OFF);

				table_printf(&tbl, "* EOT");

				table_flush(&tbl);

				++cnt;
			}

			ctlmsg(&fsa, VCTL_INFO, VCTL_MSGCT, "%d user(s) notified.", cnt);
			break;
		}
		case VCTL_SHUTDOWN:
		{
			ctlmsg(&fsa, VCTL_INFO, VCTL_MSGCT, "Shutting down VChat conference server");
			vlog(VLOG_NOTICE, "Server shutdown requested on control socket");
			vserv_shutdown = 1;
			break;
		}
		default:
		{
			vlog(VLOG_ERR, "ERROR: Invalid control message type $%08x received",
				vctl.type);
			break;
		}
	}
}

void ctlmsg(struct sockaddr_un *to, u_char type, u_char flags, char *fmt, ...)
{
	va_list ap;
	VCTL vctl;

	va_start(ap, fmt);

	BZERO(&vctl, sizeof(VCTL));

	vctl.type = type;
	vctl.flags = flags;
	vsprintf(vctl.data, fmt, ap);
	vctl.len = VCTL_LEN(&vctl);

	if(SENDTO(ctlsock, &vctl, vctl.len, 0, to, SUN_LEN(to)) == -1)
	{
		vlog(VLOG_ERR, "ERROR: unable to send control message to %s (%s)", to->sun_path, strerror(errno));
	}

	va_end(ap);
}

#else /* WIN32 */

int init_ctlsock(void)
{
	return(0);
}

void process_ctlcmd(void)
{
}

#endif /* WIN32 */
