/* $Id: vcommand.c,v 1.3 2002/06/23 23:16:07 mickey Exp $ */

#include <vserver.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>

/*** Code ***/

void handle_cmd(u_short cmd, CLDATA *cl, u_char *data, size_t len)
{
	/*
	 * Update timestamp of last activity, unless
	 * when measuring round trip delays via RTCMP.
	 */

	if(cmd != CMD_RTCMP)
		timestamp(&(cl->lastsend));

	/*
	 * Process command.
	 */

	switch(cmd)
	{
		case CMD_ACTION:	cmd_action(cl, data);
					++vs.vs_requests_action;
					break;

		case CMD_ASSTOPIC:	cmd_asstopic(cl, data);
					++vs.vs_requests_asstopic;
					break;

		case CMD_CHANLIST:	cmd_chanlist(cl);
					++vs.vs_requests_chanlist;
					break;

		case CMD_CHCHAN:	cmd_chchan(cl, data);
					++vs.vs_requests_chchan;
					break;

		case CMD_CHNICK:	cmd_chnick(cl, data);
					++vs.vs_requests_chnick;
					break;

		case CMD_CHTOPIC:	cmd_chtopic(cl, data);
					++vs.vs_requests_chtopic;
					break;

		case CMD_CLIENTINFO:	cmd_clientinfo(cl, data);
					++vs.vs_requests_clientinfo;
					break;

		case CMD_INVITE:	cmd_invite(cl, data);
					++vs.vs_requests_invite;
					break;

		case CMD_PAGE:		cmd_page(cl, data);
					++vs.vs_requests_page;
					break;

		case CMD_PRVMSG:	cmd_prvmsg(cl, data);
					++vs.vs_requests_prvmsg;
					break;

		case CMD_PUBMSG:	cmd_pubmsg(cl, data);
					++vs.vs_requests_pubmsg;
					break;

		case CMD_RTCMP:		cmd_rtcmp(cl, data);
					++vs.vs_requests_rtcmp;
					break;

		case CMD_STATUS:	cmd_status(cl, data, len);
					++vs.vs_requests_sv_stat;
					break;

		case CMD_SIGNMSG:	cmd_signmsg(cl);
					++vs.vs_requests_signmsg;
					break;

		case CMD_SIGNOFF:	BZERO(cl->offmsg, sizeof(cl->offmsg));
					if(data && len > 0)
						strncpy(cl->offmsg, data, sizeof(cl->offmsg)-1);
					cl->flags |= CFLAG_SIGNOFF;
					++vs.vs_requests_signoff;
					break;

		case CMD_ULIST:		cmd_ulist(cl, data);
					++vs.vs_requests_ulist;
					break;

		case CMD_VERSION:	cmd_version(cl);
					++vs.vs_requests_snd_id;
					break;

		case CMD_WAKEUP:	cmd_wakeup(cl, data);
					++vs.vs_requests_wakeup;
					break;

		case CMD_CHPASSWD:	cmd_chpasswd(cl, data);
					++vs.vs_requests_chpasswd;
					break;

		case CMD_SETPASSWD:	cmd_setpasswd(cl, data);
					++vs.vs_requests_setpasswd;
					break;

		case CMD_ADDNICK:	cmd_addnick(cl, data);
					++vs.vs_requests_addnick;
					break;

		case CMD_DELNICK:	cmd_delnick(cl, data);
					++vs.vs_requests_delnick;
					break;

		case CMD_CHMODPERM:	cmd_chmodperm(cl, data, len);
					++vs.vs_requests_chmodperm;
					break;

		case CMD_CHACCPERM:	cmd_chaccperm(cl, data, len);
					++vs.vs_requests_chaccperm;
					break;

		case CMD_KILL:		cmd_kill(cl, data);
					++vs.vs_requests_kill;
					break;

		case CMD_BROADCAST:	cmd_broadcast(cl, data);
					++vs.vs_requests_broadcast;
					break;

		case CMD_ACL_ADD:	cmd_acl_add(cl, data, len);
					++vs.vs_requests_acl_add;
					break;

		case CMD_ACL_DELETE:	cmd_acl_delete(cl, data, len);
					++vs.vs_requests_acl_delete;
					break;

		case CMD_ACL_FLUSH:	cmd_acl_flush(cl);
					++vs.vs_requests_acl_flush;
					break;

		case CMD_ACL_LIST:	cmd_acl_list(cl);
					++vs.vs_requests_acl_list;
					break;

		case CMD_QUERY:		cmd_query(cl, data);
					++vs.vs_requests_query;
					break;

		case CMD_LASTLOG:	cmd_lastlog(cl, data);
					++vs.vs_requests_lastlog;
					break;

		case CMD_CLOPT:		cmd_clopt(cl, data, len);
					++vs.vs_requests_clopt;
					break;

		case CMD_SVOPT:		cmd_svopt(cl, data, len);
					++vs.vs_requests_svopt;
					break;

		case CMD_FIXCHAN:	cmd_fixchan(cl, data, len);
					++vs.vs_requests_fixchan;
					break;

		case CMD_SVDNSOPT:	cmd_svdnsopt(cl, data, len);
					++vs.vs_requests_svdnsopt;
					break;

		case CMD_EXCLUDE:	cmd_exclude(cl, data, len);
					++vs.vs_requests_exclude;
					break;

		default:		snd_svmsg(cl, "* Your client sent an unknown command byte: %s$%04x%s",
						S_FG_RED, cmd, S_OFF);
					vlog(VLOG_WARNING, "WARNING: Unknown client command CMD: $%04x USER: %s NICK: %s FROM: %s",
						cmd, cl->user, cl->nick, cl->host);
					++vs.vs_requests_unknown;
					break;
	}

	/*
	 * Update total request counter
	 */

	++vs.vs_requests_processed;
}
