/* $Id: vlog.c,v 1.1 2001/04/09 18:30:45 mickey Exp $ */

#include <vserver.h>
#include <proto.h>
#include <extern.h>

#include <stdarg.h>
#include <errno.h>
#include <time.h>
#include <fcntl.h>
#include <sys/mman.h>

/****** Code ******/

void init_logfile(void)
{
	if(cfdata.logswitch == VLOG_SYSLOG)
	{
#if HAVE_SYSLOG
		openlog(prog_name, LOG_NDELAY, LOG_LOCAL1);
		vlog(VLOG_NOTICE, "Log started");
#else
		fprintf(stderr, "WARNING! syslog support is not compiled in this binary!\n");
#endif
	}
	else if(cfdata.logswitch == VLOG_FILE)
	{
		if((lgf = fopen(cfdata.log_file, "a")) == NULL)
		{
			fprintf(stderr, "%s: unable to open logfile '%s' for writing.\n",
				prog_name, cfdata.log_file);
			fprintf(stderr, "%s: logging disabled\n\n", prog_name);
			cfdata.logswitch = VLOG_NONE;
		}
		else
		{
			setvbuf(lgf, NULL, _IONBF, 0);
			vlog(VLOG_INFO, "Logfile started");
		}
	}
}

void exit_logfile(void)
{
	if(cfdata.logswitch == VLOG_SYSLOG)
	{
		closelog();
	}
	else if(cfdata.logswitch == VLOG_FILE)
	{
		if(lgf)
		{
			fclose(lgf);
			lgf = NULL;
		}
	}
}

void vlog(int pri, char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);

	if(cfdata.logswitch == VLOG_SYSLOG)
	{
#if HAVE_SYSLOG
  #if HAVE_VSYSLOG
		vsyslog(pri, fmt, ap);
  #else
		u_char logmsg[512];

		vsprintf(logmsg, fmt, ap);

		syslog(pri, logmsg);
  #endif
#endif
	}
	else if(cfdata.logswitch == VLOG_FILE)
	{
		char	dbuf[64];
		time_t	now;

		time(&now);
		strftime(dbuf, 64, "%b %d %H:%M:%S", localtime(&now));

		fprintf(lgf, "%s - ", dbuf);
		vfprintf(lgf, fmt, ap);
		putc('\n', lgf);
	}

	va_end(ap);
}

void init_lastlog(void)
{
	/*
	 * Nothing to do if lastlog feature is disabled
	 */

	if(cfdata.llog_entries <= 0)
		return;

	/*
	 * Does the last log file already exist?
	 */

	if(!access(cfdata.llog_file, F_OK))
	{
		LLOGHEAD	lhead;

		/*
		 * Open existing lastlog database file.
		 */

		vlog(VLOG_NOTICE, "Opening existing lastlog database file '%s'",
			cfdata.llog_file);

		if((llog_fd = open(cfdata.llog_file, O_RDWR, 0)) == -1)
		{
			vlog(VLOG_ERR, "failed to open lastlog file '%s' (%s)",
				cfdata.llog_file, strerror(errno));
			vlog(VLOG_ERR, "lastlog feature disabled due to error");
			cfdata.llog_entries = 0;
			BZERO(&cfdata.llog_file, sizeof(cfdata.llog_file));
			return;
		}

		/*
		 * Read the header and determine number of entries.
		 */

		if(read(llog_fd, &lhead, sizeof(lhead)) != sizeof(lhead))
		{
			close(llog_fd);
			vlog(VLOG_ERR, "failed to read lastlog file '%s' header (%s)",
				cfdata.llog_file, strerror(errno));
			vlog(VLOG_ERR, "lastlog feature disabled due to error");
			cfdata.llog_entries = 0;
			BZERO(&cfdata.llog_file, sizeof(cfdata.llog_file));
			return;

		}

		if(lseek(llog_fd, 0, SEEK_SET) == -1)
		{
			close(llog_fd);
			vlog(VLOG_ERR, "failed to seek lastlog file '%s' (%s)",
				cfdata.llog_file, strerror(errno));
			vlog(VLOG_ERR, "lastlog feature disabled due to error");
			cfdata.llog_entries = 0;
			BZERO(&cfdata.llog_file, sizeof(cfdata.llog_file));
			return;
		}

		cfdata.llog_entries = lhead.total_records;
	}
	else if(errno == ENOENT)	/* Last log file does not exist */
	{
		LLOGHEAD	lhead;
		u_char		buf[sizeof(LLOGDAT)];

		register int	i;

		/*
		 * Create a new lastlog database file.
		 */

		vlog(VLOG_NOTICE, "Creating new lastlog database file '%s'",
			cfdata.llog_file);

		if((llog_fd = open(cfdata.llog_file, (O_RDWR | O_CREAT), 0640)) == -1)
		{
			vlog(VLOG_ERR, "failed to create new lastlog file '%s' (%s)",
				cfdata.llog_file, strerror(errno));
			vlog(VLOG_ERR, "lastlog feature disabled due to error");
			cfdata.llog_entries = 0;
			BZERO(&cfdata.llog_file, sizeof(cfdata.llog_file));
			return;
		}

		lhead.first_record = lhead.next_record = lhead.used_records = 0;
		lhead.total_records = cfdata.llog_entries;

		BZERO(&buf, sizeof(buf));

		if(write(llog_fd, (void *)&lhead, sizeof(lhead)) != sizeof(lhead))
		{
			vlog(VLOG_ERR, "write error on lastlog database file '%s' (%s)",
				cfdata.llog_file, strerror(errno));
			vlog(VLOG_ERR, "lastlog feature disabled due to error");
			unlink(cfdata.llog_file);
			cfdata.llog_entries = 0;
			BZERO(&cfdata.llog_file, sizeof(cfdata.llog_file));
			return;
		}

		for(i = 0; i < cfdata.llog_entries; i++)
		{
			if(write(llog_fd, &buf, sizeof(buf)) != sizeof(buf))
			{
				
				vlog(VLOG_ERR, "lastlog feature disabled due to error");
				unlink(cfdata.llog_file);
				cfdata.llog_entries = 0;
				BZERO(&cfdata.llog_file, sizeof(cfdata.llog_file));
				return;
			}
		}
	}
	else
	{
		/*
		 * Error while trying to access(2) file.
		 */

		vlog(VLOG_ERR, "failed to access lastlog file '%s' (%s)",
			cfdata.llog_file, strerror(errno));
		vlog(VLOG_ERR, "lastlog feature disabled due to error");
		cfdata.llog_entries = 0;
		BZERO(&cfdata.llog_file, sizeof(cfdata.llog_file));
		return;
	}

	/*
	 * Map the lastlog database file into memory.
	 */

	vlog(VLOG_NOTICE, "System Page size: %d  Lastlog entry size: %d  Lastlog header size: %d",
		getpagesize(), sizeof(LLOGDAT), sizeof(LLOGHEAD));

	if((llog_base = mmap(NULL, LLOG_SIZE, (PROT_READ | PROT_WRITE),
		MAP_SHARED, llog_fd, 0)) == NULL)
	{
		close(llog_fd);
		vlog(VLOG_ERR, "failed to mmap file '%s' into memory (%s)",
			cfdata.llog_file, strerror(errno));
		vlog(VLOG_ERR, "lastlog feature disabled due to error");
		BZERO(&cfdata.llog_file, sizeof(cfdata.llog_file));
		cfdata.llog_entries = 0;
		return;
	}

#if HAVE_MADVISE
	if(madvise(llog_base, LLOG_SIZE, MADV_SEQUENTIAL) == -1)
	{
		vlog(VLOG_WARNING, "failed to madvise() memory at $%08x (%s)",
			llog_base, strerror(errno));
	}
#endif

	vlog(VLOG_NOTICE, "Lastlog database mapped at address $%08x size: %d bytes",
		llog_base, LLOG_SIZE);


	llog_header = LLOG_HEADER;

#if 0
	vlog(VLOG_NOTICE, "Lastlog database -> FIRST: %d NEXT: %d USED: %d TOTAL: %d",
		llog_header->first_record,
		llog_header->next_record,
		llog_header->used_records,
		llog_header->total_records);
#endif

}

void exit_lastlog(void)
{
	/*
	 * Nothing to do if lastlog feature is disabled
	 */

	if(llog_base == NULL || llog_header == NULL || cfdata.llog_entries <= 0)
		return;

	/*
	 * Unmap the lastlog database from memory and close database file.
	 */

	if(munmap(llog_base, (cfdata.llog_entries * sizeof(LLOGDAT))) == -1)
	{
		vlog(VLOG_ERR, "failed to unmap lastlog database file '%s' from memory (%s)",
			cfdata.llog_file, strerror(errno));
	}

	close(llog_fd);

	BZERO(&cfdata.llog_file, sizeof(cfdata.llog_file));
	cfdata.llog_entries = 0;
	llog_fd = 0;
	llog_base = NULL;
}

void sync_lastlog(void *addr, size_t len)
{
	/*
	 * Sync lastlog database
	 */

	if(msync(addr, len, MS_SYNC) == -1)
	{
		vlog(VLOG_ERR, "failed to sync %d bytes of lastlog database memory at address $%08x (%s)",
			len, addr, strerror(errno));
	}
}

void lastlog_addentry(CLDATA *cl, char *info)
{
	LLOGDAT	ldat;
	LLOGDAT	*lptr;

	/*
	 * Check environment
	 */

	if(llog_base == NULL || llog_header == NULL || cfdata.llog_entries <= 0)
		return;

	/*
	 * Construct the new lastlog entry
	 */

	BZERO(&ldat, sizeof(ldat));
	ldat.signon_stamp = cl->login;
	time(&ldat.signoff_stamp);
	strncpy(ldat.username, cl->user, sizeof(ldat.username)-1);
	strncpy(ldat.nickname, cl->nick, sizeof(ldat.nickname)-1);

	if(info != NULL)
		strncpy(ldat.info, info, sizeof(ldat.info)-1);

	ldat.ipaddr.s_addr = cl->ip_addr.s_addr;
	strncpy(ldat.hostname, cl->host, sizeof(ldat.hostname)-1);

	/*
	 * Insert the new entry into the database
	 */

	if(llog_header->used_records < llog_header->total_records)
	{
		++llog_header->used_records;		
	}
	else
	{
		/*
		 * Cut off one entry
		 */

		if(++llog_header->first_record >= llog_header->total_records)
			llog_header->first_record = 0;
	}

	lptr = (void *)LLOG_RECORD(llog_header->next_record);

	if(++llog_header->next_record >= llog_header->total_records)
		llog_header->next_record = 0;

	BCOPY(&ldat, (void *)lptr, sizeof(ldat));

	sync_lastlog((void *)lptr, sizeof(ldat));
}
