/* $Id: vserver.c,v 1.6 2002/07/12 23:10:50 mickey Exp $ */

#include <vserver.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>

#include <stdlib.h>
#include <errno.h>

#ifdef WIN32
  #include <windows.h>
  #include <stdio.h>
  #include <string.h>
  #include <io.h>
  #include <process.h>
  #include <tchar.h>
#else
  #include <sys/socket.h>
#endif

#if HAVE_SYS_SELECT_H
  #include <sys/select.h>
#endif

#ifdef WIN32
#define SZSERVICENAME	"VChatServer"	/* Name of this service		*/
#endif

/*
 * WIN32: VOID _CRTAPI1 main(int argc, char *argv[]);
 *  UNIX: int main(int argc, char *argv[]);
 *
 * The VChat conference server program code main entry point.
 */

#ifdef WIN32
VOID _CRTAPI1 main(int argc, char *argv[])
#else
int main(int argc, char *argv[])
#endif
{
#ifdef WIN32
	SERVICE_TABLE_ENTRY dispatchTable[] =
	{
		{ TEXT(SZSERVICENAME), (LPSERVICE_MAIN_FUNCTION)svc_main },
		{ NULL, NULL }
	};
#endif

	/*
	 * Determine program name
	 */

#ifdef WIN32
	if((prog_name = (char *)strrchr(argv[0], '\\')) != NULL)
#else
	if((prog_name = (char *)strrchr(argv[0], '/')) != NULL)
#endif
		prog_name++;
	else
		prog_name = argv[0];

#ifdef MSGDB

	/*
	 * Make sure that the INFORMIXSERVER environment is set
	 * when the message database extensions are compiled in.
	 */

	if(getenv("INFORMIXSERVER") == NULL)
	{
		fprintf(stderr, "%s: The INFORMIXSERVER environment is not set up properly\n", prog_name);
		return(-1);
	}

#endif /* MSGDB */

	/*
	 * Setup base defaults
	 */

	BZERO(&cfdata, sizeof(cfdata));
	sprintf(cfdata.auth_file, "vserver.auth");
#ifndef WIN32
	sprintf(cfdata.ctl_file, "/tmp/vctl");
#endif
	sprintf(cfdata.log_file, "vserver.log");
	sprintf(cfdata.llog_file, "vserver.lastlog");
	
	cfdata.bind_addr.s_addr = INADDR_ANY;
	cfdata.port = htons(SERVER_PORT);

	cfdata.max_pending = DEF_MAX_PENDING;
	cfdata.keep_logfiles = DEF_NUMLOGFILES;

	cfdata.dnschklvl = DNS_CHK_NONE;

	/*
	 * Initialite vtopic database
	 */

	vtop.bufaddr = NULL;
	vtop.entries = 0;

#ifdef WIN32

	/*
	 * Get configuration data from Registry.
	 */

	if(argc < 2)
	{
		
	}
	
	if(GetRegistryData(argv[1]))
	{
		DWORD	dwErr = GetLastError();
		LogMessage(VLOG_ERR, "Failed to get configuration data from windows-registry");
		ReportStatusToSCMgr(SERVICE_STOPPED, ERROR_SERVICE_SPECIFIC_ERROR, 0);
		return;
	}

	/*
	 * Call StartServiceCtrlDispatcher()
	 */

	if(!StartServiceCtrlDispatcher(dispatchTable))
		LogMessage(VLOG_ERR, "StartServiceCtrlDispatcher failed.");

#else	/* ------------------------- UNIX ------------------------- */

	/*
	 * Process commandline arguments.
	 */

	if(process_cmdline(argc, argv))
		return(-1);


  #if HAVE_DAEMON

	/*
	 * Detach from controlling terminal if running as daemon
	 */

	if(cfdata.flags & CF_DETACH)
	{
		if(daemon(1, 0) == -1)
		{
			fprintf(stderr, "%s: error detaching from controlling terminal (%s)\n",
				prog_name, strerror(errno));
		}
	}

  #endif

	/*
	 * Initialize server socket and enter server main loop
	 * to start servicing remote clients.
	 */

	return(ServiceStart());
#endif
}

/*
 * int ServiceStart(void);
 *
 * The function that performs the server main-loop and thus starts
 * servicing the clients.
 */

int ServiceStart(void)
{
	u_char		buffer[SVMSGBUF];
	VMSG		svmsg;
	fd_set		rfdset;
	CLDATA		*cl_ptr;
	u_short		cmd;
	u_long		len;
	int		i;
	int		n;
	int		sel;
	struct timeval	acttime;
	struct timeval	sTimeout;
	struct timeval	*timeGone;

#ifdef WIN32
	if(!ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000))
		goto iserror;
#endif
	/*
	 * Initialize global settings
	 */

	if(init_global())
		return(-1);

#ifdef WIN32
	if(!ReportStatusToSCMgr(SERVICE_RUNNING, NO_ERROR, 0))
		goto iserror;

	LogMessage(VLOG_INFO, "Starting normal server operation");
#endif

	/*
	 * Server main-loop
	 */

	while(!vserv_shutdown)
	{
		FD_ZERO(&rfdset);
#ifndef WIN32
		FD_SET(ctlsock, &rfdset);
#endif

		/*
		 * If flood protection is turned on
		 * check wheter to reenable accept
		 */

		if((cfdata.flags & CF_FLOOD_GUARD) != 0
		&& (cfdata.flags & CF_REG_ONLY) == 0)
		{
			/*
			 * Regonly is not set, flood protect is set
			 */

			timestamp(&acttime);
			timeGone = diff_t(&lastaccept, &acttime);

			if(timeGone->tv_sec >= 1)
			{
				FD_SET(sock, &rfdset);
			}
		}
		else
		{
			FD_SET(sock, &rfdset);
		}

		/*
		 * Put client fds into descriptor set.
		 */

		for(i = 0; i < clients; i++)
		{
			cl_ptr = cl_addr(i);

			/*
			 * If flood protection is turned on
			 * check wheter to enable this client
			 */

			if((cfdata.flags & CF_FLOOD_GUARD) != 0
			&& (cfdata.flags & CF_REG_ONLY) == 0
			&& (cl_ptr->flags & CFLAG_REGISTERED) == 0)
			{
			        /*
				 * Regonly is not set, flood protect is set
				 * AND client is not registered
				 */

				timestamp(&acttime);
				timeGone = diff_t(&(cl_ptr->lastsend), &acttime);

				if(timeGone->tv_sec >= 2)
				{
					FD_SET(cl_ptr->fd, &rfdset);
				}
			}
			else
			{
				FD_SET(cl_ptr->fd, &rfdset);
			}
		}

		/*
		 * The select timeout needs to be set again during each loop
		 * since there are stupid UN*Xes believing they should change
		 * the timeout value on return *shrug*
		 */

		sTimeout.tv_sec = 1;
		sTimeout.tv_usec = 0;

		/*
		 * Wait until something happens on our descriptors.
		 */

		sel = select(FD_SETSIZE, &rfdset, (fd_set *)NULL, (fd_set *)NULL, &sTimeout);

#ifdef WIN32
		if(sel == SOCKET_ERROR)
		{
			int errcod = WSAGetLastError();

			vlog(VLOG_CRIT, "FATAL: select() returned status %d (%s) - server exiting",
					errcod, strerror(errcod));
			break;
		}
#else
		if(sel < 0)
		{
			if(errno == EINTR)
			{
				vlog(VLOG_NOTICE, "select() has been interrupted.");
				continue;
			}
			else
			{
				vlog(VLOG_CRIT, "FATAL: select() returned status %d (%s) - server exiting",
					errno, strerror(errno));
				break;
			}
		}
#endif
		else if(sel == 0)
		{
			if(!acl_delete_expired_rules())
			{
				continue;
			}
			else
			{
				vlog(VLOG_CRIT, "FATAL: acl_delete_expired_rules() has failed.");
				break;
			}
		}

#ifdef WIN32
		/*
		 * Check if the main socket is zero which states that
		 * the server shutdown signal has been received.
		 */

		if(!sock)
			break;

#endif

		/* If there's data available */
		/* at our original socket, a */
		/* new client process wants  */
		/* to connect us.            */

		if(FD_ISSET(sock, &rfdset))
		{
			assign_client();
			timestamp(&lastaccept);
		}

		/* Check for any data waiting */
		/* from our client processes, */
		/* and process input.         */

		for(i = 0; i < clients; i++)
		{
			cl_ptr = cl_addr(i);

			/* Check for data waiting */

			if(FD_ISSET(cl_ptr->fd, &rfdset))
			{
				/*
				 * Get command packet
				 */

				if(vio_read(cl_ptr->fd, (char *)&svmsg, sizeof(VMSG), CONN_TIMEOUT) == -1)
				{
					cl_ptr->flags |= CFLAG_ERROR;
					cl_ptr->error = errno;
					continue;
				}

				cmd = (u_short)ntohl(svmsg.cmd);
				len = (ntohl(svmsg.len) < SVMSGBUF) ? ntohl(svmsg.len) : (SVMSGBUF - 1);

				/*
				 * Get appended data packet if any.
				 */
				
				if(vio_read(cl_ptr->fd, &buffer[0], len, CONN_TIMEOUT) == -1)
				{
					cl_ptr->flags |= CFLAG_ERROR;
					cl_ptr->error = errno;
					continue;
				}

				/*
				 * Assure correct buffer termination
				 */

				buffer[len] = '\0';

				/*
				 * Finally execute the client command.
				 */
				
				timestamp(&tmstat.startmark);
				handle_cmd(cmd, cl_ptr, buffer, len);
				timestamp(&tmstat.endmark);

				store_tm_value(diff_t(&tmstat.startmark, &tmstat.endmark));
			}
		}

		/*
		 * Lookup clients which have their error or kill flag
		 * set and remove them. This is done in reverse order
		 * to be aware of changes in the client table due to
		 * removing clients from the table.
		 */

		n = clients - 1;

		for(i = n; i >= 0; i--)
		{
			cl_ptr = cl_addr(i);

			if(cl_ptr->flags & CFLAG_SIGNOFF)
			{
				if(cl_ptr->offmsg[0] != '\0')
					cmd_signoff(cl_ptr, cl_ptr->offmsg, NULL);
				else
					cmd_signoff(cl_ptr, NULL, NULL);
			}
			else if(cl_ptr->flags & CFLAG_ERROR)
			{
				vlog(VLOG_ERR, "ERROR: Error condition on client descriptor (%s) DESC: %d USER: %s NICK: %s HOST: %s",
					strerror(cl_ptr->error), cl_ptr->fd,
					cl_ptr->user, cl_ptr->nick,
					cl_ptr->host);

				++vs.vs_err_conn;
				cmd_signoff(cl_ptr, NULL, (char *)strerror(cl_ptr->error));
			}
			else if(cl_ptr->flags & CFLAG_KILL)
			{
				u_char	offmsg[256];

				/*
				 * This client has been killed.
				 */

				sprintf(offmsg, "Killed by %s", cl_ptr->killinfo);
				cmd_signoff(cl_ptr, NULL, offmsg);
			}
			else if(cl_ptr->flags & CFLAG_EXCLUDE)
			{
				u_char	offmsg[256];

				/*
				 * This client has been excluded.
				 */

				sprintf(offmsg, "Excluded by %s", cl_ptr->killinfo);
				cmd_signoff(cl_ptr, NULL, offmsg);
			}
		}

#ifndef WIN32
		/* If we received a udp msg  */
		/* on our control socket we  */
		/* have to process the ctl   */
		/* command now.              */

		if(FD_ISSET(ctlsock, &rfdset))
			process_ctlcmd();
#endif
	}

	/*
	 * Signoff all connected clients
	 */

	n = clients - 1;

	for(i = n; i >= 0; i--)
	{
		cl_ptr = cl_addr(i);

		cmd_signoff(cl_ptr, NULL, "Server shutdown");
	}	

#ifdef WIN32
	closesocket(sock);

	if(wsa_is_init)
		WSACleanup();
#else
	close(sock);
#endif
	free(cl_buf);

	exit_global();

	return(0);

#ifdef WIN32
iserror:

	closesocket(sock);

	if(wsa_is_init)
		WSACleanup();

	free(cl_buf);

	exit_global();

	return(-1);
#endif
}
