/* $Id: vtopic.c,v 1.1 2001/04/09 18:30:45 mickey Exp $ */

#include <vserver.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>

#include <stdlib.h>
#include <errno.h>

/****** Code ******/

int store_topic(signed long channel, char *top, u_long flags)
{
	int			tidx;
	char			*tmpaddr;
	struct chantopic	*ct;

	/*
	 * Store topic
	 */

	if(vtop.bufaddr)
	{
		if((tidx = topic_idx(channel)) != -1)
		{
			ct = TOPADDR(tidx);

			if(ct->flags & flags)
			{
				strncpy(ct->topic, top, VPTOPICSIZE);
				ct->topic[VPTOPICSIZE] = '\0';
				ct->flags = flags;
			}
			else
			{
				return(-1);
			}
		}
		else
		{
			if((tmpaddr = (char *)realloc(vtop.bufaddr, ((vtop.entries + 1) * sizeof(struct chantopic)))) != NULL)
			{
				vtop.bufaddr = tmpaddr;
				ct = TOPADDR(vtop.entries++);

				ct->channel = channel;
				strncpy(ct->topic, top, VPTOPICSIZE);
				ct->topic[VPTOPICSIZE] = '\0';
				ct->flags = flags;
			}
			else
			{
				vlog(VLOG_ERR, "ERROR: Topic database memory reallocation failed (%s)", strerror(errno));
			}
		}
	}
	else
	{
		if((vtop.bufaddr = (char *)malloc(sizeof(struct chantopic))) == NULL)
		{
			vlog(VLOG_WARNING, "ERROR: Topic database memory allocation failed (%s)", strerror(errno));
		}
		else
		{
			ct = (struct chantopic *)vtop.bufaddr;

			ct->channel = channel;
			strncpy(ct->topic, top, VPTOPICSIZE);
			ct->topic[VPTOPICSIZE] = '\0';
			ct->flags = flags;

			vtop.entries = 1;
		}
	}

	return(0);
}

/*
 * Remove a conference channel from the topic database
 * if it's flags correspond to those specified for the
 * <flags> parameter.
 *
 * RETURN VALUES:  0 - Success
 *                -1 - Specified channel does not exist
 *                -2 - Channel exists but flags not equal
 */

int remove_topic(signed long channel, u_int32_t flags)
{
	int			topnum;
	char			*tmpaddr;
	struct chantopic	*ct;

	if((topnum = topic_idx(channel)) == -1)
		return(-1);

	ct = TOPADDR(topnum);

	if(!(ct->flags & flags))
		return(-2);

	if(vtop.entries > 1)
	{
		if((u_int)topnum < (vtop.entries - 1))
		{
			BCOPY(TOPADDR((topnum + 1)), TOPADDR(topnum), ((vtop.entries - (topnum + 1)) * sizeof(struct chantopic)));
		}

		--vtop.entries;

		if((tmpaddr = (char *) realloc(vtop.bufaddr, (vtop.entries * sizeof(struct chantopic)))) == NULL)
			vlog(VLOG_WARNING, "ERROR: Topic database memory reallocation failed (%s)", strerror(errno));
		else
			vtop.bufaddr = tmpaddr;
	}
	else
	{
		free(vtop.bufaddr);
		vtop.bufaddr = NULL;
		vtop.entries = 0;
	}

	return(0);
}

int topic_idx(signed long channel)
{
	u_int			i;
	struct chantopic	*ct;

	for(i = 0; i < vtop.entries; i++)
	{
		ct = TOPADDR(i);

		if(ct->channel == channel)
		{
			return(i);
		}
	}

	return(-1);
}

char *topic_ptr(signed long channel)
{
	u_long			i;
	struct chantopic	*ct;

	for(i = 0; i < vtop.entries; i++)
	{
		ct = TOPADDR(i);

		if(ct->channel == channel)
			return(ct->topic);
	}

	return(NULL);
}

void touch_channel(signed long channel)
{
	u_long			i;
	struct chantopic	*ct;

	for(i = 0; i < vtop.entries; i++)
	{
		ct = TOPADDR(i);

		if(ct->channel == channel)
			time(&ct->stamp);
	}
}

char *pftime(time_t clock)
{
	time_t	now;

	time(&now);

	return(pf2time(clock, now));
}

char *pf2time(time_t clock1, time_t clock2)
{
	time_t diff;

	u_int sec;
	u_int min;
	u_int hrs;
	u_int day;
	u_int wks;

	static char tstr[32];

	diff = (clock2 - clock1);

	wks = ((int)diff / 0x93a80);
	day = ((int)(diff % 0x93a80) / 0x15180);
	hrs = ((int)(diff % 0x15180) / 0xe10);
	min = ((int)(diff % 0xe10) / 0x3c);
	sec = (int)(diff % 0x3c);

	if(wks)
		sprintf(tstr, "%dw+%dd+%02d:%02dh", wks, day, hrs, min);
	else if(day)
		sprintf(tstr, "%dd+%02d:%02dh", day, hrs, min);
	else if(hrs)
		sprintf(tstr, "%02d:%02d:%02dh", hrs, min, sec);
	else if(min)
		sprintf(tstr, "%02d:%02dm", min, sec);
	else
		sprintf(tstr, "%ds", sec);

	return(tstr);
}
