/* $Id: win32util.c,v 1.1 2001/04/09 18:30:45 mickey Exp $ */

#ifdef WIN32	/* --------> This file only valid for Windows 32 <--------- */

#include <vserver.h>
#include <vproto/types.h>
#include <proto.h>
#include <extern.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include <windows.h>
#include <io.h>
#include <process.h>
#include <tchar.h>
#include <winsock.h>

#define	SVC_KEY	"SYSTEM\\CurrentControlSet\\Services"

/*
 * Get configuration data from Windows Registry.
 */

int GetRegistryData(char *cfgname)
{
	HKEY	hKey;
	u_char	szKeyName[MAX_PATH+1]	= "\0";
	u_char	szValName[MAX_PATH+1]	= "\0";
	u_char	szData[MAX_PATH+1]	= "\0";
	DWORD	dwValNameSize		= MAX_PATH;
	DWORD	dwDataSize		= MAX_PATH;
	DWORD	dwValIdx		= 0;
	DWORD	dwValType;
	DWORD	dwRet;

	/*
	 * If no configuration name is given we cannot proceed.
	 */

	if(!cfgname)
	{
		LogMessage(VLOG_ERR, "No configuration name specified - cannot continue.");
		return(-1);
	}

	/*
	 * Open the "Parameter" key of the given configuration name.
	 */

	sprintf(szKeyName, "%s\\VChatServer.%s\\Parameter", SVC_KEY, cfgname);

	if((dwRet = RegOpenKeyEx(HKEY_LOCAL_MACHINE, szKeyName, 0, KEY_QUERY_VALUE, &hKey)))
	{
		LogMessage(VLOG_ERR, "Failed to open window registry key '%s' (%s)",
			szKeyName, winerror(dwRet));
		return(-1);
	}

	LogMessage(VLOG_INFO, "Reading configuration from '%s' ...", szKeyName);

	/*
	 * Step through all values for the "Parameter" key
	 * and look out for known value names.
	 */

	while(!RegEnumValue(hKey, dwValIdx, szValName, &dwValNameSize, NULL, &dwValType, szData, &dwDataSize))
	{
		if(!strcmp(szValName, "AuthorityFile")
		&& (dwValType == REG_EXPAND_SZ || dwValType == REG_SZ))
		{
			BZERO(&cfdata.auth_file, sizeof(cfdata.auth_file));
			strncpy(cfdata.auth_file, szData, sizeof(cfdata.auth_file)-1);
		}
		else if(!strcmp(szValName, "ACLFile")
		&& (dwValType == REG_EXPAND_SZ || dwValType == REG_SZ))
		{
			BZERO(&cfdata.acl_file, sizeof(cfdata.acl_file));
			strncpy(cfdata.acl_file, szData, sizeof(cfdata.acl_file)-1);
		}
		else if(!strcmp(szValName, "LogFile")
		&& (dwValType == REG_EXPAND_SZ || dwValType == REG_SZ))
		{
			BZERO(&cfdata.log_file, sizeof(cfdata.log_file));
			strncpy(cfdata.log_file, szData, sizeof(cfdata.log_file)-1);
		}
		else if(!strcmp(szValName, "PIDFile")
		&& (dwValType == REG_EXPAND_SZ || dwValType == REG_SZ))
		{
			BZERO(&cfdata.pid_file, sizeof(cfdata.pid_file));
			strncpy(cfdata.pid_file, szData, sizeof(cfdata.pid_file)-1);
		}
		else if(!strcmp(szValName, "Port") && dwValType == REG_DWORD)
		{
			cfdata.port = htons((u_short)*(DWORD *)szData);
		}
		else if(!strcmp(szValName, "MaxPending") && dwValType == REG_DWORD)
		{
			cfdata.max_pending = *((DWORD *)szData);
		}
		else if(!strcmp(szValName, "LogEnable") && dwValType == REG_DWORD)
		{
			if(*((DWORD *)szData))
				cfdata.logswitch = VLOG_FILE;
			else
				cfdata.logswitch = VLOG_NONE;
		}
		else if(!strcmp(szValName, "BindAddress")
		&& (dwValType == REG_EXPAND_SZ || dwValType == REG_SZ))
		{
			cfdata.bind_addr.S_un.S_addr = *((DWORD *)szData);
		}
		else if(!strcmp(szValName, "KeepLogfiles") && dwValType == REG_DWORD)
		{
			cfdata.keep_logfiles = *((DWORD *)szData);
		}
		else if(!strcmp(szValName, "SuppressAddressDisplay") && dwValType == REG_DWORD)
		{
			if(*((DWORD *)szData))
				cfdata.flags |= CF_SUPPRESS_IP;
			else
				cfdata.flags &= ~CF_SUPPRESS_IP;
		}
		else if(!strcmp(szValName, "RegisteredOnly") && dwValType == REG_DWORD)
		{
			if(*((DWORD *)szData))
				cfdata.flags |= CF_REG_ONLY;
			else
				cfdata.flags &= ~CF_REG_ONLY;
		}
		else if(!strcmp(szValName, "RejectDNSFail") && dwValType == REG_DWORD)
		{
			if(*((DWORD *)szData))
				cfdata.flags |= CF_DNSFAIL_REJ;
			else
				cfdata.flags &= ~CF_DNSFAIL_REJ;
		}

		++dwValIdx;
		dwValNameSize = dwDataSize = MAX_PATH;
	}

	/*
	 * Close the key.
	 */

	RegCloseKey(hKey);

	LogMessage(VLOG_INFO, "Succeeded.");

	return(0);
}

/*
 * svc_main - execute actual service initialization.
 */

VOID WINAPI svc_main(DWORD dwArgc, LPTSTR *lpszArgv)
{
	/*
	 * Register service control handler.
	 */

	if(!(sshStatusHandle = RegisterServiceCtrlHandler(TEXT(SZSERVICENAME), svc_ctl)))
	{
		LogMessage(VLOG_ERR, "Failed to register service control handler");
		goto cleanup;
	}

	/*
	 * Service status members
	 */

	ssStatus.dwServiceType = SERVICE_WIN32_OWN_PROCESS;
	ssStatus.dwServiceSpecificExitCode = 0;

	/*
	 * Report the status to the service control manager.
	 */

	if(!ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000))
	{
		LogMessage(VLOG_ERR, "Failed to report status to service control manager");
		goto cleanup;
	}

	ServiceStart();

cleanup:

	/*
	 * Try to report "Stopped" status to the service control manager.
	 */

	if(sshStatusHandle)
		ReportStatusToSCMgr(SERVICE_STOPPED, dwErr, 0);
}

/*
 * svc_ctl - handler function called by SCMmgr whenever ControlService()
 *           is invoked on this service.
 */

VOID WINAPI svc_ctl(DWORD dwCtlCode)
{
	/*
	 * Handle the requested control code.
	 */

	switch(dwCtlCode)
	{
	case SERVICE_CONTROL_STOP:		/* Shutdown the service */
		ReportStatusToSCMgr(SERVICE_STOP_PENDING, NO_ERROR, 0);
		ServiceStop();
		return;

	case SERVICE_CONTROL_INTERROGATE:	/* Update service status */
		break;

	default:				/* Invalid control code */
		break;
	}

	ReportStatusToSCMgr(ssStatus.dwCurrentState, NO_ERROR, 0);
}

/*
 * ReportStatusToSCMgr - Sets the current status of the service and
 *                       reports it to the service control manager.
 */

BOOL ReportStatusToSCMgr(DWORD dwCurrentState, DWORD dwWin32ExitCode, DWORD dwWaitHint)
{
	static DWORD	dwCheckPoint	= 1;
	BOOL		fResult		= TRUE;

	if(dwCurrentState == SERVICE_START_PENDING)
		ssStatus.dwControlsAccepted = 0;
	else
		ssStatus.dwControlsAccepted = SERVICE_ACCEPT_STOP;

	ssStatus.dwCurrentState = dwCurrentState;
	ssStatus.dwWin32ExitCode = dwWin32ExitCode;
	if(dwWin32ExitCode == ERROR_SERVICE_SPECIFIC_ERROR)
		ssStatus.dwServiceSpecificExitCode = -1;
	ssStatus.dwWaitHint = dwWaitHint;

	if((dwCurrentState == SERVICE_RUNNING)
	|| (dwCurrentState == SERVICE_STOPPED))
	{
		ssStatus.dwCheckPoint = 0;
	}
	else
	{
		++ssStatus.dwCheckPoint;
	}

	/*
	 * Report status to service control manager.
	 */

	if(!(fResult = SetServiceStatus(sshStatusHandle, &ssStatus)))
		LogMessage(VLOG_ERR, "SetServiceStatus() failed");

	return(fResult);
}

/*
 * LogMessage - Log a status/error message to the Windows NT system log.
 */

void LogMessage(int pri, char *fmt, ...)
{
	va_list	ap;
	HANDLE	hEventSource;

	va_start(ap, fmt);

	if((hEventSource = RegisterEventSource(NULL, SZSERVICENAME)))
	{
		WORD	winpri;
		char	msg[512];
		LPTSTR	lpszStrings[1];

		vsprintf(msg, fmt, ap);
		lpszStrings[0] = msg;

		switch(pri)
		{
			case VLOG_EMERG:
			case VLOG_ALERT:
			case VLOG_CRIT:
			case VLOG_ERR:		winpri = EVENTLOG_ERROR_TYPE;
						break;
			case VLOG_WARNING:	winpri = EVENTLOG_WARNING_TYPE;
						break;
			case VLOG_NOTICE:
			case VLOG_INFO:
			case VLOG_DEBUG:
			default:		winpri = EVENTLOG_INFORMATION_TYPE;
						break;
		}

		ReportEvent(hEventSource, winpri, 0, 0, NULL,
			1, 0, lpszStrings, NULL);
		DeregisterEventSource(hEventSource);
	}

	va_end(ap);
}

/*
 * VOID ServiceStop(void);
 *
 * Stop the VChat server and all its threads, detach
 * from all attached DLLs and return an exit code of 0.
 */

 VOID ServiceStop(void)
 {
	int tmpsock;

	vlog(VLOG_NOTICE, "Server shutdown signaled");

	/*
	 * Save a reference of the original server socket descriptor
	 * and set it to zero to inform the server main loop of the
	 * server shutdown, then vlose the original socket descriptor
	 * in order to cause the select() call in the main loop to exit
	 * its blocking state.
	 */

	tmpsock = sock;
	sock = 0;
	closesocket(tmpsock);
 }

/*
 * char *winerror(DWORD code);
 *
 * Format a windows error message and return a pointer
 * to a static buffer containing the resulting error
 * message string.
 */

char *winerror(DWORD code)
{
	static	u_char	msgbuf[MAX_PATH]	= "\0";

	FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM,	// System error msg
		NULL,					// Message source
		code,					// Error code
		0,					// Language ID
		msgbuf,					// Message buffer
		MAX_PATH, NULL);			// Buffer len

	return(msgbuf);
}

#endif /* ----------------------------> WIN32 <---------------------------- */
